@@ -7,4 +7,5 @@
 #undef VERSION
 #undef HAVE_IMLIB
 #undef HAVE_GDK_IMLIB
-
+#undef HAVE_SESSION
+#undef HAVE_GDK_PIXBUF
