@@ -24,9 +24,12 @@
 #include <gtk/gtkmain.h>
 #include "mygtkclock.h"
 
-#define SCROLL_DELAY_LENGTH  300
 #define MY_CLOCK_DEFAULT_SIZE 100
 
+#ifndef M_PI
+  #define M_PI 3.141592654
+#endif
+
 /* Forward declarations */
 
 static void my_gtk_clock_class_init               (MyGtkClockClass    *klass);
@@ -64,8 +67,8 @@ my_gtk_clock_get_type ()
 	sizeof (MyGtkClockClass),
 	(GtkClassInitFunc) my_gtk_clock_class_init,
 	(GtkObjectInitFunc) my_gtk_clock_init,
-	(GtkArgSetFunc) NULL,
-	(GtkArgGetFunc) NULL,
+	(gpointer) NULL,
+	(gpointer) NULL,
       };
 
       clock_type = gtk_type_unique (gtk_widget_get_type (), &clock_info);
@@ -116,7 +119,7 @@ my_gtk_clock_realize (GtkWidget *widget)
   attributes.height = widget->allocation.height;
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK;
+  attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;
   attributes.visual = gtk_widget_get_visual (widget);
   attributes.colormap = gtk_widget_get_colormap (widget);
 
