@@ -160,36 +160,6 @@ set_notebook_tab                       (GtkWidget       *notebook,
   gtk_widget_unref (notebook_page);
 }
 
-static GList *pixmaps_directories = NULL;
-
-/* Use this function to set the directory containing installed pixmaps. */
-void
-add_pixmap_directory                   (gchar           *directory)
-{
-  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
-}
-
-/* This is an internally used function to check if a pixmap file exists. */
-#ifndef G_DIR_SEPARATOR_S
-#define G_DIR_SEPARATOR_S "/"
-#endif
-gchar*
-check_file_exists                      (gchar           *directory,
-                                        gchar           *filename)
-{
-  static char full_filename [MAXSTRLEN + 1];
-  struct stat s;
-  gint status;
-
-  snprintf(full_filename, MAXSTRLEN, "%s%s%s", directory, 
-                                               G_DIR_SEPARATOR_S, 
-					       filename);
-  status = stat (full_filename, &s);
-  if (status == 0 && S_ISREG (s.st_mode))
-    return full_filename;
-  return NULL;
-}
-
 void
 lock_size (GtkWidget *toplevel)
 {
@@ -227,44 +197,49 @@ set_icon(GtkWidget *toplevel, gchar *name,  gchar **data)
    Example usage:
     my_util_show_message ("Error saving file");
  */
+
+static GtkWidget *dialog_show_message;
+
+static void 
+on_ok_show_message (GtkWidget *twidget)
+{
+  gtk_widget_destroy (dialog_show_message);
+  gtk_main_quit();
+}
+
+static gboolean
+delete_event_show_message( GtkWidget *widget,
+	      GdkEvent  *event,
+	      gpointer   data )
+{
+  gtk_widget_destroy (dialog_show_message);
+  gtk_main_quit();
+  return (TRUE);
+}
+
 void
 my_show_message (gchar * message)
 {
-  GtkWidget *dialog, *label, *button, *hbox, *pixmapwid;
+  GtkWidget *label, *button, *hbox, *pixmapwid;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   GtkStyle *style;
 
-  void on_ok (GtkWidget *twidget)
-  {
-     gtk_widget_destroy (dialog);
-     gtk_main_quit();
-  }
 
-  gboolean
-  delete_event( GtkWidget *widget,
-              	GdkEvent  *event,
-               	gpointer   data )
-  {
-    gtk_widget_destroy (dialog);
-    gtk_main_quit();
-    return (TRUE);
-  }
+  dialog_show_message = gtk_dialog_new ();
+  gtk_window_position (GTK_WINDOW (dialog_show_message), GTK_WIN_POS_CENTER);
+  gtk_container_border_width (GTK_CONTAINER (dialog_show_message), 5);
 
-  dialog = gtk_dialog_new ();
-  gtk_window_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-  gtk_container_border_width (GTK_CONTAINER (dialog), 5);
-
-  gtk_widget_realize (dialog);
-  gnome_sticky (dialog->window);
+  gtk_widget_realize (dialog_show_message);
+  gnome_sticky (dialog_show_message->window);
 
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (hbox), 5);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_show_message)->vbox), hbox, TRUE, TRUE, 0);
   gtk_widget_show (hbox);
 
-  style = gtk_widget_get_style( dialog );
-  pixmap = gdk_pixmap_create_from_xpm_d( dialog->window,  &mask,
+  style = gtk_widget_get_style( dialog_show_message );
+  pixmap = gdk_pixmap_create_from_xpm_d( dialog_show_message->window,  &mask,
                                            &style->bg[GTK_STATE_NORMAL],
                                            (gchar **)warning );
 
@@ -282,26 +257,26 @@ my_show_message (gchar * message)
 
   button = gtk_button_new_with_label (_("OK"));
   gtk_widget_set_usize (button, 80, -1);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_show_message)->action_area), button,
 		      FALSE, FALSE, 14);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);
 
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (on_ok),
-			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			     GTK_SIGNAL_FUNC (on_ok_show_message),
+			     GTK_OBJECT (dialog_show_message));
 
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
         		     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (dialog));
-  gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-                      GTK_SIGNAL_FUNC (delete_event), NULL);
-  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-                      GTK_SIGNAL_FUNC (on_ok), NULL);
-  gtk_window_set_modal (GTK_WINDOW(dialog),TRUE);
-  gtk_window_set_title (GTK_WINDOW(dialog),_("Message"));
-  gtk_widget_show (dialog);
+			     GTK_OBJECT (dialog_show_message));
+  gtk_signal_connect (GTK_OBJECT (dialog_show_message), "delete_event",
+                      GTK_SIGNAL_FUNC (delete_event_show_message), NULL);
+  gtk_signal_connect (GTK_OBJECT (dialog_show_message), "destroy",
+                      GTK_SIGNAL_FUNC (on_ok_show_message), NULL);
+  gtk_window_set_modal (GTK_WINDOW(dialog_show_message),TRUE);
+  gtk_window_set_title (GTK_WINDOW(dialog_show_message),_("Message"));
+  gtk_widget_show (dialog_show_message);
   gtk_main();
 }
 
@@ -374,7 +349,7 @@ my_show_dialog (  gchar * message,
 
   for (i = 0; i < nbuttons; i++)
     {
-      button = gtk_button_new_with_label (buttons[i]);
+      button = gtk_button_new_with_label (_(buttons[i]));
       gtk_container_add (GTK_CONTAINER (bbox), button);
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       if (i == default_button - 1)
@@ -385,56 +360,59 @@ my_show_dialog (  gchar * message,
 	gtk_signal_connect (GTK_OBJECT (button), "clicked", signal_handlers[i],
 			    data);
 
-      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+      gtk_signal_connect (GTK_OBJECT (button), "clicked",
 				 GTK_SIGNAL_FUNC (gtk_widget_destroy),
 				 GTK_OBJECT (dialog));
     }
   return dialog;
 }
 
+static GtkWidget *dialog_yesno_dialog;
+static int rep_yesno_dialog = 0;
+
+static void 
+on_yes_yesno_dialog (GtkWidget *twidget)
+{
+  rep_yesno_dialog = 1;
+  gtk_widget_destroy (dialog_yesno_dialog);
+  gtk_main_quit();
+}
+
+static void 
+on_no_yesno_dialog (GtkWidget *twidget)
+{
+  rep_yesno_dialog = 0;
+  gtk_widget_destroy (dialog_yesno_dialog);
+  gtk_main_quit();
+}
+
+static gboolean
+delete_event_yesno_dialog( GtkWidget *widget,
+	      GdkEvent  *event,
+	      gpointer   data )
+{
+  gtk_widget_destroy (dialog_yesno_dialog);
+  gtk_main_quit();
+  return (TRUE);
+}
 
 gint
 my_yesno_dialog (gchar * message)
 {
-     int rep = 0;
-     GtkWidget *dialog;
-     gchar *buttons[] = { _("Yes"), _("No") };
-
-     void on_yes (GtkWidget *twidget)
-     {
-        rep = 1;
-        gtk_widget_destroy (dialog);
-        gtk_main_quit();
-     }
-
-     void on_no (GtkWidget *twidget)
-     {
-        rep = 0;
-        gtk_widget_destroy (dialog);
-        gtk_main_quit();
-     }
-
-     gboolean
-     delete_event( GtkWidget *widget,
-                	GdkEvent  *event,
-                	gpointer   data )
-     {
-       gtk_widget_destroy (dialog);
-       gtk_main_quit();
-       return (TRUE);
-     }
-
-     GtkSignalFunc signal_handlers[] = { on_yes, on_no };
-     dialog = my_show_dialog (message, 2, buttons, 2, signal_handlers, NULL);
-     gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-                          GTK_SIGNAL_FUNC (delete_event), NULL);
-     gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-                      GTK_SIGNAL_FUNC (delete_event), NULL);
-     gtk_window_set_modal (GTK_WINDOW(dialog),TRUE);
-     gtk_window_set_title (GTK_WINDOW(dialog),_("Question"));
-     gtk_widget_show (dialog);
-     gtk_main();
-     return rep;
+  /* FIXME : how make this short and ANSI ? */
+  gchar *buttons[] = { _("Yes"), _("No") };
+  
+  GtkSignalFunc signal_handlers[] = { on_yes_yesno_dialog, on_no_yesno_dialog };
+  dialog_yesno_dialog = my_show_dialog (message, 2, buttons, 2, signal_handlers, NULL);
+  gtk_signal_connect (GTK_OBJECT (dialog_yesno_dialog), "delete_event",
+		      GTK_SIGNAL_FUNC (delete_event_yesno_dialog), NULL);
+  gtk_signal_connect (GTK_OBJECT (dialog_yesno_dialog), "destroy",
+                      GTK_SIGNAL_FUNC (delete_event_yesno_dialog), NULL);
+  gtk_window_set_modal (GTK_WINDOW(dialog_yesno_dialog),TRUE);
+  gtk_window_set_title (GTK_WINDOW(dialog_yesno_dialog),_("Question"));
+  gtk_widget_show (dialog_yesno_dialog);
+  gtk_main();
+  return rep_yesno_dialog;
 }
 
 void
