@@ -47,7 +47,7 @@ static GdkAtom atom_read_colors = GDK_NONE;
 static GdkAtom atom_rcfiles = GDK_NONE;
 static gboolean REPAINT_IN_PROGRESS = FALSE;
 
-char *rccolor = "/.xfcolors";
+char *rccolor = "xfcolors";
 char *gtkrcfile = "/.gtkrc";
 
 unsigned long
@@ -115,13 +115,10 @@ XFCE_palette *
 newpal (void)
 {
   XFCE_palette *p;
-  int i;
-  
+
   p = (XFCE_palette *) malloc (sizeof (XFCE_palette));
   p->fnt=(char *) malloc (MAXSTRLEN);
   p->texture=(char *) malloc (MAXSTRLEN);
-  for (i=0; i < NB_XFCE_COLORS; i++)
-    p->cm[i] = gtk_style_copy(gtk_widget_get_default_style());
   strcpy(p->fnt, DEFAULTFONT);
   strcpy(p->texture, "");
   return (p);
@@ -159,7 +156,10 @@ freepal (XFCE_palette * p)
   int i;
 
   for (i = 0; i < NB_XFCE_COLORS; i++)
-    g_free (p->cm[i]);
+    {
+      while (p->cm[i]->attach_count > 0)
+        gtk_style_detach (p->cm[i]);
+    }
   free (p->fnt);
   free (p->texture);
   free (p);
@@ -217,11 +217,7 @@ makepalstyle(XFCE_palette * pal, int i)
 void 
 initpal (XFCE_palette * p)
 {
-  int i;
   if (p) {
-    for (i=0; i < NB_XFCE_COLORS; i++) {
-	p->cm[i] = gtk_style_copy(gtk_widget_get_default_style());
-    }
     if (p->fnt)
       strcpy(p->fnt, DEFAULTFONT);
     if (p->texture)
@@ -386,12 +382,16 @@ GtkStyle *
 applycolor(XFCE_palette * p, short int col_index, int r, int g, int b)
 {
   int howbright;
-  GtkStyle *style;
+  GtkStyle *style = NULL;
 
   p->r[col_index] = r;
   p->g[col_index] = g;
   p->b[col_index] = b;
-  style = gtk_style_copy(p->cm[col_index]);
+  style = gtk_style_new();
+  gdk_font_unref(style->font);
+  if ((style->font = gdk_font_load(p->fnt)) == NULL)
+    if ((style->font = gdk_font_load(DEFAULTFONT)) == NULL)
+      style->font = gdk_font_load("fixed");
   howbright = brightness (p->r[col_index], p->g[col_index], p->b[col_index]);
   if (howbright < fadeblack) {
     style->fg[GTK_STATE_NORMAL].red          = COLOR_GDK;
@@ -566,11 +566,7 @@ applypal (XFCE_palette * p, GtkWidget *toplevel_widget)
    while (gtk_events_pending()) gtk_main_iteration_do(TRUE);
    for (i = 0; i < NB_XFCE_COLORS; i++) {
        style = applycolor(p, i, p->r[i], p->g[i], p->b[i]);
-       gdk_font_unref(style->font);
        gtk_widget_push_style(style);
-       if ((style->font = gdk_font_load(p->fnt)) == NULL)
-         if ((style->font = gdk_font_load(DEFAULTFONT)) == NULL)
-	   style->font = gdk_font_load("fixed");
        if ((toplevel_widget != NULL) && GTK_IS_WIDGET(toplevel_widget))
          {
            widget_reset_styles (toplevel_widget, p->cm[i], style);
@@ -601,30 +597,30 @@ defpal (XFCE_palette * p)
   int i;
 
   initpal (p);
-  p->r[0] = 192;
+  p->r[0] = 255;
   p->g[0] = 0;
   p->b[0] = 0;
-  p->r[1] = 92;
-  p->g[1] = 160;
-  p->b[1] = 195;
-  p->r[2] = 197;
-  p->g[2] = 189;
-  p->b[2] = 174;
-  p->r[3] = 115;
-  p->g[3] = 159;
-  p->b[3] = 189;
-  p->r[4] = 135;
-  p->g[4] = 163;
-  p->b[4] = 165;
-  p->r[5] =  73;
-  p->g[5] = 146;
-  p->b[5] = 167;
+  p->r[1] =   0;
+  p->g[1] = 136;
+  p->b[1] = 168;
+  p->r[2] =  72;
+  p->g[2] = 160;
+  p->b[2] = 175;
+  p->r[3] = 153;
+  p->g[3] = 153;
+  p->b[3] = 153;
+  p->r[4] = 216;
+  p->g[4] = 148;
+  p->b[4] = 148;
+  p->r[5] =   0;
+  p->g[5] = 149;
+  p->b[5] = 200;
   p->r[6] =   0;
-  p->g[6] = 114;
-  p->b[6] = 168;
-  p->r[7] = 213;
-  p->g[7] = 204;
-  p->b[7] = 187;
+  p->g[6] =  91;
+  p->b[6] = 128;
+  p->r[7] = 192;
+  p->g[7] = 192;
+  p->b[7] = 192;
   strcpy(p->fnt, DEFAULTFONT);
   for (i = 0; i < NB_XFCE_COLORS; i++) {
     p->cm[i] = applycolor(p, i, p->r[i], p->g[i], p->b[i]);
@@ -660,11 +656,6 @@ loadnamepal (XFCE_palette * p, const char *name)
   char *lineread, *a;
   FILE *f;
   int i, err = 0;
-  XFCE_palette *backup_pal;
-
-  backup_pal = newpal ();
-  if (!copypal (backup_pal, p))
-    defpal (backup_pal);
 
   lineread = (char *) malloc ((80) * sizeof (char));
 
@@ -712,9 +703,6 @@ loadnamepal (XFCE_palette * p, const char *name)
       fclose (f);
     }
   free (lineread);
-  if (err)
-    copypal (p, backup_pal);
-  freepal (backup_pal);
   return ((!err && (f != NULL)));
 }
 
@@ -726,6 +714,7 @@ savepal (XFCE_palette * p)
 
   homedir = (char *) malloc ((MAXSTRLEN + 1) * sizeof (char));
   strcpy (homedir, (char *) getenv ("HOME"));
+  strcat (homedir, "/.");
   strcat (homedir, rccolor);
   x = savenamepal (p, homedir);
   free (homedir);
@@ -740,8 +729,16 @@ loadpal (XFCE_palette * p)
 
   homedir = (char *) malloc ((MAXSTRLEN + 1) * sizeof (char));
   strcpy (homedir, (char *) getenv ("HOME"));
+  strcat (homedir, "/.");
   strcat (homedir, rccolor);
   x = loadnamepal (p, homedir);
+  if (!x)
+    {
+      strcpy (homedir, XFCE_CONFDIR);
+      strcat (homedir, "/");
+      strcat (homedir, rccolor);
+      x = loadnamepal (p, homedir);
+    }
   free (homedir);
   return (x);
 }
