@@ -36,6 +36,10 @@
 
 #ifdef HAVE_GDK_IMLIB
   #include <gdk_imlib.h>
+#else
+#ifdef HAVE_GDK_PIXBUF
+  #include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
 #endif
 
 #include "my_intl.h"
@@ -59,19 +63,22 @@ Pixmap
 duppix(Pixmap pixmap, int width, int height)
 {
     Display *tmpDpy;
-    Pixmap copyP;
+    Pixmap copyP = 0;
 
-    tmpDpy = XOpenDisplay("");
-    if ((!tmpDpy) || (!width) || (!height)) 
+    if ((!width) || (!height))
+      return 0;
+    
+    if (!(tmpDpy = XOpenDisplay(""))) 
 	return 0;
     else 
       {
 	XSync(GDK_DISPLAY(), False);
 
 	copyP = XCreatePixmap(tmpDpy, GDK_WINDOW_XWINDOW(GDK_ROOT_PARENT()), width, height,
-			       DefaultDepth(tmpDpy, DefaultScreen(GDK_DISPLAY())));
-	XCopyArea(tmpDpy, pixmap, copyP, DefaultGC(tmpDpy, DefaultScreen(GDK_DISPLAY())),
+			       DefaultDepth(tmpDpy, DefaultScreen(tmpDpy)));
+	XCopyArea(tmpDpy, pixmap, copyP, DefaultGC(tmpDpy, DefaultScreen(tmpDpy)),
 		  0, 0, width, height, 0, 0);
+        
 	XSync(tmpDpy, False);
 
 	XSetCloseDownMode(tmpDpy, RetainPermanent);
@@ -120,7 +127,7 @@ setPixmapProperty(Pixmap pixmap)
 void GradientXpmImage(int sizex,int sizey,XpmImage *Imagedest, 
                    int r1, int g1, int b1, int r2, int g2, int b2, int ncolors)
 {
-  register int i, j, v;
+  int i, j, v;
   
   if (ncolors <= 0) 
        ncolors = 2;
@@ -155,6 +162,8 @@ void GradientXpmImage(int sizex,int sizey,XpmImage *Imagedest,
 
 
 #ifndef HAVE_GDK_IMLIB
+#ifndef HAVE_GDK_PIXBUF
+
 /* The following function is from DFM (c) 1997 by Achim Kaiser */
 
 void XpmShapeToFit_dither(int newsizex,int newsizey,XpmImage *I,XpmImage *Imagedest)
@@ -260,7 +269,7 @@ void XpmShapeToFit_dither(int newsizex,int newsizey,XpmImage *I,XpmImage *Imaged
 
 void XpmShapeToFit_fast(int newsizex,int newsizey,XpmImage *I,XpmImage *Imagedest)
 {
-  register int k,x,y;
+  int k,x,y;
   double step_x, step_y;
 
   Imagedest->width      = MIN (newsizex, I->width);
@@ -291,7 +300,7 @@ void XpmShapeToFit_fast(int newsizex,int newsizey,XpmImage *I,XpmImage *Imagedes
     }
 }
 
-inline void XpmShapeToFit(int newsizex,int newsizey,XpmImage *I,XpmImage *Imagedest)
+void XpmShapeToFit(int newsizex,int newsizey,XpmImage *I,XpmImage *Imagedest)
 {
   if (DEFAULT_DEPTH > 8)
     XpmShapeToFit_fast(newsizex, newsizey, I, Imagedest);
@@ -312,6 +321,7 @@ MyCreateDataFromXpmImage(XpmImage *Img, char ***data_return)
   return (status); 
 }
 
+#endif /* HAVE_GDK_PIXBUF */
 #endif /* HAVE_GDK_IMLIB */
 
 int 
@@ -342,12 +352,16 @@ MyCreateGdkPixmapFromFile(char *filename, GtkWidget *widget, GdkBitmap **mask)
 {
 #ifdef HAVE_GDK_IMLIB
   GdkImlibImage         *im;
+#else
+#ifdef HAVE_GDK_PIXBUF
+  GdkPixbuf *source, *destination;
 #else
   int result = -1;
   XpmImage I;
   XpmImage I2;
   GdkColormap *colormap;
   char **pix_data;
+#endif /* HAVE_GDK_PIXBUF */
 #endif /* HAVE_GDK_IMLIB */
   GdkPixmap *pixmap = NULL;
   GtkRequisition requisition;
@@ -370,11 +384,36 @@ MyCreateGdkPixmapFromFile(char *filename, GtkWidget *widget, GdkBitmap **mask)
         gdk_imlib_render(im,w,h);
       else
         gdk_imlib_render(im,im->rgb_width,im->rgb_height);
-      pixmap = gdk_imlib_move_image(im);
-      *mask   = gdk_imlib_move_mask(im);
+      pixmap  = gdk_imlib_copy_image(im);
+      *mask   = gdk_imlib_copy_mask(im);
       gdk_imlib_kill_image(im);
     }
 #else /* without HAVE_GDK_IMLIB */
+#ifdef HAVE_GDK_PIXBUF
+  source = gdk_pixbuf_new_from_file (filename);
+  if (!source)
+    {
+      fprintf (stderr, _("*** WARNING ***: Cannot load image file %s\n"), filename);
+      source = gdk_pixbuf_new_from_xpm_data(empty);
+    }
+  if (source)
+    {
+      gtk_widget_size_request ((GtkWidget *) widget, &requisition);
+      w = requisition.width  - (GTK_CONTAINER (widget)->border_width + 2) * 2;
+      h = requisition.height - (GTK_CONTAINER (widget)->border_width + 2) * 2;
+      if ((w > 0) && (h > 0) && 
+          ((gdk_pixbuf_get_width (source) > w) || gdk_pixbuf_get_height (source) > h))
+        destination = gdk_pixbuf_scale_simple (source, w, h, GDK_INTERP_NEAREST);
+      else
+        destination = gdk_pixbuf_scale_simple (source, gdk_pixbuf_get_width (source), 
+                                         gdk_pixbuf_get_height (source),
+                                         GDK_INTERP_NEAREST);
+      gdk_pixbuf_render_pixmap_and_mask (destination, &pixmap, mask, 
+                                         gdk_pixbuf_get_has_alpha (destination));
+      gdk_pixbuf_unref (source);
+      gdk_pixbuf_unref (destination);
+    }
+#else
   if (existfile (filename))
     result = XpmReadFileToXpmImage(filename, &I, NULL);
   if (result)
@@ -403,6 +442,7 @@ MyCreateGdkPixmapFromFile(char *filename, GtkWidget *widget, GdkBitmap **mask)
     XpmFree(pix_data);
     XpmFreeXpmImage(&I);
   }
+#endif /* HAVE_GDK_PIXBUF */
 #endif /* HAVE_GDK_IMLIB */
   return (pixmap);
 }
@@ -412,12 +452,16 @@ MyCreateGdkPixmapFromData(char **data, GtkWidget *widget, GdkBitmap **mask)
 {
 #ifdef HAVE_GDK_IMLIB
   GdkImlibImage         *im;
+#else
+#ifdef HAVE_GDK_PIXBUF
+  GdkPixbuf *source, *destination;
 #else
   int result;
   XpmImage I;
   XpmImage I2;
   GdkColormap *colormap;
   char **pix_data;
+#endif /* HAVE_GDK_PIXBUF */
 #endif /* HAVE_GDK_IMLIB */
   GdkPixmap *pixmap = NULL;
   GtkRequisition requisition;
@@ -437,11 +481,33 @@ MyCreateGdkPixmapFromData(char **data, GtkWidget *widget, GdkBitmap **mask)
         gdk_imlib_render(im,w,h);
       else
         gdk_imlib_render(im,im->rgb_width,im->rgb_height);
-      pixmap  = gdk_imlib_move_image(im);
-      *mask   = gdk_imlib_move_mask(im);
+      pixmap  = gdk_imlib_copy_image(im);
+      *mask   = gdk_imlib_copy_mask(im);
       gdk_imlib_kill_image(im);
     }
 #else /* without HAVE_GDK_IMLIB */
+#ifdef HAVE_GDK_PIXBUF
+  source = gdk_pixbuf_new_from_xpm_data (data);
+  if (!source)
+      source = gdk_pixbuf_new_from_xpm_data(empty);
+  if (source)
+    {
+      gtk_widget_size_request ((GtkWidget *) widget, &requisition);
+      w = requisition.width  - (GTK_CONTAINER (widget)->border_width + 2) * 2;
+      h = requisition.height - (GTK_CONTAINER (widget)->border_width + 2) * 2;
+      if ((w > 0) && (h > 0) && 
+          ((gdk_pixbuf_get_width (source) > w) || (gdk_pixbuf_get_height (source) > h)))
+        destination = gdk_pixbuf_scale_simple (source, w, h, GDK_INTERP_NEAREST);
+      else
+        destination = gdk_pixbuf_scale_simple (source, gdk_pixbuf_get_width (source), 
+                                         gdk_pixbuf_get_height (source),
+                                         GDK_INTERP_NEAREST);
+      gdk_pixbuf_render_pixmap_and_mask (destination, &pixmap, mask, 
+                                         gdk_pixbuf_get_has_alpha (destination));
+      gdk_pixbuf_unref (source);
+      gdk_pixbuf_unref (destination);
+    }
+#else
   result = XpmCreateXpmImageFromData (data, &I, NULL);
   if (result == 0)
   {
@@ -464,6 +530,7 @@ MyCreateGdkPixmapFromData(char **data, GtkWidget *widget, GdkBitmap **mask)
     XpmFree(pix_data);
     XpmFreeXpmImage(&I);
   }
+#endif /* HAVE_GDK_PIXBUF */
 #endif /* HAVE_GDK_IMLIB */
   return (pixmap);
 }
@@ -476,11 +543,10 @@ MySetPixmapData(GtkWidget * widget, GtkWidget * parent, char **data)
 
   pixmap = MyCreateGdkPixmapFromData(data, parent, &mask);
   gtk_pixmap_set (GTK_PIXMAP(widget), pixmap, mask);
-#ifdef HAVE_GDK_IMLIB
-  gdk_imlib_free_pixmap(pixmap);
-#else  
-  gdk_pixmap_unref(pixmap);
-#endif
+  /*
+  if (pixmap) gdk_pixmap_unref (pixmap);
+  if (mask)   gdk_bitmap_unref (mask);
+   */
 }
 
 void
@@ -491,11 +557,10 @@ MySetPixmapFile(GtkWidget * widget, GtkWidget * parent, char *filename)
 
   pixmap = MyCreateGdkPixmapFromFile(filename, parent, &mask);
   gtk_pixmap_set (GTK_PIXMAP(widget), pixmap, mask);
-#ifdef HAVE_GDK_IMLIB
-  gdk_imlib_free_pixmap(pixmap);
-#else  
-  gdk_pixmap_unref(pixmap);
-#endif
+  /*
+  if (pixmap) gdk_pixmap_unref (pixmap);
+  if (mask)   gdk_bitmap_unref (mask);
+   */
 }
 
 GtkWidget * 
@@ -507,7 +572,10 @@ MyCreateFromPixmapData(GtkWidget * widget, char **data)
 
   pixmap = MyCreateGdkPixmapFromData(data, widget, &mask);
   result = gtk_pixmap_new (pixmap, mask);
-
+  /*
+  if (pixmap) gdk_pixmap_unref (pixmap);
+  if (mask)   gdk_bitmap_unref (mask);
+   */
   return result;
 }
 
@@ -520,8 +588,10 @@ MyCreateFromPixmapFile(GtkWidget * widget, char *file)
 
   pixmap = MyCreateGdkPixmapFromFile(file, widget, &mask);
   result = gtk_pixmap_new (pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-
+  /*
+  if (pixmap) gdk_pixmap_unref (pixmap);
+  if (mask)   gdk_bitmap_unref (mask);
+   */
   return result;
 }
 
