@@ -22,7 +22,7 @@
 #include <gtk/gtk.h>
 #include "xfcolor.h"
 
-#define RCFILE          ".xfbdrc"
+#define RCFILE          "xfbdrc"
 #define NOBACK          "None"
 #define BACKTYPE        "*.xpm"
 
@@ -34,7 +34,6 @@
   char         *backdrp;
 
 int setroot (char *str);
-void defroot (char *str);
 void readstr (char *str);
 int writestr (char *str);
 void display_back (char *str);
