@@ -34,7 +34,7 @@
 #include "fileutil.h"
 #include "my_intl.h"
 
-char *rcfile = ".xfce3rc";
+char *rcfile = "xfce3rc";
 int nl = 0;
 int buffersize = 127;
 
@@ -97,9 +97,9 @@ initconfig(config *newconf)
    newconf->startup_flags = (F_SOUNDMODULE | F_MOUSEMODULE  |
    			     F_BACKDROPMODULE | F_PAGERMODULE | F_GNOMEMODULE);
    newconf->iconpos = 0; /* Top of screen */
-   newconf->fonts[0] = (char *) malloc (sizeof(char) * MAXSTRLEN);
-   newconf->fonts[1] = (char *) malloc (sizeof(char) * MAXSTRLEN);
-   newconf->fonts[2] = (char *) malloc (sizeof(char) * MAXSTRLEN);
+   newconf->fonts[0] = (char *) malloc (sizeof(char) * (strlen(XFWM_TITLEFONT) + 1));
+   newconf->fonts[1] = (char *) malloc (sizeof(char) * (strlen(XFWM_MENUFONT) + 1));
+   newconf->fonts[2] = (char *) malloc (sizeof(char) * (strlen(XFWM_ICONFONT) + 1));
    strcpy(newconf->fonts[0], XFWM_TITLEFONT);
    strcpy(newconf->fonts[1], XFWM_MENUFONT);
    strcpy(newconf->fonts[2], XFWM_ICONFONT);
@@ -120,7 +120,7 @@ backupconfig (char *extension)
   int nb_read;
 
   strcpy (homedir, (char *) getenv ("HOME"));
-  strcat (homedir, "/");
+  strcat (homedir, "/.");
   strcat (homedir, rcfile);
   /*
      Backup any existing config file before creating a new one 
@@ -153,7 +153,7 @@ writeconfig (void)
   int x, y;
 
   strcpy (homedir, (char *) getenv ("HOME"));
-  strcat (homedir, "/");
+  strcat (homedir, "/.");
   strcat (homedir, rcfile);
   /*
      Backup any existing config file before creating a new one 
@@ -281,7 +281,7 @@ resetconfig (void)
   int i;
 
   strcpy (homedir, (char *) getenv ("HOME"));
-  strcat (homedir, "/");
+  strcat (homedir, "/.");
   strcat (homedir, rcfile);
   configfile = fopen (homedir, "w");
   if (!configfile)
@@ -410,6 +410,59 @@ localize_rcfilename (char * rcfile)
     }
 }
 
+char * get_first_ok(char *p)
+{
+  char *tok;
+  static char ret  [MAXSTRLEN + 1];
+  int nb;
+  if (p == NULL) return(p);
+  if (p == "") return(p);
+  tok = strtok (p,",");
+  while(tok!=NULL)
+    {
+      tok = skiphead(tok);
+      if(strstr(tok,"Module"))
+	{
+	  tok+=7;
+	  nb=(int) (strcspn (tok," "));
+	  strncpy (ret, tok, nb);
+	  ret[nb]='\0';
+	  if(existfile (ret))
+	    {
+	      tok-=7;
+	      strcpy(ret,tok);
+	      skiptail(ret);
+	      return(ret);
+	    }
+	}
+      if(strstr(tok,"Term"))
+	{
+	  tok+=5;
+	  nb=(int) (strcspn (tok," "));
+	  strncpy (ret, tok, nb);
+	  ret[nb]='\0';
+	  if(existfile (ret))
+	    {
+	      tok-=5;
+	      strcpy(ret,tok);
+	      skiptail(ret);
+	      return(ret);
+	    }
+	} 
+      nb=(int) (strcspn (tok," "));
+      strncpy (ret, tok, nb);
+      ret[nb]='\0';
+      if(existfile (ret))
+	{
+	  strcpy(ret,tok);
+	  skiptail(ret);
+	  return(ret);
+	}
+      tok = strtok (NULL,",");
+    }
+  return("None");
+}
+
 void 
 readconfig (void)
 {
@@ -420,14 +473,14 @@ readconfig (void)
   char label    [256];
   char dummy    [16];
   char *p;
-  FILE *configfile;
+  FILE *configfile = NULL;
 
 
   int i, j;
-
+  int newconf=0;
   nl = 0;
   strcpy (homedir, (char *) getenv ("HOME"));
-  strcat (homedir, "/");
+  strcat (homedir, "/.");
   strcat (homedir, rcfile);
   if (existfile (homedir))
     {
@@ -436,7 +489,7 @@ readconfig (void)
   else
     {
       fprintf (stderr, _("XFce : %s File not found.\n"), homedir);
-      strcpy (homedir, XFCE_DIR);
+      strcpy (homedir, XFCE_CONFDIR);
       strcat (homedir, "/");
       strcat (homedir, rcfile);
       if (existfile (homedir))
@@ -445,10 +498,8 @@ readconfig (void)
 	}
       else
 	{
+	  newconf=1;
 	  fprintf (stderr, _("XFce : %s File not found.\n"), homedir);
-	  strcpy (homedir, XFCE_DIR);
-	  strcat (homedir, "/sample");
-	  strcat (homedir, rcfile);
 	  localize_rcfilename (homedir);
 	  configfile = fopen (homedir, "r");
 	}
@@ -459,7 +510,7 @@ readconfig (void)
       fprintf (stderr, _("XFce : %s File not found.\n"), homedir);
       resetconfig ();
       strcpy (homedir, (char *) getenv ("HOME"));
-      strcat (homedir, "/");
+      strcat (homedir, "/.");
       strcat (homedir, rcfile);
       configfile = fopen (homedir, "r");
     }
@@ -476,7 +527,7 @@ readconfig (void)
 	  fclose (configfile);
 	  resetconfig ();
 	  strcpy (homedir, (char *) getenv ("HOME"));
-	  strcat (homedir, "/");
+	  strcat (homedir, "/.");
 	  strcat (homedir, rcfile);
 	  configfile = fopen (homedir, "r");
           if (!configfile) {
@@ -532,12 +583,10 @@ readconfig (void)
 	{
 	  p = nextline (configfile, lineread);
 	  load_icon_str (p);
-	  setup_icon ();
 	  p = nextline (configfile, lineread);
 	}
       else {
 	default_icon_str ();
-        setup_icon ();
       }
       if (!my_strncasecmp (p, "[WorkSpace]", strlen ("[WorkSpace]")))
 	{
@@ -591,10 +640,16 @@ readconfig (void)
 	  else
 	    current_config.iconpos = 0;
 	  p = nextline (configfile, lineread);
+          if (current_config.fonts[0]) free (current_config.fonts[0]);
+          current_config.fonts[0] = (char *) malloc (sizeof (char) * (strlen (p) + 1));
 	  strcpy(current_config.fonts[0], p);
 	  p = nextline (configfile, lineread);
+          if (current_config.fonts[1]) free (current_config.fonts[1]);
+          current_config.fonts[1] = (char *) malloc (sizeof (char) * (strlen (p) + 1));
 	  strcpy(current_config.fonts[1], p);
 	  p = nextline (configfile, lineread);
+          if (current_config.fonts[2]) free (current_config.fonts[2]);
+          current_config.fonts[2] = (char *) malloc (sizeof (char) * (strlen (p) + 1));
 	  strcpy(current_config.fonts[2], p);
 	  p = nextline (configfile, lineread);
 	}
@@ -627,6 +682,7 @@ readconfig (void)
 	    current_config.select_icon_size = 2;
 	  else
 	    current_config.select_icon_size = 1;
+          update_gxfce_size();
   	  p = nextline (configfile, lineread);
           if (!my_strncasecmp (p, "SmallMenuIcons", strlen ("SmallMenuIcons")))
 	    current_config.popup_icon_size = 0;
@@ -634,6 +690,7 @@ readconfig (void)
 	    current_config.popup_icon_size = 2;
 	  else
 	    current_config.popup_icon_size = 1;
+          update_popup_size();
   	  p = nextline (configfile, lineread);
 	 }
       if (!my_strncasecmp (p, "[Commands]", strlen ("[Commands]")))
@@ -641,7 +698,10 @@ readconfig (void)
           p = nextline (configfile, lineread);
           for (i = 0; i < NBSELECTS; i++)
     	    {
-  	      set_command (i, p);
+  	      if (newconf==1)
+		set_command (i, get_first_ok(p));
+	      else
+		set_command (i, p);
               p = nextline (configfile, lineread);
 	    }
 	}
@@ -656,11 +716,16 @@ readconfig (void)
 	    {
 	      strcpy (label, (p ? p :"None"));
 	      p = nextline (configfile, lineread);
-	      strcpy (pixfile, (p ? p :"Default"));
+	      strcpy (pixfile, (p ? p :"Default icon"));
 	      p = nextline (configfile, lineread);
-	      strcpy (command, (p ? p :"None"));
-	      if (j++ < NBMAXITEMS)
-	        add_popup_entry (i - 1, label, pixfile, command);
+	      if(newconf==1)
+		p = get_first_ok(p);
+	      if(strcmp(p,"None"))
+		{
+		  strcpy (command, (p ? p :"None"));
+		  if (j++ < NBMAXITEMS)
+		    add_popup_entry (i - 1, label, pixfile, command);
+		}
 	      p = nextline (configfile, lineread);
 	    }
 	}
