@@ -29,6 +29,7 @@
 #include "my_intl.h"
 #include "xfce_main.h"
 #include "xfce-common.h"
+#include "fileutil.h"
 #include "modify.h"
 #include "popup.h"
 #include "xpmext.h"
@@ -66,7 +67,6 @@ create_modify (XFCE_palette * pal)
   gtk_window_set_title (GTK_WINDOW (modify), _("Modify Item ..."));
   gtk_window_position (GTK_WINDOW (modify), GTK_WIN_POS_CENTER);
   gtk_widget_realize(modify);
-  gnome_sticky(modify->window);
 
   accel_group = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (modify), accel_group);
@@ -325,13 +325,13 @@ open_modify(GtkWidget *modify, int menu, int item)
     gtk_entry_set_text(GTK_ENTRY(modify_command_entry), get_popup_entry_command(menu, item));
     gtk_entry_set_text(GTK_ENTRY(modify_displayed_entry), get_popup_entry_label(menu, item));
     pixfile = get_popup_entry_icon(menu, item);
-    if (check_file_exists ("/", pixfile)) {
+    if (existfile (pixfile)) {
       gtk_entry_set_text(GTK_ENTRY(modify_icon_entry), pixfile);
       pixmap = MyCreateGdkPixmapFromFile(pixfile, modify_preview_frame, &mask);
       gtk_pixmap_set (GTK_PIXMAP(modify_preview_pixmap), pixmap, mask);
     }
     else {
-      gtk_entry_set_text(GTK_ENTRY(modify_icon_entry), _("Default"));
+      gtk_entry_set_text(GTK_ENTRY(modify_icon_entry), "Default icon");
       pixmap = MyCreateGdkPixmapFromData(defaulticon, modify_preview_frame, &mask);
       gtk_pixmap_set (GTK_PIXMAP(modify_preview_pixmap), pixmap, mask);
     }
@@ -348,6 +348,7 @@ open_modify(GtkWidget *modify, int menu, int item)
   gtk_entry_set_position(GTK_ENTRY(modify_icon_entry)	  , 0);
   gtk_entry_set_position(GTK_ENTRY(modify_displayed_entry), 0);
   gtk_window_set_modal (GTK_WINDOW(modify),TRUE);
+  gnome_sticky(modify->window);
   gtk_widget_show (modify);
   gtk_main ();
 }
