@@ -27,6 +27,8 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <sys/time.h>
+#include <unistd.h>
 #include "my_intl.h"
 #include "constant.h"
 #include "xfcolor.h"
@@ -38,11 +40,13 @@
 #include "xpmext.h"
 #include "popup_cb.h"
 #include "configfile.h"
+#include "fileutil.h"
 #include "my_tooltips.h"
 #include "gnome_protocol.h"
 
 #include "menu_icon.h"
 #include "addiconpix.h"
+#include "empty.h"
 #include "defaulticon.h"
 
 
@@ -68,13 +72,9 @@ create_popup_item (GtkWidget *toplevel, XFCE_palette * pal, gint nbr_menu, gint
 {
   GtkWidget *hbox;
   
-  popup_menus[nbr_menu].popup_buttons[nbr_item].label   = (char *) malloc (sizeof (char) * MAXSTRLEN + 1);
-  popup_menus[nbr_menu].popup_buttons[nbr_item].pixfile = (char *) malloc (sizeof (char) * MAXSTRLEN + 1);
-  popup_menus[nbr_menu].popup_buttons[nbr_item].command = (char *) malloc (sizeof (char) * MAXSTRLEN + 1);
-
-  strcpy (popup_menus[nbr_menu].popup_buttons[nbr_item].label,   "Empty");
-  strcpy (popup_menus[nbr_menu].popup_buttons[nbr_item].pixfile, "Default");
-  strcpy (popup_menus[nbr_menu].popup_buttons[nbr_item].command, "");
+  popup_menus[nbr_menu].popup_buttons[nbr_item].label   = NULL;
+  popup_menus[nbr_menu].popup_buttons[nbr_item].pixfile = NULL;
+  popup_menus[nbr_menu].popup_buttons[nbr_item].command = NULL;
 
   popup_menus[nbr_menu].popup_buttons[nbr_item].item_button = gtk_button_new ();
   gtk_widget_set_name (popup_menus[nbr_menu].popup_buttons[nbr_item].item_button, "popup_button");
@@ -106,7 +106,7 @@ create_popup_item (GtkWidget *toplevel, XFCE_palette * pal, gint nbr_menu, gint
   gtk_widget_set_usize (popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap_frame, POPUP_ICON_SIZE, POPUP_ICON_SIZE);
   gtk_frame_set_shadow_type (GTK_FRAME (popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap_frame), GTK_SHADOW_IN);
 
-  popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap = MyCreateFromPixmapData (popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap_frame, defaulticon);
+  popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap = MyCreateFromPixmapData (popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap_frame, empty);
   if (popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap == NULL)
     g_error (_("Couldn't create pixmap"));
   gtk_widget_set_name (popup_menus[nbr_menu].popup_buttons[nbr_item].item_pixmap, "popup_pixmap");
@@ -121,7 +121,7 @@ create_popup_item (GtkWidget *toplevel, XFCE_palette * pal, gint nbr_menu, gint
   /* gtk_widget_set_style(popup_menus[nbr_menu].popup_buttons[nbr_item].item_label, pal->cm[7]); */
   gtk_widget_show (popup_menus[nbr_menu].popup_buttons[nbr_item].item_label);
   gtk_box_pack_start (GTK_BOX (hbox), popup_menus[nbr_menu].popup_buttons[nbr_item].item_label, TRUE, TRUE, 0);
-  gtk_widget_set_usize (popup_menus[nbr_menu].popup_buttons[nbr_item].item_label, 120, 0);
+  gtk_widget_set_usize (popup_menus[nbr_menu].popup_buttons[nbr_item].item_label, 130, 0);
   gtk_label_set_justify (GTK_LABEL (popup_menus[nbr_menu].popup_buttons[nbr_item].item_label), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (popup_menus[nbr_menu].popup_buttons[nbr_item].item_label), 0.1, 0.5);
 
@@ -150,7 +150,6 @@ create_popup_menu (XFCE_palette *pal, gint nbr)
 {
   GtkWidget *popup_mainframe;
   GtkWidget *popup_vbox;
-  GtkWidget *popup_addicon_button;
   GtkTooltips *popup_addicon_tooltip;
   GtkWidget *popup_addicon_hbox;
   GtkWidget *popup_addicon_icon_pixmap;
@@ -165,11 +164,10 @@ create_popup_menu (XFCE_palette *pal, gint nbr)
   popup_menus[nbr].uposy = 0; 
   popup_menus[nbr].lastpx = 0;
   popup_menus[nbr].lastpy = 0;
+  popup_menus[nbr].animatewin = NULL;
 
   popup_menus[nbr].popup_toplevel = gtk_window_new (GTK_WINDOW_DIALOG);
   gtk_window_set_policy (GTK_WINDOW (popup_menus[nbr].popup_toplevel), FALSE, FALSE, TRUE);
-  gtk_widget_realize (popup_menus[nbr].popup_toplevel);
-  gnome_sticky(popup_menus[nbr].popup_toplevel->window);
   gtk_widget_set_name (popup_menus[nbr].popup_toplevel, "popup");
   gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "popup", popup_menus[nbr].popup_toplevel);
   gtk_window_set_title (GTK_WINDOW (popup_menus[nbr].popup_toplevel), "Popup Menu");
@@ -191,31 +189,23 @@ create_popup_menu (XFCE_palette *pal, gint nbr)
   gtk_widget_show (popup_vbox);
   gtk_container_add (GTK_CONTAINER (popup_mainframe), popup_vbox);
 
-  popup_menus[nbr].popup_addicon_frame = gtk_frame_new (NULL);
-  gtk_widget_set_name (popup_menus[nbr].popup_addicon_frame, "popup_addicon_frame");
-  gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "popup_menus[nbr].popup_addicon_frame", popup_menus[nbr].popup_addicon_frame);
-  /* gtk_widget_set_style(popup_menus[nbr].popup_addicon_frame, pal->cm[7]); */
-  gtk_widget_show (popup_menus[nbr].popup_addicon_frame);
-  gtk_box_pack_start (GTK_BOX (popup_vbox), popup_menus[nbr].popup_addicon_frame, FALSE, TRUE, 0);
-  gtk_widget_set_usize (popup_menus[nbr].popup_addicon_frame, 0, 0);
-
-  popup_addicon_button = gtk_button_new ();
-  gtk_widget_set_name (popup_addicon_button, "popup_addicon_button");
-  gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "popup_addicon_button", popup_addicon_button);
-  gtk_button_set_relief((GtkButton *) popup_addicon_button, GTK_RELIEF_NONE);
-  /* gtk_widget_set_style(popup_addicon_button, pal->cm[7]); */
-  gtk_widget_show (popup_addicon_button);
-  gtk_container_add (GTK_CONTAINER (popup_menus[nbr].popup_addicon_frame), popup_addicon_button);
+  popup_menus[nbr].popup_addicon_button = gtk_button_new ();
+  gtk_widget_set_name (popup_menus[nbr].popup_addicon_button, "popup_addicon_button");
+  gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "popup_addicon_button", popup_menus[nbr].popup_addicon_button);
+  gtk_button_set_relief((GtkButton *) popup_menus[nbr].popup_addicon_button, GTK_RELIEF_NONE);
+  /* gtk_widget_set_style(popup_menus[nbr].popup_addicon_button, pal->cm[7]); */
+  gtk_widget_show (popup_menus[nbr].popup_addicon_button);
+  gtk_box_pack_start (GTK_BOX (popup_vbox), popup_menus[nbr].popup_addicon_button, FALSE, TRUE, 0);
 
   popup_addicon_tooltip = my_tooltips_new(current_config.tooltipsdelay);
-  gtk_tooltips_set_tip (popup_addicon_tooltip, popup_addicon_button, _("Add a new entry..."), "ContextHelp/buttons/?");
+  gtk_tooltips_set_tip (popup_addicon_tooltip, popup_menus[nbr].popup_addicon_button, _("Add a new entry..."), "ContextHelp/buttons/?");
   gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "tooltips", popup_addicon_tooltip);
 
   popup_addicon_hbox = gtk_hbox_new (FALSE, 0);
   gtk_widget_set_name (popup_addicon_hbox, "popup_addicon_hbox");
   gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "popup_addicon_hbox", popup_addicon_hbox);
   gtk_widget_show (popup_addicon_hbox);
-  gtk_container_add (GTK_CONTAINER (popup_addicon_button), popup_addicon_hbox);
+  gtk_container_add (GTK_CONTAINER (popup_menus[nbr].popup_addicon_button), popup_addicon_hbox);
 
   popup_menus[nbr].popup_addicon_icon_frame = gtk_frame_new (NULL);
   gtk_widget_set_name (popup_menus[nbr].popup_addicon_icon_frame, "popup_addicon_icon_frame");
@@ -245,6 +235,13 @@ create_popup_menu (XFCE_palette *pal, gint nbr)
   gtk_label_set_justify (GTK_LABEL (popup_addicon_icon_label), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (popup_addicon_icon_label), 0.1, 0.5);
 
+  popup_menus[nbr].popup_addicon_separator = gtk_hseparator_new ();
+  gtk_widget_set_name (popup_menus[nbr].popup_addicon_separator, "popup_addicon_separator");
+  gtk_object_set_data (GTK_OBJECT (popup_menus[nbr].popup_toplevel), "popup_addicon_separator", popup_menus[nbr].popup_addicon_separator);
+  /* gtk_widget_set_style(popup_menus[nbr].popup_addicon_separator, pal->cm[7]); */
+  gtk_widget_show (popup_menus[nbr].popup_addicon_separator);
+  gtk_box_pack_start (GTK_BOX (popup_vbox), popup_menus[nbr].popup_addicon_separator, FALSE, TRUE, 0);
+
   for (i=0; i < NBMAXITEMS; i++)
     gtk_box_pack_start (GTK_BOX (popup_vbox), create_popup_item (popup_menus[nbr].popup_toplevel, pal, nbr, i), FALSE, TRUE, 0);
 
@@ -274,16 +271,16 @@ create_popup_menu (XFCE_palette *pal, gint nbr)
                              GTK_SIGNAL_FUNC (detach_cb),
                              (gpointer) nbr);
 
-  gtk_signal_connect (GTK_OBJECT (popup_addicon_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (popup_menus[nbr].popup_addicon_button), "clicked",
 			   GTK_SIGNAL_FUNC (popup_addicon_cb),
 			   (gpointer) nbr);
 
-  gtk_drag_dest_set (popup_addicon_button,
+  gtk_drag_dest_set (popup_menus[nbr].popup_addicon_button,
 		     GTK_DEST_DEFAULT_ALL,
 		     popup_target_table, n_popup_targets,
 		     GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
-  gtk_signal_connect (GTK_OBJECT (popup_addicon_button), "drag_data_received",
+  gtk_signal_connect (GTK_OBJECT (popup_menus[nbr].popup_addicon_button), "drag_data_received",
 		      GTK_SIGNAL_FUNC (popup_addicon_drag_data_received), (gpointer) nbr);
 
   set_icon (popup_menus[nbr].popup_toplevel, "Popup Menu", menu_icon);
@@ -306,33 +303,40 @@ create_all_popup_menus (XFCE_palette *pal, GtkWidget *main)
 }
 
 void
-show_popup_menu (gint nbr, gint x, gint y)
+show_popup_menu (gint nbr, gint x, gint y, gboolean animation)
 {
   gint uwidth  = 0;
   gint uheight = 0;
   gint px      = 0;
   gint py      = 0;
   gint tempx, tempy;
+  gboolean anim;
   gint i;
+  gdouble speed = 0.0;
   GtkRequisition requisition;
+  struct timeval new_tv, old_tv;
+  struct timezone tz;
   
-  if (!GTK_WIDGET_REALIZED(popup_menus[nbr].popup_toplevel))
-    {
-      gtk_widget_realize (popup_menus[nbr].popup_toplevel);
-      gnome_sticky(popup_menus[nbr].popup_toplevel->window);
-    }
+  anim = ((!popup_menus[nbr].detach) && animation 
+          && current_config.opaquemove);
+
   if ((popup_menus[nbr].entries == 0) || (popup_menus[nbr].detach))
     gtk_widget_hide (popup_menus[nbr].popup_detach);
+  else if (current_config.detach_menu)
+    gtk_widget_show (popup_menus[nbr].popup_detach);
   else
-    if (current_config.detach_menu)
-      gtk_widget_show (popup_menus[nbr].popup_detach);
-    else
-      gtk_widget_hide (popup_menus[nbr].popup_detach);
+    gtk_widget_hide (popup_menus[nbr].popup_detach);
 
   if (popup_menus[nbr].entries == NBMAXITEMS)
-    gtk_widget_hide (popup_menus[nbr].popup_addicon_frame);
+    {
+      gtk_widget_hide (popup_menus[nbr].popup_addicon_button);
+      gtk_widget_hide (popup_menus[nbr].popup_addicon_separator);
+    }
   else
-    gtk_widget_show (popup_menus[nbr].popup_addicon_frame);
+    {
+      gtk_widget_show (popup_menus[nbr].popup_addicon_button);
+      gtk_widget_show (popup_menus[nbr].popup_addicon_separator);
+    }
 
   if ((x > 0) && (y > 0)) {
     px = x;
@@ -345,6 +349,12 @@ show_popup_menu (gint nbr, gint x, gint y)
       px = popup_menus[nbr].lastpx;
       py = popup_menus[nbr].lastpy;
     }
+  if (!GTK_WIDGET_REALIZED(popup_menus[nbr].popup_toplevel))
+    {
+      gtk_widget_realize (popup_menus[nbr].popup_toplevel);
+      set_icon (popup_menus[nbr].popup_toplevel, "Popup Menu", menu_icon);
+      gnome_sticky(popup_menus[nbr].popup_toplevel->window);
+    }
   gtk_widget_size_request ((GtkWidget *) popup_menus[nbr].popup_toplevel, &requisition);
   uwidth  = requisition.width;
   uheight = requisition.height;
@@ -364,30 +374,121 @@ show_popup_menu (gint nbr, gint x, gint y)
     gtk_widget_set_uposition (popup_menus[nbr].popup_toplevel, popup_menus[nbr].uposx,
                                                                popup_menus[nbr].uposy);
   
-  gdk_window_set_decorations (((GtkWidget *)popup_menus[nbr].popup_toplevel)->window, ((popup_menus[nbr].detach) ? (GDK_DECOR_TITLE | GDK_DECOR_MENU) : 0));
-  gtk_widget_show (popup_menus[nbr].popup_toplevel);
-  for (i = 0 ; i < NBMAXITEMS; i++)
+  if ((!popup_menus[nbr].animatewin) && anim)
     {
-      gtk_button_leave (GTK_BUTTON (popup_menus[nbr].popup_buttons[i].item_button));
+      popup_menus[nbr].animatewin = gtk_window_new (GTK_WINDOW_POPUP);
+      gtk_window_set_modal (GTK_WINDOW(popup_menus[nbr].animatewin), TRUE);
     }
+
+  if (anim) 
+    {
+      gtk_window_set_policy (GTK_WINDOW (popup_menus[nbr].animatewin), FALSE, FALSE, FALSE);
+      gtk_widget_set_uposition (popup_menus[nbr].animatewin, tempx, tempy + uheight - speed);
+      gtk_widget_set_usize (popup_menus[nbr].animatewin, uwidth, 0);
+      gtk_widget_show_now (popup_menus[nbr].animatewin);
+      gettimeofday(&old_tv, &tz);
+      for (i = 0; i < (uheight - speed); i += speed)
+        {
+          gdk_window_move_resize (popup_menus[nbr].animatewin->window, tempx, tempy + uheight - i, uwidth, i);
+          gdk_flush ();
+          gettimeofday(&new_tv, &tz);
+          /* Kind of smart here : We compute the speed of the animation based on the speed of the system 
+             and/or network. The value 400.0 means 400 pixels per second */
+          speed = 400.0/100000.0 * ((new_tv.tv_sec * 1000000.0 + new_tv.tv_usec) - 
+                                    (old_tv.tv_sec * 1000000.0 + old_tv.tv_usec));
+          if (speed < 1.0) speed = 1.0; /* in case we've got a *very* fast system */
+          old_tv = new_tv;
+        }
+      gdk_window_move_resize (popup_menus[nbr].animatewin->window, tempx, tempy, uwidth, uheight);
+    }
+  gdk_window_set_decorations (((GtkWidget *)popup_menus[nbr].popup_toplevel)->window, ((popup_menus[nbr].detach) ? (GDK_DECOR_TITLE | GDK_DECOR_MENU) : 0));
+  if (!GTK_WIDGET_VISIBLE(popup_menus[nbr].popup_toplevel))
+    gtk_widget_show_now (popup_menus[nbr].popup_toplevel);
+  gdk_flush ();
+
+  for (i = 0 ; i < NBMAXITEMS; i++)
+    gtk_button_leave (GTK_BUTTON (popup_menus[nbr].popup_buttons[i].item_button));
+
+  if (popup_menus[nbr].animatewin && anim)
+    gtk_widget_hide (popup_menus[nbr].animatewin);
 }
 
 void
-hide_popup_menu (gint nbr)
+hide_popup_menu (gint nbr, gboolean animation)
 {
-  gdk_window_get_root_origin (((GtkWidget *) popup_menus[nbr].popup_toplevel)->window,
+  gint uwidth  = 0;
+  gint uheight = 0;
+  gint uposx   = 0;
+  gint uposy   = 0;
+  gboolean anim;
+  gint i;
+  GtkRequisition requisition;
+  gdouble speed = 0.0;
+  struct timeval new_tv, old_tv;
+  struct timezone tz;
+
+  anim = ((!popup_menus[nbr].detach) && animation 
+          && current_config.opaquemove);
+
+  if (anim)
+    {
+      gdk_window_get_root_origin (((GtkWidget *) popup_menus[nbr].popup_toplevel)->window,
+                               &uposx, 
+			       &uposy);
+      gtk_widget_size_request ((GtkWidget *) popup_menus[nbr].popup_toplevel, &requisition);
+      uwidth  = requisition.width;
+      uheight = requisition.height;
+    }
+
+  if ((popup_menus[nbr].popup_toplevel)->window)
+    gdk_window_get_root_origin (((GtkWidget *) popup_menus[nbr].popup_toplevel)->window,
                                &popup_menus[nbr].uposx, 
-			       &popup_menus[nbr].uposy);
+                               &popup_menus[nbr].uposy);
+  if ((!popup_menus[nbr].animatewin) && anim)
+    {
+      popup_menus[nbr].animatewin = gtk_window_new (GTK_WINDOW_POPUP);
+      gtk_window_set_modal (GTK_WINDOW(popup_menus[nbr].animatewin), TRUE);
+    }
+
+  if (anim)
+    {
+      gtk_window_set_policy (GTK_WINDOW (popup_menus[nbr].animatewin), FALSE, FALSE, FALSE);
+      gtk_widget_set_uposition (popup_menus[nbr].animatewin, uposx, uposy);
+      gtk_widget_set_usize (popup_menus[nbr].animatewin, uwidth, uheight);
+      gtk_widget_show_now (popup_menus[nbr].animatewin);
+    }
+
   gtk_widget_hide (popup_menus[nbr].popup_toplevel);
-  gtk_widget_unrealize (popup_menus[nbr].popup_toplevel);
-  gtk_widget_realize (popup_menus[nbr].popup_toplevel);
-  gnome_sticky(popup_menus[nbr].popup_toplevel->window);
+  gdk_window_withdraw ((popup_menus[nbr].popup_toplevel)->window);
+  
+  my_flush_events ();
+  gettimeofday(&old_tv, &tz);
+
+  if (anim)
+    {
+      for (i = uheight; i > 0; i -= speed)
+        {
+          gdk_window_move_resize (popup_menus[nbr].animatewin->window, 
+                                  uposx, uposy + uheight - i, uwidth, i);
+          gdk_flush ();
+          gettimeofday(&new_tv, &tz);
+          /* Kind of smart here : We compute the speed of the animation based on the speed of the system 
+             and/or network. The value 400.0 means 400 pixels per second */
+          speed = 400.0/100000.0 * ((new_tv.tv_sec * 1000000.0 + new_tv.tv_usec) - 
+                                    (old_tv.tv_sec * 1000000.0 + old_tv.tv_usec));
+          if (speed < 1.0) speed = 1.0; /* in case we've got a *very* fast system */
+          old_tv = new_tv;
+        }
+    }
+
+  if (popup_menus[nbr].animatewin && anim)
+    gtk_widget_hide (popup_menus[nbr].animatewin);
 }
 
 void
 close_popup_menu (gint nbr)
 {
-  hide_popup_menu(nbr);
+  hide_popup_menu(nbr, TRUE);
   popup_menus[nbr].detach = FALSE;
 }
 
@@ -408,47 +509,76 @@ void
 update_popup_size(void)
 {
   int i, j;
-  GdkPixmap *pixmap = NULL;
-  GdkBitmap *mask = NULL;
-  char * pixfile;
   
   for (i = 0; i < NBPOPUPS; i++) {
     gtk_widget_set_usize (popup_menus[i].popup_addicon_icon_frame, POPUP_ICON_SIZE, POPUP_ICON_SIZE);
-    for (j = 0; j < NBMAXITEMS; j++) {
+    for (j = 0; j < popup_menus[i].entries; j++) {
       gtk_widget_set_usize (popup_menus[i].popup_buttons[j].item_pixmap_frame, POPUP_ICON_SIZE, POPUP_ICON_SIZE);
       if (j < popup_menus[i].entries) {
-	pixfile = popup_menus[i].popup_buttons[j].pixfile;
-	if (check_file_exists ("/", pixfile)) {
-	  pixmap = MyCreateGdkPixmapFromFile(pixfile, popup_menus[i].popup_buttons[j].item_pixmap_frame, &mask);
+	if (existfile(popup_menus[i].popup_buttons[j].pixfile))
+	  {
+	    MySetPixmapFile (popup_menus[i].popup_buttons[j].item_pixmap, 
+                	     popup_menus[i].popup_buttons[j].item_pixmap_frame,
+                	     popup_menus[i].popup_buttons[j].pixfile);
+	  }
+	else
+	  {
+	    MySetPixmapData (popup_menus[i].popup_buttons[j].item_pixmap, 
+                	     popup_menus[i].popup_buttons[j].item_pixmap_frame,
+                	     defaulticon);
+	  }
 	}
-	else {
-	  pixmap = MyCreateGdkPixmapFromData (defaulticon, popup_menus[i].popup_buttons[j].item_pixmap_frame, &mask);
-	}
-	gtk_pixmap_set (GTK_PIXMAP(popup_menus[i].popup_buttons[j].item_pixmap), pixmap, mask);
       }
+    for (j = popup_menus[i].entries; j < NBMAXITEMS; j++)
+      gtk_widget_set_usize (popup_menus[i].popup_buttons[j].item_pixmap_frame, POPUP_ICON_SIZE, POPUP_ICON_SIZE);
     }
-  }
 }
 
 void
 set_entry(gint menu, gint entry, char *label, char *pixfile, char *command)
 {
-  GdkPixmap *pixmap = NULL;
-  GdkBitmap *mask = NULL;
+  int l1, l2, l3;
+  char defpix[] = "Default Icon";
+  gboolean nullpix = False;
   
-  strcpy (popup_menus[menu].popup_buttons[entry].label  , label  );
-  strcpy (popup_menus[menu].popup_buttons[entry].pixfile, pixfile);
-  strcpy (popup_menus[menu].popup_buttons[entry].command, command);
-  
-  if (check_file_exists ("/", pixfile)) {
-    pixmap = MyCreateGdkPixmapFromFile(pixfile, popup_menus[menu].popup_buttons[entry].item_pixmap_frame, &mask);
-  }
-  else {
-    pixmap = MyCreateGdkPixmapFromData (defaulticon, popup_menus[menu].popup_buttons[entry].item_pixmap_frame, &mask);
-  }
+  l1 = strlen (label);
+  l2 = strlen (pixfile);
+  l3 = strlen (command);    
 
-  gtk_pixmap_set (GTK_PIXMAP(popup_menus[menu].popup_buttons[entry].item_pixmap), pixmap, mask);
-  
+  if (l2 == 0) 
+    {
+      l2 = strlen (defpix);
+      nullpix = True;
+    }
+
+  if (popup_menus[menu].popup_buttons[entry].label)
+    free (popup_menus[menu].popup_buttons[entry].label);
+  if (popup_menus[menu].popup_buttons[entry].pixfile)
+    free (popup_menus[menu].popup_buttons[entry].pixfile);
+  if (popup_menus[menu].popup_buttons[entry].command)
+    free (popup_menus[menu].popup_buttons[entry].command);
+
+  popup_menus[menu].popup_buttons[entry].label   = (char *) malloc (sizeof (char) * (l1 + 1));
+  popup_menus[menu].popup_buttons[entry].pixfile = (char *) malloc (sizeof (char) * (l2 + 1));
+  popup_menus[menu].popup_buttons[entry].command = (char *) malloc (sizeof (char) * (l3 + 1));
+
+  strcpy (popup_menus[menu].popup_buttons[entry].label  , label  );
+  strcpy (popup_menus[menu].popup_buttons[entry].pixfile, (nullpix ? defpix : pixfile));
+  strcpy (popup_menus[menu].popup_buttons[entry].command, command);
+
+  if (existfile(popup_menus[menu].popup_buttons[entry].pixfile))
+    {
+      MySetPixmapFile (popup_menus[menu].popup_buttons[entry].item_pixmap, 
+                       popup_menus[menu].popup_buttons[entry].item_pixmap_frame,
+                       popup_menus[menu].popup_buttons[entry].pixfile);
+    }
+  else
+    {
+      MySetPixmapData (popup_menus[menu].popup_buttons[entry].item_pixmap, 
+                       popup_menus[menu].popup_buttons[entry].item_pixmap_frame,
+                       defaulticon);
+    }
+      
   gtk_label_set_text (GTK_LABEL(popup_menus[menu].popup_buttons[entry].item_label), label);
 
   gtk_tooltips_set_tip (popup_menus[menu].popup_buttons[entry].item_tooltip,
@@ -462,10 +592,42 @@ set_like_entry(gint menu_s, gint entry_s, gint menu_d, gint entry_d)
 {
   GdkPixmap *pixmap = NULL;
   GdkBitmap *mask = NULL;
-  
-  strcpy (popup_menus[menu_d].popup_buttons[entry_d].label  , popup_menus[menu_s].popup_buttons[entry_s].label  );
-  strcpy (popup_menus[menu_d].popup_buttons[entry_d].pixfile, popup_menus[menu_s].popup_buttons[entry_s].pixfile);
-  strcpy (popup_menus[menu_d].popup_buttons[entry_d].command, popup_menus[menu_s].popup_buttons[entry_s].command);
+  int l1, l2, l3;
+  char defpix[] = "Default Icon";
+  gboolean nullpix = False;
+
+  if((!popup_menus[menu_s].popup_buttons[entry_s].label) ||
+     (!popup_menus[menu_s].popup_buttons[entry_s].pixfile) ||
+     (!popup_menus[menu_s].popup_buttons[entry_s].command))
+    return;
+
+  l1 = strlen (popup_menus[menu_s].popup_buttons[entry_s].label);
+  l2 = strlen (popup_menus[menu_s].popup_buttons[entry_s].pixfile);
+  l3 = strlen (popup_menus[menu_s].popup_buttons[entry_s].command);    
+
+  if (l2 == 0) 
+    {
+      l2 = strlen (defpix);
+      nullpix = True;
+    }
+
+  if (popup_menus[menu_d].popup_buttons[entry_d].label)
+    free (popup_menus[menu_d].popup_buttons[entry_d].label);
+  if (popup_menus[menu_d].popup_buttons[entry_d].pixfile)
+    free (popup_menus[menu_d].popup_buttons[entry_d].pixfile);
+  if (popup_menus[menu_d].popup_buttons[entry_d].command)
+    free (popup_menus[menu_d].popup_buttons[entry_d].command);
+
+  popup_menus[menu_d].popup_buttons[entry_d].label   = (char *) malloc (sizeof (char) * (l1 + 1));
+  popup_menus[menu_d].popup_buttons[entry_d].pixfile = (char *) malloc (sizeof (char) * (l2 + 1));
+  popup_menus[menu_d].popup_buttons[entry_d].command = (char *) malloc (sizeof (char) * (l3 + 1));
+
+  strcpy (popup_menus[menu_d].popup_buttons[entry_d].label  , 
+       popup_menus[menu_s].popup_buttons[entry_s].label  );
+  strcpy (popup_menus[menu_d].popup_buttons[entry_d].pixfile, 
+       (nullpix ? defpix : popup_menus[menu_s].popup_buttons[entry_s].pixfile));
+  strcpy (popup_menus[menu_d].popup_buttons[entry_d].command, 
+       popup_menus[menu_s].popup_buttons[entry_s].command);
   
   gtk_pixmap_get (GTK_PIXMAP(popup_menus[menu_s].popup_buttons[entry_s].item_pixmap), &pixmap, &mask);
   gtk_pixmap_set (GTK_PIXMAP(popup_menus[menu_d].popup_buttons[entry_d].item_pixmap), pixmap, mask);
@@ -492,13 +654,31 @@ void
 remove_popup_entry(gint nbr_menu, gint entry)
 {
   gint i;
+  gint j;
 
-  for (i = entry; i < (NBMAXITEMS - 1); i++) {
+  j = --popup_menus[nbr_menu].entries;
+  for (i = entry; i < j; i++) {
     set_like_entry(nbr_menu, i + 1, nbr_menu, i);
   }
-  set_entry(nbr_menu, (NBMAXITEMS - 1), "Empty", "Default", "");
-  popup_menus[nbr_menu].entries--;
   update_popup_entries(nbr_menu);
+  if (popup_menus[nbr_menu].popup_buttons[j].label)
+    {
+      free(popup_menus[nbr_menu].popup_buttons[j].label);
+      popup_menus[nbr_menu].popup_buttons[j].label = NULL;
+    }
+  if (popup_menus[nbr_menu].popup_buttons[j].pixfile)
+    {
+      free(popup_menus[nbr_menu].popup_buttons[j].pixfile);
+      popup_menus[nbr_menu].popup_buttons[j].pixfile = NULL;
+    }
+  if (popup_menus[nbr_menu].popup_buttons[j].command)
+    {
+      free(popup_menus[nbr_menu].popup_buttons[j].command);
+      popup_menus[nbr_menu].popup_buttons[j].command = NULL;
+    }
+  MySetPixmapData (popup_menus[nbr_menu].popup_buttons[j].item_pixmap, 
+                   popup_menus[nbr_menu].popup_buttons[j].item_pixmap_frame,
+                   empty);
 }
 
 char *
