@@ -170,7 +170,6 @@ create_setup (XFCE_palette * pal)
   gtk_widget_set_usize (setup, -2, 400);
   gtk_window_set_policy (GTK_WINDOW (setup), FALSE, TRUE, FALSE);
   gtk_widget_realize (setup);
-  gnome_sticky(setup->window);
   
   accel_group = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (setup), accel_group);
@@ -906,15 +905,65 @@ create_setup (XFCE_palette * pal)
   gtk_container_border_width (GTK_CONTAINER (setup_options.setup_cancel_button), 5);
   GTK_WIDGET_SET_FLAGS (setup_options.setup_cancel_button, GTK_CAN_DEFAULT);
 
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_ok_button), "clicked",
-				 GTK_SIGNAL_FUNC (setup_ok_cb),
-				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_cancel_button), "clicked",
-				 GTK_SIGNAL_FUNC (setup_cancel_cb),
-				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_apply_button), "clicked",
-				 GTK_SIGNAL_FUNC (setup_apply_cb),
-				 NULL);
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_ok_button), "clicked",
+		      GTK_SIGNAL_FUNC (setup_ok_cb),
+		      NULL);
+                      
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_cancel_button), "clicked",
+		      GTK_SIGNAL_FUNC (setup_cancel_cb),
+		      NULL);
+                      
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_apply_button), "clicked",
+		      GTK_SIGNAL_FUNC (setup_apply_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_palette_default_button), "clicked",
+		      GTK_SIGNAL_FUNC (setup_default_cb),
+		      NULL);
+                                 
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_palette_load_button), "clicked",
+		      GTK_SIGNAL_FUNC (setup_loadpal_cb),
+		      NULL);
+                                 
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_palette_save_button), "clicked",
+		      GTK_SIGNAL_FUNC (setup_savepal_cb),
+		      NULL);
+                                 
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_font_xfce_font_button), "clicked",
+		      GTK_SIGNAL_FUNC (xfce_font_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_digital_clock_checkbutton), "clicked",
+		      GTK_SIGNAL_FUNC (toggle_digital_clock_checkbutton_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_focusmode_checkbutton), "clicked",
+		      GTK_SIGNAL_FUNC (toggle_focusmode_checkbutton_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_repaint_checkbutton), "clicked",
+		      GTK_SIGNAL_FUNC (toggle_repaint_checkbutton_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup), "delete_event",
+                      GTK_SIGNAL_FUNC (setup_delete_event), 
+                      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup), "destroy",
+                      GTK_SIGNAL_FUNC (setup_cancel_cb), 
+                      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_font_title_button), "clicked",
+		      GTK_SIGNAL_FUNC (xfwm_titlefont_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_font_menu_button), "clicked",
+		      GTK_SIGNAL_FUNC (xfwm_menufont_cb),
+		      NULL);
+
+  gtk_signal_connect (GTK_OBJECT (setup_options.setup_font_icon_button), "clicked",
+		      GTK_SIGNAL_FUNC (xfwm_iconfont_cb),
+		      NULL);
 
   gtk_widget_add_accelerator (setup_options.setup_ok_button, "clicked", accel_group,
                               GDK_Return, 0, GTK_ACCEL_VISIBLE);
@@ -929,49 +978,6 @@ create_setup (XFCE_palette * pal)
   gtk_widget_add_accelerator (setup_options.setup_cancel_button, "clicked", accel_group,
                               GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_palette_default_button), "clicked",
-				 GTK_SIGNAL_FUNC (setup_default_cb),
-				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_palette_load_button), "clicked",
-				 GTK_SIGNAL_FUNC (setup_loadpal_cb),
-				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_palette_save_button), "clicked",
-				 GTK_SIGNAL_FUNC (setup_savepal_cb),
-				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_font_xfce_font_button), "clicked",
-				 GTK_SIGNAL_FUNC (xfce_font_cb),
-				 NULL);
-
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_digital_clock_checkbutton), "clicked",
-				 GTK_SIGNAL_FUNC (toggle_digital_clock_checkbutton_cb),
-				 NULL);
-
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_focusmode_checkbutton), "clicked",
-				 GTK_SIGNAL_FUNC (toggle_focusmode_checkbutton_cb),
-				 NULL);
-
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_repaint_checkbutton), "clicked",
-				 GTK_SIGNAL_FUNC (toggle_repaint_checkbutton_cb),
-				 NULL);
-
-  gtk_signal_connect (GTK_OBJECT (setup), "delete_event",
-                          GTK_SIGNAL_FUNC (setup_delete_event), NULL);
-
-  gtk_signal_connect (GTK_OBJECT (setup), "destroy",
-                      GTK_SIGNAL_FUNC (setup_cancel_cb), NULL);
-
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_font_title_button), "clicked",
-				 GTK_SIGNAL_FUNC (xfwm_titlefont_cb),
-				 NULL);
-
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_font_menu_button), "clicked",
-				 GTK_SIGNAL_FUNC (xfwm_menufont_cb),
-				 NULL);
-
-  gtk_signal_connect_object (GTK_OBJECT (setup_options.setup_font_icon_button), "clicked",
-				 GTK_SIGNAL_FUNC (xfwm_iconfont_cb),
-				 NULL);
-
   return setup;
 }
 
@@ -1064,6 +1070,7 @@ show_setup (XFCE_palette * pal)
                                 (current_config.startup_flags & F_PAGERMODULE));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (setup_options.setup_gnomemodule_checkbutton),
                                 (current_config.startup_flags & F_GNOMEMODULE));
+  gnome_sticky(setup->window);
   gtk_widget_show (setup);
   gtk_main ();
 }
@@ -1071,6 +1078,7 @@ show_setup (XFCE_palette * pal)
 void
 get_setup_values(void)
 {
+  char *s1, *s2, *s3;
   current_config.colorize_root = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_repaint_checkbutton));
   current_config.gradient_root = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_gradient_checkbutton));
   current_config.detach_menu = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_tearoff_checkbutton));
@@ -1088,9 +1096,18 @@ get_setup_values(void)
   current_config.iconpos = (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_iconpos_leftbutton)) ? 1 : 0)
                      + 2 * (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_iconpos_botbutton)) ? 1 : 0)
 		     + 3 * (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_iconpos_rightbutton)) ? 1 : 0);
-  strcpy (current_config.fonts[0], gtk_entry_get_text(GTK_ENTRY(setup_options.setup_font_title_entry)));
-  strcpy (current_config.fonts[1], gtk_entry_get_text(GTK_ENTRY(setup_options.setup_font_menu_entry)));
-  strcpy (current_config.fonts[2], gtk_entry_get_text(GTK_ENTRY(setup_options.setup_font_icon_entry)));
+  if (current_config.fonts[0]) free(current_config.fonts[0]);
+  if (current_config.fonts[1]) free(current_config.fonts[1]);
+  if (current_config.fonts[2]) free(current_config.fonts[2]);
+  s1 = gtk_entry_get_text(GTK_ENTRY(setup_options.setup_font_title_entry));
+  s2 = gtk_entry_get_text(GTK_ENTRY(setup_options.setup_font_menu_entry));
+  s3 = gtk_entry_get_text(GTK_ENTRY(setup_options.setup_font_icon_entry));
+  current_config.fonts[0] = (char *) malloc (sizeof (char) * (strlen (s1) + 1));
+  current_config.fonts[1] = (char *) malloc (sizeof (char) * (strlen (s2) + 1));
+  current_config.fonts[2] = (char *) malloc (sizeof (char) * (strlen (s3) + 1));
+  strcpy (current_config.fonts[0], s1);
+  strcpy (current_config.fonts[1], s2);
+  strcpy (current_config.fonts[2], s3);
   current_config.digital_clock = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_digital_clock_checkbutton));
   current_config.hrs_mode = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_options.setup_hrs_mode_checkbutton));
   current_config.tooltipsdelay = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (setup_options.setup_tooltipsdelay_spinbutton));				
