@@ -46,7 +46,7 @@
 #include "mininf.h"
 #include "minipnt.h"
 #include "minilock.h"
-#include "defaulticon.h"
+#include "empty.h"
 #include "handle.h"
 #include "closepix.h"
 #include "iconify.h"
@@ -88,18 +88,20 @@ gxfce_clock_make_popup (MyGtkClock *clock)
   gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM(gxfce_clock_digital_mode), 1);
 
   gtk_menu_append (GTK_MENU(gxfce_clock_popup_menu), gxfce_clock_digital_mode);
-  gtk_signal_connect_object(GTK_OBJECT(gxfce_clock_digital_mode), "activate",
-        		    GTK_SIGNAL_FUNC(clock_digital_cb),
-        		    (gpointer) clock);
+  gtk_signal_connect (GTK_OBJECT(gxfce_clock_digital_mode), "activate",
+        	      GTK_SIGNAL_FUNC(clock_digital_cb),
+        	      (gpointer) clock);
+                      
   gtk_widget_show (gxfce_clock_digital_mode);
 
   gxfce_clock_hrs_mode = gtk_check_menu_item_new_with_label(_("24 hours mode"));
   gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM(gxfce_clock_hrs_mode), 1);
 
   gtk_menu_append (GTK_MENU(gxfce_clock_popup_menu), gxfce_clock_hrs_mode);
-  gtk_signal_connect_object(GTK_OBJECT(gxfce_clock_hrs_mode), "activate",
-        		    GTK_SIGNAL_FUNC(clock_hrs_cb),
-        		    (gpointer) clock);
+  gtk_signal_connect (GTK_OBJECT(gxfce_clock_hrs_mode), "activate",
+        	      GTK_SIGNAL_FUNC(clock_hrs_cb),
+        	      (gpointer) clock);
+                     
   gtk_widget_show (gxfce_clock_hrs_mode);
   return gxfce_clock_popup_menu;
 }
@@ -203,14 +205,13 @@ create_gxfce_select_button (GtkWidget *toplevel, XFCE_palette * pal, gint nbr)
                              GTK_SIGNAL_FUNC (select_cb),
                              (gpointer) nbr);
 		      
-  select_buttons.select_pixmap[nbr] = MyCreateFromPixmapData (select_buttons.select_button[nbr], defaulticon);
+  select_buttons.select_pixmap[nbr] = MyCreateFromPixmapData (select_buttons.select_button[nbr], empty);
   if (select_buttons.select_pixmap[nbr] == NULL)
     g_error (_("Couldn't create pixmap"));
   gtk_widget_set_name (select_buttons.select_pixmap[nbr], "gxfce_select_button_pixmap");
   gtk_object_set_data (GTK_OBJECT (toplevel), "gxfce_select_button_pixmap", select_buttons.select_pixmap[nbr]);
   gtk_widget_show (select_buttons.select_pixmap[nbr]);
   gtk_container_add (GTK_CONTAINER (select_buttons.select_button[nbr]), select_buttons.select_pixmap[nbr]);
-  gtk_widget_set_style(select_buttons.select_pixmap[nbr], pal->cm[1]);
   
   return select_buttons.select_button[nbr];
 }
@@ -369,7 +370,8 @@ create_gxfce (XFCE_palette *pal)
   GtkWidget *gxfce_iconify_pixmap;
 
   int i;  
-  
+  int nbselects = NBSELECTS;
+
   gxfce = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_policy (GTK_WINDOW (gxfce), FALSE, FALSE, TRUE);
   gtk_widget_set_name (gxfce, "gxfce");
@@ -717,29 +719,29 @@ create_gxfce (XFCE_palette *pal)
                       GTK_SIGNAL_FUNC (destroy_cb), NULL);
 
 
-  gtk_signal_connect_object (GTK_OBJECT (gxfce_quit), "clicked",
-                             GTK_SIGNAL_FUNC (quit_cb),
-                             GTK_OBJECT (gxfce));
+  gtk_signal_connect  (GTK_OBJECT (gxfce_quit), "clicked",
+                       GTK_SIGNAL_FUNC (quit_cb),
+                       GTK_OBJECT (gxfce));
 
-  gtk_signal_connect_object (GTK_OBJECT (gxfce_close_button), "clicked",
-                             GTK_SIGNAL_FUNC (quit_cb),
-                             GTK_OBJECT (gxfce));
+  gtk_signal_connect  (GTK_OBJECT (gxfce_close_button), "clicked",
+                       GTK_SIGNAL_FUNC (quit_cb),
+                       GTK_OBJECT (gxfce));
 
   gtk_signal_connect (GTK_OBJECT (gxfce_setup), "clicked",
-                             GTK_SIGNAL_FUNC (setup_cb),
-                             pal);
+                      GTK_SIGNAL_FUNC (setup_cb),
+                      pal);
 
-  gtk_signal_connect_object (GTK_OBJECT (gxfce_info), "clicked",
-                             GTK_SIGNAL_FUNC (info_cb),
-                             GTK_OBJECT (gxfce));
+  gtk_signal_connect (GTK_OBJECT (gxfce_info), "clicked",
+                      GTK_SIGNAL_FUNC (info_cb),
+                      GTK_OBJECT (gxfce));
 
-  gtk_signal_connect( GTK_OBJECT(select_buttons.select_button[NBSELECTS]), "button_press_event",
-                              GTK_SIGNAL_FUNC(select_modify_cb),
-			      (gpointer) NBSELECTS);
+  gtk_signal_connect (GTK_OBJECT(select_buttons.select_button[NBSELECTS]), "button_press_event",
+                      GTK_SIGNAL_FUNC(select_modify_cb),
+		      (gpointer) nbselects);
 
   gtk_signal_connect (GTK_OBJECT (select_buttons.select_button[NBSELECTS]), "clicked",
-                             GTK_SIGNAL_FUNC (select_cb),
-                             (gpointer) NBSELECTS);
+                      GTK_SIGNAL_FUNC (select_cb),
+                      (gpointer) nbselects);
  
   gtk_signal_connect (GTK_OBJECT (gxfce_clock_event), "button_press_event",
                       GTK_SIGNAL_FUNC (gxfce_clock_show_popup_cb),
