@@ -22,16 +22,16 @@
 void quit_cb(GtkWidget *, GdkEvent *, gpointer);
 gint delete_event(GtkWidget *, GdkEvent *, gpointer);
 void destroy_cb(GtkWidget *);
-void font_cb(GtkWidget *);
-void foreground_cb(GtkWidget *);
-void background_cb(GtkWidget *);
+void font_cb(GtkWidget *, gpointer);
+void foreground_cb(GtkWidget *, gpointer);
+void background_cb(GtkWidget *, gpointer);
 gint show_popup_cb (GtkWidget *, GdkEventButton *, gpointer);
-void toggle_menubar_cb(GtkWidget *);
-void toggle_calendar_cb(GtkWidget *);
-void toggle_digital_cb(GtkWidget *);
-void toggle_secs_cb(GtkWidget *);
-void toggle_ampm_cb(GtkWidget *);
-void toggle_military_cb(GtkWidget *);
-void about_cb(GtkWidget *);
+void toggle_menubar_cb(GtkWidget *, gpointer);
+void toggle_calendar_cb(GtkWidget *, gpointer);
+void toggle_digital_cb(GtkWidget *, gpointer);
+void toggle_secs_cb(GtkWidget *, gpointer);
+void toggle_ampm_cb(GtkWidget *, gpointer);
+void toggle_military_cb(GtkWidget *, gpointer);
+void about_cb(GtkWidget *, gpointer data);
 
 #endif
