@@ -30,7 +30,6 @@
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
-#include <X11/cursorfont.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <stdio.h>
@@ -38,21 +37,20 @@
 #include <unistd.h>
 #include <errno.h>
 
+#if defined ___AIX || defined _AIX || defined __QNX__ || defined ___AIXV3 || defined AIXV3 || defined _SEQUENT_
+#include <sys/select.h>
+#endif
+
 #include "utils.h"
 #include "module.h"
 #include "../xfwm/version.h"
 #include "../xfwm/xfwm.h"
-
 #include "xfgnome.h"
 
 static s_list *window_list; /* single linked list of window id's etc */
 static int current_desk = 0;/* current workspace */
 static char **desk_names;   /* holds workspace names */
 static int desk_count = 0;  /* number of workspaces */
-static Bool PAGES = False;  /* virtual paging or no */
-static int area_x = 0;      /* area coordinates for virtual paging */
-static int area_y = 0;      /*                ""                   */ 
-  
   
 /* inserts a window id into the list
  * args: list, id
@@ -198,7 +196,7 @@ int s_list_free (s_list *list)
     return 0;
 }
 
-/* returns the count of items in a list, skips transient and windowlistskip
+/* returns the count of items in a list, skips windowlistskip
  * windows
  * args: list
  */
@@ -212,8 +210,7 @@ int s_list_count (s_list *list)
     while (tmp)
     {
         if (!(tmp->flags & WINDOWLISTSKIP))
-            if (!(tmp->flags & TRANSIENT))
-                count++;
+          count++;
         tmp = tmp->next;
     }
     return count;
@@ -244,12 +241,6 @@ gnome_compliance_init ()
     _XA_WIN_DESKTOP_BUTTON_PROXY = XInternAtom(dpy, "_WIN_DESKTOP_BUTTON_PROXY",
                                    False);
 
-    if (PAGES)
-    {
-        _XA_WIN_AREA_COUNT = XInternAtom(dpy, "_WIN_AREA_COUNT", False);
-        _XA_WIN_AREA = XInternAtom(dpy, "_WIN_AREA", False);
-    }
-
     /* create the GNOME window */
     gnome_win = XCreateSimpleWindow (dpy, root_win, 0, 0, 5, 5, 0, 0, 0);
 
@@ -270,41 +261,11 @@ gnome_compliance_init ()
     supported_list[count++] = _XA_WIN_WORKSPACE_COUNT; /* done */
     supported_list[count++] = _XA_WIN_WORKSPACE_NAMES; /* done */
     supported_list[count++] = _XA_WIN_CLIENT_LIST;     /* done */
-    if (PAGES)
-    {
-        supported_list[count++] = _XA_WIN_AREA_COUNT;
-        supported_list[count++] = _XA_WIN_AREA;
-    }
-
     XChangeProperty(dpy, root_win, _XA_WIN_PROTOCOLS, XA_ATOM, 32, PropModeReplace,
                     (unsigned char *)supported_list, count);
 
 }
 
-/* sets the number of virtual pages per desk */
-static void
-gnome_set_area_count ()
-{
-    CARD32 val[2];
-
-    val[0] = 2;
-    val[1] = 2;
-    XChangeProperty (dpy, root_win, _XA_WIN_AREA_COUNT, XA_CARDINAL, 32,
-                     PropModeReplace, (unsigned char *)&val, 2);
-}
-
-/* sets the current visible page */
-static void
-gnome_set_current_area (int x, int y)
-{
-    CARD32 val[2];
-
-    val[0] = x;
-    val[1] = y;
-    XChangeProperty (dpy, root_win, _XA_WIN_AREA, XA_CARDINAL, 32,
-                     PropModeReplace, (unsigned char *)&val, 2);
-}
-
 /* sets the current workspace */
 static void
 gnome_set_current_workspace (int current_desk)
@@ -337,7 +298,7 @@ gnome_set_workspace_names (int count, char **names)
     }
 }
 
-/* sets the window list property, skips transients and winlistskip windows */
+/* sets the window list property, skips winlistskip windows */
 static void
 gnome_set_client_list (s_list *list)
 {
@@ -361,11 +322,7 @@ gnome_set_client_list (s_list *list)
         while (tmp)
         {
             if (!(tmp->flags & WINDOWLISTSKIP))
-            {
-                if (!(tmp->flags & TRANSIENT))
-                    windows[count++] = tmp->id;
-            }
-
+              windows[count++] = tmp->id;
             tmp=tmp->next;
         }
     XChangeProperty (dpy, root_win, _XA_WIN_CLIENT_LIST, XA_CARDINAL, 32,
@@ -393,9 +350,6 @@ gnome_set_win_hints (list_item *xfwm_window)
         if (xfwm_window->workspace != current_desk)
             flags |= WIN_STATE_HID_WORKSPACE;
 
-    if (xfwm_window->flags & TRANSIENT)
-        flags |= WIN_STATE_HID_TRANSIENT;
-
     XChangeProperty (dpy, (Window) xfwm_window->id,  _XA_WIN_WORKSPACE, XA_CARDINAL,
                      32, PropModeReplace, (unsigned char *)&xfwm_window->workspace, 1);
 
@@ -604,10 +558,6 @@ process_message (unsigned long type, int elements, unsigned long *body)
     case M_ICON_NAME:
         status = list_icon_name (body);
         break;
-    case M_NEW_PAGE:
-        if (PAGES)
-            status = list_page_change (body);
-        break;
     case M_NEW_DESK:
         status = list_desk_change (body);
         break;
@@ -685,8 +635,6 @@ list_add_window (unsigned long *body)
     item = s_list_insert (window_list, body[0]);
     item->flags = body[8];
     item->workspace = body[7];
-    item->area_x = area_x;
-    item->area_y = area_y;
     gnome_check_client_hints (item->id);
     gnome_set_win_hints (item);
     gnome_set_client_list (window_list);
@@ -724,6 +672,9 @@ list_destroy_window (unsigned long *body)
     {
         s_list_remove_by_data (window_list, body[0]);
         gnome_set_client_list (window_list);
+        /* Perform some cleanup */
+        XDeleteProperty(dpy, body[0], _XA_WIN_WORKSPACE);
+        XDeleteProperty(dpy, body[0], _XA_WIN_STATE);
         return 1;
     }
     return 0;
@@ -739,11 +690,6 @@ list_end (void)
     gnome_set_current_workspace (current_desk);
     gnome_set_workspace_count (desk_count);
     gnome_set_workspace_names (desk_count, desk_names);
-    if (PAGES)
-    {
-        gnome_set_area_count ();
-        gnome_set_current_area (0, 0);
-    }
     gnome_set_client_list (window_list);
     return 0;
 }
@@ -768,26 +714,6 @@ list_iconify (unsigned long *body)
     return 0;
 }
 
-int
-list_page_change (unsigned long *body)
-{
-    current_desk = (int) body[2];
-
-    if (body[0] > 0)
-    	area_x = 1;
-    else
-    	area_x = 0;
-
-    if (body[1] > 0)
-    	area_y = 1;
-    else
-    	area_y = 0;
-    gnome_set_current_area (area_x, area_y);
-    XFlush (dpy);
-
-    return 0;
-}
-
 int
 list_desk_change (unsigned long *body)
 {
@@ -975,11 +901,14 @@ int main(int argc, char **argv)
   gnome_compliance_init ();
   
   SetMessageMask(fd, 
-  		 M_ADD_WINDOW | 
-		 M_CONFIGURE_WINDOW | 
-		 M_DESTROY_WINDOW | 
-		 M_NEW_PAGE | 
-		 M_NEW_DESK | 
+                 M_ADD_WINDOW|
+                 M_CONFIGURE_WINDOW|
+                 M_DESTROY_WINDOW|
+                 M_NEW_DESK|
+                 M_ICONIFY|
+		 M_DEICONIFY|
+		 M_WINDOW_NAME|
+                 M_ICON_NAME|
 		 M_END_WINDOWLIST);
 
   window_list = s_list_new ();
