@@ -26,7 +26,7 @@
 #define DENOMINATOR	3
 #define THRESH_MIN	1
 #define THRESH_MAX	20
-#define RCFILE          ".xfmouserc"
+#define RCFILE          "xfmouserc"
 
 typedef struct
 {
