@@ -160,7 +160,6 @@ int main(int argc, char **argv)
                  M_CONFIGURE_WINDOW|
                  M_DESTROY_WINDOW|
                  M_FOCUS_CHANGE|
-                 M_NEW_PAGE|
                  M_NEW_DESK|
                  M_RAISE_WINDOW|
                  M_LOWER_WINDOW|
@@ -229,9 +228,6 @@ void process_message(unsigned long type,unsigned long *body)
     case M_FOCUS_CHANGE:
       list_focus(body);
       break;
-    case M_NEW_PAGE:
-      list_new_page(body);
-      break;
     case M_NEW_DESK:
       list_new_desk(body);
       break;
@@ -501,29 +497,6 @@ void list_focus(unsigned long *body)
     }
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *	list_new_page - displays packet contents to stderr
- *
- ***********************************************************************/
-void list_new_page(unsigned long *body)
-{
-  Scr.Vx = (long)body[0];
-  Scr.Vy = (long)body[1];
-  Scr.CurrentDesk = (long)body[2];
-  if((Scr.VxMax != body[3])||(Scr.VyMax != body[4]))
-    {
-      Scr.VxMax = body[3];
-      Scr.VyMax = body[4];
-      ReConfigure();
-    }
-  MovePage();
-  MoveStickyWindows();
-  Hilight(FocusWin,OFF);
-  Hilight(FocusWin,ON);
-}
-
 /***********************************************************************
  *
  *  Procedure:
@@ -794,7 +767,7 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   FD_SET(fd[1],&in_fdset);
 
 #ifdef __hpux
-  select(fd_width,(int *)&in_fdset, 0, 0, NULL);
+  select(fd_width,(fd_set *)&in_fdset, 0, 0, NULL);
 #else
   select(fd_width,&in_fdset, 0, 0, NULL);
 #endif  
@@ -842,16 +815,7 @@ void ParseOptions()
   
   Scr.MyDisplayWidth = DisplayWidth(dpy, Scr.screen);
   Scr.MyDisplayHeight = DisplayHeight(dpy, Scr.screen);
-  
-  Scr.VxMax = 3*Scr.MyDisplayWidth - Scr.MyDisplayWidth;
-  Scr.VyMax = 3*Scr.MyDisplayHeight - Scr.MyDisplayHeight;
-  if(Scr.VxMax <0)
-    Scr.VxMax = 0;
-  if(Scr.VyMax <0)
-    Scr.VyMax = 0;
-  Scr.Vx = 0;
-  Scr.Vy = 0;
-  
+ 
   Clength = strlen(MyName);
   GetConfigLine(fd,&tline);
 
