@@ -111,7 +111,7 @@ void initialize_pager(void)
   extern char *PagerFore, *PagerBack, *HilightC;
   extern char *WindowBack, *WindowFore, *WindowHiBack, *WindowHiFore;
   extern char *font_string, *smallFont;
-  int n,m,w,h,i,x,y;
+  int w,h,i,x,y;
   XGCValues gcv;
   unsigned long gcm;
 
@@ -166,9 +166,6 @@ void initialize_pager(void)
     }
 
 
-  n = Scr.VxMax/Scr.MyDisplayWidth;
-  m = Scr.VyMax/Scr.MyDisplayHeight;
-
   /* Size the window */
   if(Rows < 0)
     {
@@ -195,15 +192,11 @@ void initialize_pager(void)
     Rows = ndesks/Columns;
   if(window_w > 0)
     {
-      window_w = ((window_w - n)/(n+1))*(n+1)+n;
-      Scr.VScale = Columns*(Scr.VxMax + Scr.MyDisplayWidth)/
-	(window_w-Columns+1-Columns*n);
+      Scr.VScale = Columns*Scr.MyDisplayWidth/(window_w-Columns + 1);
     }
   if(window_h > 0)
     {
-      window_h = ((window_h - m)/(m+1))*(m+1)+m;
-      Scr.VScale = Rows*(Scr.VyMax + Scr.MyDisplayHeight)/
-	(window_h+2-Rows*(label_h -m-1));
+      Scr.VScale = Rows*Scr.MyDisplayHeight/(window_h+2-Rows*(label_h - 1));
     }
   if(window_w < 30 * ((desk2 + 1) / Rows))
     {
@@ -259,8 +252,6 @@ void initialize_pager(void)
   if(icon_h < 30)
     icon_h = 30;
 
-  icon_w = (icon_w / (n+1)) *(n+1)+n;
-  icon_h = (icon_h / (m+1)) *(m+1)+m;
   icon_win = XCreateWindow (dpy, Scr.Root, window_x, window_y, 
 			    icon_w,icon_h,
 			    (unsigned int) 1,
@@ -323,8 +314,8 @@ void initialize_pager(void)
       attributes.event_mask = 0;
       attributes.background_pixel = hi_pix;
       
-      w = (window_w - n)/(n+1);
-      h = (window_h - label_h - m)/(m+1);
+      w = window_w;
+      h = window_h - label_h;
       Desks[i].CPagerWin=XCreateWindow(dpy,Desks[i].w,-1000, -1000, w, h,0,
 				       CopyFromParent,
 				       InputOutput,CopyFromParent,
@@ -492,8 +483,8 @@ void DrawGrid(int i, int erase)
   if((i < 0 ) ||(i >= ndesks))
     return;
 
-  MaxW = (Scr.VxMax + Scr.MyDisplayWidth);
-  MaxH = Scr.VyMax + Scr.MyDisplayHeight;
+  MaxW = Scr.MyDisplayWidth;
+  MaxH = Scr.MyDisplayHeight;
 
   x = Scr.MyDisplayWidth;
   y1 = 0;
@@ -570,7 +561,7 @@ void ReConfigure(void)
 {
   Window root;
   unsigned border_width, depth;
-  int n,m,w,h,n1,m1,x,y,i,j,k;
+  int w,h,x,y,i,j,k;
 
 
   XGetGeometry(dpy,Scr.Pager_w,&root,&x,&y,
@@ -578,19 +569,15 @@ void ReConfigure(void)
 	       &border_width,&depth);
 
 
-  n1 = Scr.Vx/Scr.MyDisplayWidth;
-  m1 = Scr.Vy/Scr.MyDisplayHeight;
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
   desk_w = (window_w - Columns + 1)/Columns;
   desk_h = (window_h - Rows*label_h - Rows + 2)/Rows;
-  w = (desk_w - n)/(n+1);
-  h = (desk_h - m)/(m+1);
+  w = desk_w;
+  h = desk_h;
 
   XSetWMNormalHints(dpy,Scr.Pager_w,&sizehints);
 
-  x = (desk_w-n)* Scr.Vx/(Scr.VxMax+Scr.MyDisplayWidth) +n1;
-  y = (desk_h-m)*Scr.Vy/(Scr.VyMax+Scr.MyDisplayHeight) +m1;
+  x = (desk_w)/(Scr.MyDisplayWidth);
+  y = (desk_h)/(Scr.MyDisplayHeight);
 
   for(k=0;k<Rows;k++)
     {
@@ -624,19 +611,14 @@ void ReConfigure(void)
 
 void MovePage(void)
 {
-  int n1,m1,x,y,n,m,i;
+  int x,y,i;
   XTextProperty name;
   char str[100],*sptr;
   static int icon_desk_shown = -1000;
 
   Wait = 0;
-  n1 = Scr.Vx/Scr.MyDisplayWidth;
-  m1 = Scr.Vy/Scr.MyDisplayHeight;
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
-
-  x = (desk_w-n)* Scr.Vx/(Scr.VxMax+Scr.MyDisplayWidth) +n1;
-  y = (desk_h-m)*Scr.Vy/(Scr.VyMax+Scr.MyDisplayHeight) +m1;
+  x = (desk_w)/(Scr.MyDisplayWidth);
+  y = (desk_h)/(Scr.MyDisplayHeight);
   for(i=0;i<ndesks;i++)
     {
       if(i == Scr.CurrentDesk)
@@ -685,30 +667,25 @@ void ReConfigureAll(void)
 void ReConfigureIcons(void)
 {
   PagerWindow *t;
-  int x,y,w,h,n,m,n1,m1;
-
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
+  int x,y,w,h,n1,m1;
 
   t = Start;
   while(t!= NULL)
     {
-      n1 = (Scr.Vx+t->x)/Scr.MyDisplayWidth;
-      m1 = (Scr.Vy+t->y)/Scr.MyDisplayHeight;
-      x = (Scr.Vx + t->x)*(icon_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-      y = (Scr.Vy + t->y)*(icon_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-      w = (Scr.Vx + t->x + t->width+2)*(icon_w-n)/
-	(Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-      h = (Scr.Vy + t->y + t->height+2)*(icon_h-m)/
-	(Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+      n1 = (t->x)/Scr.MyDisplayWidth;
+      m1 = (t->y)/Scr.MyDisplayHeight;
+      x = (t->x)*(icon_w)/(Scr.MyDisplayWidth);
+      y = (t->y)*(icon_h)/(Scr.MyDisplayHeight);
+      w = (t->x + t->width+2)*(icon_w)/(Scr.MyDisplayWidth) - 2 - x;
+      h = (t->y + t->height+2)*(icon_h)/(Scr.MyDisplayHeight) -2 - y;
       
       if (w < 1)
 	w = 1;
       if (h < 1)
 	h = 1;
       
-      t->icon_view_width      = w;
-      t->icon_view_height     = h;
+      t->icon_view_width  = w;
+      t->icon_view_height = h;
       if(Scr.CurrentDesk == t->desk)
 	XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
       else
@@ -733,41 +710,15 @@ void SwitchToDeskAndPage(int Desk, XEvent *Event)
 
   if (Scr.CurrentDesk != (Desk))
     {
-      SendInfo(fd,"Desk 0 10000\n",0);
-      sprintf(command,"GotoPage %d %d\n",
-	      Event->xbutton.x*(Scr.VxMax+Scr.MyDisplayWidth)/
-	      (desk_w*Scr.MyDisplayWidth),
-	      Event->xbutton.y*(Scr.VyMax+Scr.MyDisplayHeight)/
-	      (desk_h*Scr.MyDisplayHeight));
-      SendInfo(fd,command,0);  
-
       sprintf(command,"Desk 0 %d\n",Desk);
       SendInfo(fd,command,0);  
 
     }
-  else
-    {
-      sprintf(command,"GotoPage %d %d\n",
-	      Event->xbutton.x*(Scr.VxMax+Scr.MyDisplayWidth)/
-	      (desk_w*Scr.MyDisplayWidth),
-	      Event->xbutton.y*(Scr.VyMax+Scr.MyDisplayHeight)/
-	      (desk_h*Scr.MyDisplayHeight));
-      SendInfo(fd,command,0);  
-    }
   Wait = 1;
 }
 
 void IconSwitchPage(XEvent *Event)
 {
-  char command[256];
-  
-  sprintf(command,"GotoPage %d %d\n",
-	  Event->xbutton.x*(Scr.VxMax+Scr.MyDisplayWidth)/
-	  (icon_w*Scr.MyDisplayWidth),
-	  Event->xbutton.y*(Scr.VyMax+Scr.MyDisplayHeight)/
-	  (icon_h*Scr.MyDisplayHeight));
-  SendInfo(fd,command,0);  
-  Wait = 1;
 }
 
 
@@ -775,26 +726,22 @@ void AddNewWindow(PagerWindow *t)
 {
   unsigned long valuemask;
   XSetWindowAttributes attributes;
-  int i,x,y,w,h,n,m,n1,m1;
+  int i,x,y,w,h,n1,m1;
 
   i = t->desk;
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
-  n1 = (Scr.Vx+t->x)/Scr.MyDisplayWidth;
-  m1 = (Scr.Vy+t->y)/Scr.MyDisplayHeight;
-  x = (Scr.Vx + t->x)*(desk_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  y = (Scr.Vy + t->y)*(desk_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-  w = (Scr.Vx + t->x + t->width+2)*(desk_w-n)/
-    (Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-  h = (Scr.Vy + t->y + t->height+2)*(desk_h-m)/
-    (Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+  n1 = (t->x)/Scr.MyDisplayWidth;
+  m1 = (t->y)/Scr.MyDisplayHeight;
+  x = (t->x)*(desk_w)/(Scr.MyDisplayWidth);
+  y = (t->y)*(desk_h)/(Scr.MyDisplayHeight);
+  w = (t->x + t->width+2)*(desk_w)/(Scr.MyDisplayWidth) - 2 - x;
+  h = (t->y + t->height+2)*(desk_h)/(Scr.MyDisplayHeight) -2 - y;
   if(w<1)
     w = 1;
   if(h<1)
     h = 1;
 
-  t->pager_view_width		= w;
-  t->pager_view_height	= h;
+  t->pager_view_width = w;
+  t->pager_view_height = h;
   valuemask = (CWBackPixel | CWBorderPixel | CWEventMask);
   attributes.background_pixel = t->back;
   attributes.border_pixel = fore_pix;
@@ -813,19 +760,17 @@ void AddNewWindow(PagerWindow *t)
     t->PagerView = None;
 
 
-  x = (Scr.Vx + t->x)*(icon_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  y = (Scr.Vy + t->y)*(icon_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-  w = (Scr.Vx + t->x + t->width+2)*(icon_w-n)/
-    (Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-  h = (Scr.Vy + t->y + t->height+2)*(icon_h-m)/
-    (Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+  x = (t->x)*(icon_w)/(Scr.MyDisplayWidth);
+  y = (t->y)*(icon_h)/(Scr.MyDisplayHeight);
+  w = (t->x + t->width+2)*(icon_w)/ (Scr.MyDisplayWidth) - 2 - x;
+  h = (t->y + t->height+2)*(icon_h)/(Scr.MyDisplayHeight) -2 - y;
   if(w<1)
     w = 1;
   if(h<1)
     h = 1;
 
-  t->icon_view_width	= w;
-  t->icon_view_height	= h;
+  t->icon_view_width = w;
+  t->icon_view_height = h;
   if(Scr.CurrentDesk == t->desk)
     {
       t->IconView = XCreateWindow(dpy,icon_win, x, y, w, h,1,
@@ -853,7 +798,7 @@ void AddNewWindow(PagerWindow *t)
 
 void ChangeDeskForWindow(PagerWindow *t,long newdesk)
 {
-  int i,x,y,w,h,n,m,n1,m1;
+  int i,x,y,w,h,n1,m1;
 
   i = newdesk;
 
@@ -865,20 +810,18 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
       return;
     }
 
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
-  n1 = (Scr.Vx+t->x)/Scr.MyDisplayWidth;
-  m1 = (Scr.Vy+t->y)/Scr.MyDisplayHeight;
-  x = (Scr.Vx + t->x)*(desk_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  y = (Scr.Vy + t->y)*(desk_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-  w = (Scr.Vx + t->x + t->width+2)*(desk_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-  h = (Scr.Vy + t->y + t->height+2)*(desk_h-m)/(Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+  n1 = (t->x)/Scr.MyDisplayWidth;
+  m1 = (t->y)/Scr.MyDisplayHeight;
+  x = (t->x)*(desk_w)/(Scr.MyDisplayWidth);
+  y = (t->y)*(desk_h)/(Scr.MyDisplayHeight);
+  w = (t->x + t->width+2)*(desk_w)/(Scr.MyDisplayWidth) - 2 - x;
+  h = (t->y + t->height+2)*(desk_h)/(Scr.MyDisplayHeight) -2 - y;
   if (w < 1)
     w = 1;
   if (h < 1)
     h = 1;
-	t->pager_view_width		= w;
-	t->pager_view_height	= h;
+  t->pager_view_width = w;
+  t->pager_view_height = h;
   if((i >= 0)&&(i < ndesks))
     {
       XReparentWindow(dpy, t->PagerView, Desks[i].w, x,y);
@@ -891,16 +834,16 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
     }
   t->desk = i;
 
-  x = (Scr.Vx + t->x)*(icon_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  y = (Scr.Vy + t->y)*(icon_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-  w = (Scr.Vx + t->x + t->width+2)*(icon_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-  h = (Scr.Vy + t->y + t->height+2)*(icon_h-m)/(Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+  x = (t->x)*(icon_w)/(Scr.MyDisplayWidth);
+  y = (t->y)*(icon_h)/(Scr.MyDisplayHeight);
+  w = (t->x + t->width+2)*(icon_w)/ (Scr.MyDisplayWidth) - 2 - x;
+  h = (t->y + t->height+2)*(icon_h)/(Scr.MyDisplayHeight) -2 - y;
   if (w < 1)
     w = 1;
   if (h < 1)
     h = 1;
-	t->icon_view_width	= w;
-	t->icon_view_height	= h;
+  t->icon_view_width = w;
+  t->icon_view_height = h;
   if(Scr.CurrentDesk == t->desk)
     XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
   else
@@ -909,25 +852,20 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
 
 void MoveResizePagerView(PagerWindow *t)
 {
-  int x,y,w,h,n,m,n1,m1;
-
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
-  n1 = (Scr.Vx+t->x)/Scr.MyDisplayWidth;
-  m1 = (Scr.Vy+t->y)/Scr.MyDisplayHeight;
-  x = (Scr.Vx + t->x)*(desk_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  y = (Scr.Vy + t->y)*(desk_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-  w = (Scr.Vx + t->x + t->width+2)*(desk_w-n)/
-    (Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-  h = (Scr.Vy + t->y + t->height+2)*(desk_h-m)/
-    (Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+  int x,y,w,h,n1,m1;
 
+  n1 = (t->x)/Scr.MyDisplayWidth;
+  m1 = (t->y)/Scr.MyDisplayHeight;
+  x = (t->x)*(desk_w)/(Scr.MyDisplayWidth);
+  y = (t->y)*(desk_h)/(Scr.MyDisplayHeight);
+  w = (t->x + t->width+2)*(desk_w)/(Scr.MyDisplayWidth) - 2 - x;
+  h = (t->y + t->height+2)*(desk_h)/(Scr.MyDisplayHeight) -2 - y;
   if (w < 1)
     w = 1;
   if (h < 1)
     h = 1;
-	t->pager_view_width		= w;
-	t->pager_view_height	= h;
+  t->pager_view_width = w;
+  t->pager_view_height = h;
   if(t->PagerView != None)
     XMoveResizeWindow(dpy,t->PagerView,x,y,w,h);
   else if((t->desk >= 0)&&(t->desk <= desk2))
@@ -937,19 +875,17 @@ void MoveResizePagerView(PagerWindow *t)
       return;
     }
     
-  x = (Scr.Vx + t->x)*(icon_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  y = (Scr.Vy + t->y)*(icon_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
-  w = (Scr.Vx + t->x + t->width+2)*(icon_w-n)/
-    (Scr.VxMax + Scr.MyDisplayWidth) - 2 - x + n1;
-  h = (Scr.Vy + t->y + t->height+2)*(icon_h-m)/
-    (Scr.VyMax + Scr.MyDisplayHeight) -2 - y +m1;
+  x = (t->x)*(icon_w)/(Scr.MyDisplayWidth);
+  y = (t->y)*(icon_h)/(Scr.MyDisplayHeight);
+  w = (t->x + t->width+2)*(icon_w)/ (Scr.MyDisplayWidth) - 2 - x;
+  h = (t->y + t->height+2)*(icon_h)/(Scr.MyDisplayHeight) -2 - y;
 
   if (w < 1)
     w = 1;
   if (h < 1)
     h = 1;
-	t->icon_view_width	= w;
-	t->icon_view_height	= h;
+  t->icon_view_width = w;
+  t->icon_view_height = h;
   if(Scr.CurrentDesk == t->desk)
     XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
   else
@@ -1057,9 +993,8 @@ void Scroll(int Desk, int x, int y)
       if(y > desk_h)
 	y = desk_h;
 
-      sx = (100*(x*(Scr.VxMax+Scr.MyDisplayWidth)/desk_w- Scr.Vx))/Scr.MyDisplayWidth;
-      sy = (100*(y*(Scr.VyMax+Scr.MyDisplayHeight)/
-		   desk_h - Scr.Vy))/Scr.MyDisplayHeight;
+      sx = (100*(x*(Scr.MyDisplayWidth)/desk_w))/Scr.MyDisplayWidth;
+      sy = (100*(y*(Scr.MyDisplayHeight)/desk_h))/Scr.MyDisplayHeight;
       if(sx > 100)sx = 100;
       if(sx < -100)sx = -100;
       if(sy < -100)sy = -100;
@@ -1088,9 +1023,8 @@ void IconScroll(int x, int y)
       if(y > icon_h)
 	y = icon_h;
 
-      sx = (100*(x*(Scr.VxMax+Scr.MyDisplayWidth)/icon_w- Scr.Vx))/Scr.MyDisplayWidth;
-      sy = (100*(y*(Scr.VyMax+Scr.MyDisplayHeight)/
-		   icon_h - Scr.Vy))/Scr.MyDisplayHeight;
+      sx = (100*(x*(Scr.MyDisplayWidth)/icon_w))/Scr.MyDisplayWidth;
+      sy = (100*(y*(Scr.MyDisplayHeight)/icon_h))/Scr.MyDisplayHeight;
       if(sx > 100)sx = 100;
       if(sx < -100)sx = -100;
       if(sy < -100)sy = -100;
@@ -1106,10 +1040,10 @@ void IconScroll(int x, int y)
 void MoveWindow(XEvent *Event)
 {
   char command[100];
-  int x1,y1,finished = 0,wx,wy,n,x,y,xi=0,yi=0,wx1,wy1,x2,y2;
+  int x1,y1,finished = 0,wx,wy,x,y,xi=0,yi=0,wx1,wy1,x2,y2;
   Window dumwin;
   PagerWindow *t;
-  int m,n1,m1;
+  int n1,m1;
   int NewDesk,KeepMoving = 0;
   int moved = 0;
   int row,column;
@@ -1137,12 +1071,10 @@ void MoveWindow(XEvent *Event)
   if((NewDesk < 0)||(NewDesk >= ndesks))
     return;
 
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
-  n1 = (Scr.Vx+t->x)/Scr.MyDisplayWidth;
-  m1 = (Scr.Vy+t->y)/Scr.MyDisplayHeight;
-  wx = (Scr.Vx + t->x)*(desk_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  wy = (Scr.Vy + t->y)*(desk_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
+  n1 = (t->x)/Scr.MyDisplayWidth;
+  m1 = (t->y)/Scr.MyDisplayHeight;
+  wx = (t->x)*(desk_w)/(Scr.MyDisplayWidth);
+  wy = (t->y)*(desk_h)/(Scr.MyDisplayHeight);
   wx1 = wx+(desk_w+1)*(NewDesk/Rows);
   wy1 = wy + label_h + (desk_h+label_h+1)*(NewDesk%Rows);
 
@@ -1202,8 +1134,7 @@ void MoveWindow(XEvent *Event)
       NewDesk = Scr.CurrentDesk;
       if(NewDesk != t->desk)
 	{
-	  XMoveWindow(dpy,t->w,Scr.MyDisplayWidth+Scr.VxMax,
-		      Scr.MyDisplayHeight+Scr.VyMax);
+	  XMoveWindow(dpy,t->w,Scr.MyDisplayWidth, Scr.MyDisplayHeight);
 	  XSync(dpy,0);
 	  sprintf(command,"WindowsDesk %d", NewDesk);
 	  SendInfo(fd,command,t->w);
@@ -1247,20 +1178,14 @@ void MoveWindow(XEvent *Event)
       XTranslateCoordinates(dpy, Scr.Pager_w,Desks[NewDesk].w,
 			    x-x1, y-y1, &x2,&y2,&dumwin);
 
-      n1 = x2*(Scr.VxMax + Scr.MyDisplayWidth)/(desk_w * Scr.MyDisplayWidth);
-      m1 = y2*(Scr.VyMax + Scr.MyDisplayHeight)/(desk_h * Scr.MyDisplayHeight);
-      x = (x2-n1)*
-	(Scr.VxMax + Scr.MyDisplayWidth)/(desk_w-n) - Scr.Vx;
-      y = (y2-m1)*
-	(Scr.VyMax + Scr.MyDisplayHeight)/(desk_h-m) - Scr.Vy;
-      if(x + t->frame_width + Scr.Vx < 0 )
-	x = -Scr.Vx;
-      if(y+t->frame_height + Scr.Vy< 0)
-	y = -Scr.Vy;
-      if(x + Scr.Vx > Scr.MyDisplayWidth+Scr.VxMax)
-	x = Scr.MyDisplayWidth + Scr.VxMax - t->frame_width - Scr.Vx;
-      if(y +Scr.Vy> Scr.MyDisplayHeight+Scr.VyMax)
-	y = Scr.MyDisplayHeight+ Scr.VyMax - t->frame_height - Scr.Vy;
+      n1 = x2*(Scr.MyDisplayWidth)/(desk_w * Scr.MyDisplayWidth);
+      m1 = y2*(Scr.MyDisplayHeight)/(desk_h * Scr.MyDisplayHeight);
+      x = x2*(Scr.MyDisplayWidth)/(desk_w);
+      y = y2*(Scr.MyDisplayHeight)/(desk_h);
+      if(x > Scr.MyDisplayWidth)
+	x = Scr.MyDisplayWidth - t->frame_width;
+      if(y > Scr.MyDisplayHeight)
+	y = Scr.MyDisplayHeight - t->frame_height;
       if(((t->flags & ICONIFIED)&&(t->flags & StickyIcon))||
 	 (t->flags & STICKY))
 	{
@@ -1412,21 +1337,19 @@ void LabelIconWindow(PagerWindow *t)
 
 void IconMoveWindow(XEvent *Event,PagerWindow *t)
 {
-  int x1,y1,finished = 0,wx,wy,n,x=0,y=0,xi=0,yi=0;
+  int x1,y1,finished = 0,wx,wy,x=0,y=0,xi=0,yi=0;
   Window dumwin;
-  int m,n1,m1;
+  int n1,m1;
   int moved = 0;
   int KeepMoving = 0;
 
   if(t==NULL)
     return;
 
-  n = (Scr.VxMax)/Scr.MyDisplayWidth;
-  m = (Scr.VyMax)/Scr.MyDisplayHeight;
-  n1 = (Scr.Vx+t->x)/Scr.MyDisplayWidth;
-  m1 = (Scr.Vy+t->y)/Scr.MyDisplayHeight;
-  wx = (Scr.Vx + t->x)*(icon_w-n)/(Scr.VxMax + Scr.MyDisplayWidth) +n1;
-  wy = (Scr.Vy + t->y)*(icon_h-m)/(Scr.VyMax + Scr.MyDisplayHeight)+m1;
+  n1 = (t->x)/Scr.MyDisplayWidth;
+  m1 = (t->y)/Scr.MyDisplayHeight;
+  wx = (t->x)*(icon_w)/(Scr.MyDisplayWidth);
+  wy = (t->y)*(icon_h)/(Scr.MyDisplayHeight);
 
   XRaiseWindow(dpy,t->IconView);
 
@@ -1496,12 +1419,10 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
     {
       x = x - x1;
       y = y - y1;
-      n1 = x*(Scr.VxMax + Scr.MyDisplayWidth)/(icon_w * Scr.MyDisplayWidth);
-      m1 = y*(Scr.VyMax + Scr.MyDisplayHeight)/(icon_h * Scr.MyDisplayHeight);
-      x = (x-n1)*
-	(Scr.VxMax + Scr.MyDisplayWidth)/(icon_w-n) - Scr.Vx;
-      y = (y-m1)*
-	(Scr.VyMax + Scr.MyDisplayHeight)/(icon_h-m) - Scr.Vy;
+      n1 = x*(Scr.MyDisplayWidth)/(icon_w * Scr.MyDisplayWidth);
+      m1 = y*(Scr.MyDisplayHeight)/(icon_h * Scr.MyDisplayHeight);
+      x = x*(Scr.MyDisplayWidth)/(icon_w);
+      y = y*(Scr.MyDisplayHeight)/(icon_h);
       
       if(((t->flags & ICONIFIED)&&(t->flags & StickyIcon))||
 	 (t->flags & STICKY))
