@@ -47,7 +47,7 @@
 
 #include "xfsound_icon.xpm"
 
-#define RCFILE                  ".xfsoundrc"
+#define RCFILE                  "xfsoundrc"
 
 T_messages messages[KNOWN_MESSAGES+KNOWN_BUILTIN] =
 {
@@ -119,32 +119,19 @@ loadcfg (XFSound *s)
   FILE *f = NULL;
 
   strcpy (homedir, (char *) getenv ("HOME"));
-  strcat (homedir, "/");
-  strcat (homedir, RCFILE);
+  strcat (homedir, "/."RCFILE);
   if (existfile (homedir))
     {
       f = fopen (homedir, "r");
     }
   else
     {
-      strcpy (homedir, XFCE_DIR);
-      strcat (homedir, "/");
-      strcat (homedir, RCFILE);
+      strcpy (homedir, XFCE_CONFDIR);
+      strcat (homedir, "/"RCFILE);
       if (existfile (homedir))
 	{
 	  f = fopen (homedir, "r");
 	}
-      else
-        {
-          strcpy (homedir, XFCE_DIR);
-          strcat (homedir, "/");
-          strcat (homedir, "/sample");
-          strcat (homedir, RCFILE);
-          if (existfile (homedir))
-   	    {
-	      f = fopen (homedir, "r");
-	    }
-        }
     }
 
   if (f) 
@@ -172,8 +159,7 @@ savecfg (XFSound *s)
   FILE *f;
 
   strcpy (homedir, (char *) getenv ("HOME"));
-  strcat (homedir, "/");
-  strcat (homedir, RCFILE);
+  strcat (homedir, "/."RCFILE);
 
   if ((f = fopen (homedir, "w")))
     {
@@ -519,28 +505,28 @@ create_xfsound (XFCE_palette *pal)
                               GDK_Escape, 0, GTK_ACCEL_VISIBLE);
   gtk_widget_add_accelerator (cancel_button, "clicked", accel_group,
                               GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
 				 GTK_SIGNAL_FUNC (ok_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
 				 GTK_SIGNAL_FUNC (cancel_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (apply_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (apply_button), "clicked",
 				 GTK_SIGNAL_FUNC (apply_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (play_sound_checkbutton), "clicked",
+  gtk_signal_connect (GTK_OBJECT (play_sound_checkbutton), "clicked",
 				 GTK_SIGNAL_FUNC (doplay_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (external_player_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (external_player_button), "clicked",
 				 GTK_SIGNAL_FUNC (defaultcmd_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (internal_player_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (internal_player_button), "clicked",
 				 GTK_SIGNAL_FUNC (internal_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (browse_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (browse_button), "clicked",
 				 GTK_SIGNAL_FUNC (browsefile_cb),
 				 NULL);
-  gtk_signal_connect_object (GTK_OBJECT (test_button), "clicked",
+  gtk_signal_connect (GTK_OBJECT (test_button), "clicked",
 				 GTK_SIGNAL_FUNC (testfile_cb),
 				 NULL);
   gtk_signal_connect (GTK_OBJECT (xfsound), "delete_event",
