@@ -23,6 +23,7 @@
 #include <string.h>
 #include <glib.h>
 #include "reg.h"
+#include "my_string.h"
 
 static char *regfile = NULL;
 
@@ -31,7 +32,7 @@ static char *regfile = NULL;
 static int
 compare_sfx (reg *reg1, reg *reg2)
 {
-	return strcmp (reg1->sfx, reg2->sfx);
+	return my_strcasecmp (reg1->sfx, reg2->sfx);
 }
 
 void
@@ -199,7 +200,7 @@ reg_app_by_suffix (GList *g_reg, char *sfx)
 	reg *prg;
 	while (g_reg) {
 		prg = g_reg->data;
-		if (strcmp (prg->sfx, sfx) == 0) {
+		if (my_strcasecmp (prg->sfx, sfx) == 0) {
 			return (prg->app);
 		}
 		g_reg = g_reg->next;
@@ -215,7 +216,7 @@ reg_prog_by_suffix (GList *g_reg, char *sfx)
 	reg_t *prg;
 	while (g_reg) {
 		prg = g_reg->data;
-		if (strcmp (prg->sfx, sfx) == 0) {
+		if (my_strcasecmp (prg->sfx, sfx) == 0) {
 			return (prg);
 		}
 		g_reg = g_reg->next;
@@ -241,7 +242,7 @@ reg_app_by_file (GList *g_reg, char *file)
 			g_reg = g_reg->next;
 			continue;
 		}
-		if (strcmp (file+(len - prg->len), prg->sfx) == 0) {
+		if (my_strcasecmp (file+(len - prg->len), prg->sfx) == 0) {
 			return (prg->app);
 		}
 		g_reg = g_reg->next;
@@ -267,7 +268,7 @@ reg_prog_by_file (GList *g_reg, char *file)
 			g_reg = g_reg->next;
 			continue;
 		}
-		if (strcmp (file+(len - prg->len), prg->sfx) == 0) {
+		if (my_strcasecmp (file+(len - prg->len), prg->sfx) == 0) {
 			return (prg);
 		}
 		g_reg = g_reg->next;
@@ -278,7 +279,7 @@ reg_prog_by_file (GList *g_reg, char *file)
 /*
  */
 int
-mystrcmp (char *s1, char *s2) {
+mymy_strcasecmp (char *s1, char *s2) {
 	if ((s1 == NULL) || (s2 == NULL)) {
 		if (s1 == s2)
 			return 0;
@@ -287,7 +288,7 @@ mystrcmp (char *s1, char *s2) {
 		else
 			return -1;
 	}
-	return (strcmp (s1, s2));
+	return (my_strcasecmp (s1, s2));
 }
 
 /*
@@ -303,7 +304,7 @@ reg_add_suffix (GList *g_reg, char *sfx, char *program, char *args)
 	 */
 	while (g_tmp) {
 		prg = g_tmp->data;
-		if ((strcmp (prg->sfx, sfx) == 0) && (mystrcmp(prg->arg, args) == 0)) {
+		if ((my_strcasecmp (prg->sfx, sfx) == 0) && (mymy_strcasecmp(prg->arg, args) == 0)) {
 			found = 1;
 			break;
 		}
@@ -398,12 +399,12 @@ reg_app_list (GList *g_reg)
 		g_apps = g_list_append (g_apps, ((reg *)g_reg->data)->app);
 		g_reg = g_reg->next;
 	}
-	g_apps = g_list_sort (g_apps, (GCompareFunc)strcmp);
+	g_apps = g_list_sort (g_apps, (GCompareFunc)my_strcasecmp);
 	/* remove dupes */
 	g_tmp = g_apps;
 	while (g_tmp) {
 		if (g_tmp->next) {
-			if (strcmp ((char*)g_tmp->data, (char*)g_tmp->next->data) == 0) {
+			if (my_strcasecmp ((char*)g_tmp->data, (char*)g_tmp->next->data) == 0) {
 				g_tmp = g_apps = g_list_remove (g_apps, g_tmp->data);
 				continue;
 			}
