@@ -43,12 +43,14 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include "xfwm.h"
 #include "misc.h"
 #include "screen.h"
-#include <X11/extensions/shape.h>
 #include <X11/Xresource.h>
+#include <X11/extensions/shape.h>
 #include "module.h"
+#include "session.h"
 
 /* Used to parse command line of clients for specific desk requests. */
 /* Todo: check for multiple desks. */
@@ -88,12 +90,11 @@ AddWindow (Window w)
   XfwmWindow *tmp_win;		/* new xfwm window structure */
   unsigned long valuemask;	/* mask for create windows */
   XSetWindowAttributes attributes;	/* attributes for create windows */
-  int i, width, height;
+  int i, x, y, width, height;
   int a, b;
   char *value;
   unsigned long tflag;
   int Desk, border_width;
-  extern Bool NeedToResizeToo;
   extern XfwmWindow *colormap_win;
   char *forecolor = NULL, *backcolor = NULL;
   int client_argc;
@@ -102,9 +103,12 @@ AddWindow (Window w)
   XrmValue rm_value;
   XTextProperty text_prop;
   extern Bool PPosOverride;
+  Atom atype;
+  int aformat;
+  unsigned long nitems, bytes_remain;
+  unsigned char *prop;
 
   XSync (dpy, 0);
-  NeedToResizeToo = False;
   /* allocate space for the xfwm window */
   tmp_win = (XfwmWindow *) calloc (1, sizeof (XfwmWindow));
   if (tmp_win == (XfwmWindow *) 0)
@@ -112,7 +116,6 @@ AddWindow (Window w)
       return NULL;
     }
   tmp_win->flags = 0;
-  tmp_win->flags = 0;
   tmp_win->w = w;
   tmp_win->icon_arranged = False;
   
@@ -195,10 +198,10 @@ AddWindow (Window w)
   GetExtHints (tmp_win);
   GetKDEHints (tmp_win);
 
-
   SelectDecor (tmp_win, tflag, border_width);
 
   tmp_win->flags |= tflag & ALL_COMMON_FLAGS;
+
   /* find a suitable icon pixmap */
   if (tflag & ICON_FLAG)
     {
@@ -244,6 +247,43 @@ AddWindow (Window w)
   if (!PlaceWindow (tmp_win, tflag, Desk))
     return NULL;
 
+  /* Load Session Management saved values */
+  MatchWinToSM(tmp_win);
+
+  /* If restarting, retrieve parameters saved as Atoms */
+ if ((XGetWindowProperty (dpy, tmp_win->w, _XA_XFWM_FLAGS, 0L, 1L, True,
+                          _XA_XFWM_FLAGS, &atype, &aformat, &nitems,
+                          &bytes_remain, &prop)) == Success)
+   {
+     if (prop != NULL)
+       {
+         tmp_win->flags |= (*(unsigned long *) prop) & (STICKY | ICON_MOVED | SHADED);
+         XFree (prop);
+       }
+   }
+  
+ if ((XGetWindowProperty (dpy, tmp_win->w, _XA_XFWM_ICONPOS_X, 0L, 1L, True,
+                          _XA_XFWM_ICONPOS_X, &atype, &aformat, &nitems,
+                          &bytes_remain, &prop)) == Success)
+   {
+     if (prop != NULL)
+       {
+         tmp_win->icon_x_loc = *(int *) prop;
+         XFree (prop);
+       }
+   }
+
+ if ((XGetWindowProperty (dpy, tmp_win->w, _XA_XFWM_ICONPOS_Y, 0L, 1L, True,
+                          _XA_XFWM_ICONPOS_Y, &atype, &aformat, &nitems,
+                          &bytes_remain, &prop)) == Success)
+   {
+     if (prop != NULL)
+       {
+         tmp_win->icon_y_loc = *(int *) prop;
+         XFree (prop);
+       }
+   }
+
   /*
    * Make sure the client window still exists.  We don't want to leave an
    * orphan frame window if it doesn't.  Since we now have the server
@@ -251,13 +291,12 @@ AddWindow (Window w)
    * reparented, so we'll get a DestroyNotify for it.  We won't have
    * gotten one for anything up to here, however.
    */
-  MyXGrabServer (dpy);
+  XSync (dpy, 0);
   if (XGetGeometry (dpy, w, &JunkRoot, &JunkX, &JunkY,
 		    &JunkWidth, &JunkHeight,
 		    &JunkBW, &JunkDepth) == 0)
     {
       free ((char *) tmp_win);
-      MyXUngrabServer (dpy);
       return (NULL);
     }
 
@@ -295,8 +334,6 @@ AddWindow (Window w)
   tmp_win->flags &= ~ICONIFIED;
   tmp_win->flags &= ~ICON_UNMAPPED;
   tmp_win->flags &= ~MAXIMIZED;
-  tmp_win->flags &= ~SHADED;
-
 
   tmp_win->TextPixel = Scr.MenuColors.fore;
 
@@ -311,6 +348,8 @@ AddWindow (Window w)
 	}
     }
   tmp_win->BackPixel = GetDecor (tmp_win, LoColors.back);
+  attributes.background_pixel = tmp_win->BackPixel;
+  valuemask = CWBackPixel;
 
   /* add the window into the xfwm list */
   tmp_win->next = Scr.XfwmRoot.next;
@@ -326,54 +365,33 @@ AddWindow (Window w)
   tmp_win->frame_width = tmp_win->attr.width + 2 * tmp_win->boundary_width;
   tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height +
     2 * tmp_win->boundary_width;
+  tmp_win->shade_x = tmp_win->frame_x;
+  tmp_win->shade_y = tmp_win->frame_y;
+  tmp_win->shade_width = tmp_win->frame_width;
+  tmp_win->shade_height = tmp_win->title_height + 2 * tmp_win->boundary_width;
   ConstrainSize (tmp_win, &tmp_win->frame_width, &tmp_win->frame_height);
 
-  valuemask = CWCursor | CWEventMask;
+  valuemask |= CWCursor | CWEventMask;
   attributes.cursor = Scr.XfwmCursors[DEFAULT];
-  attributes.event_mask = (SubstructureRedirectMask | ButtonPressMask |
-			   ButtonReleaseMask | EnterWindowMask |
-			   LeaveWindowMask | ExposureMask);
+  attributes.event_mask = (SubstructureRedirectMask | 
+  		           ButtonPressMask | ButtonReleaseMask | 
+                           EnterWindowMask | LeaveWindowMask | 
+                           ExposureMask);
 
-  /* What the heck, we'll always reparent everything from now on! */
   tmp_win->frame =
     XCreateWindow (dpy, Scr.Root, tmp_win->frame_x, tmp_win->frame_y,
 		   tmp_win->frame_width, tmp_win->frame_height,
 		   tmp_win->bw, CopyFromParent, InputOutput,
 		   CopyFromParent,
-		   valuemask,
+		   (valuemask & ~CWBackPixel),
 		   &attributes);
 
   attributes.save_under = FALSE;
-
-  /* Thats not all, we'll double-reparent the window ! */
-  attributes.cursor = Scr.XfwmCursors[DEFAULT];
-  tmp_win->Parent =
-    XCreateWindow (dpy, tmp_win->frame,
-		   tmp_win->boundary_width,
-		   tmp_win->boundary_width + tmp_win->title_height,
-		   (tmp_win->frame_width - 2 * tmp_win->boundary_width),
-		   (tmp_win->frame_height - 2 * tmp_win->boundary_width -
-		    tmp_win->title_height), tmp_win->bw, CopyFromParent,
-		   InputOutput, CopyFromParent, valuemask, &attributes);
+  attributes.backing_store = WhenMapped;
+  attributes.event_mask = (ButtonPressMask | ButtonReleaseMask | 
+                           ExposureMask);
 
   valuemask |= CWBackingStore;
-  if (Scr.d_depth < 2)
-    {
-      attributes.background_pixmap = Scr.light_gray_pixmap;
-      if (tmp_win->flags & STICKY)
-	attributes.background_pixmap = Scr.sticky_gray_pixmap;
-      valuemask |= CWBackPixmap;
-    }
-  else
-    {
-      attributes.background_pixel = tmp_win->BackPixel;
-      /* attributes.background_pixmap = ParentRelative; */
-      valuemask |= CWBackPixel /* | CWBackPixmap */;
-    }
-
-  attributes.backing_store = WhenMapped;
-  attributes.event_mask = (ButtonPressMask | ButtonReleaseMask | ExposureMask |
-			   EnterWindowMask | LeaveWindowMask);
   tmp_win->title_x = tmp_win->title_y = 0;
   tmp_win->title_w = 0;
   tmp_win->title_width = tmp_win->frame_width - 2 * tmp_win->corner_width
@@ -406,9 +424,9 @@ AddWindow (Window w)
 	XCreateWindow (dpy, tmp_win->frame, tmp_win->title_x, tmp_win->title_y,
 		       tmp_win->title_width, tmp_win->title_height, 0,
 		       CopyFromParent, InputOutput, CopyFromParent,
-		       valuemask, &attributes);
+		       (valuemask & ~CWBackPixel), &attributes);
       attributes.cursor = Scr.XfwmCursors[SYS];
-      for (i = 1; i >= 0; i--)
+      for (i = 2; i >= 0; i--)
 	{
 	  if ((i < Scr.nr_left_buttons) && (tmp_win->left_w[i] > 0))
 	    {
@@ -450,23 +468,25 @@ AddWindow (Window w)
 	    XCreateWindow (dpy, tmp_win->frame, 0, 0, tmp_win->boundary_width,
 			   tmp_win->boundary_width, 0, CopyFromParent,
 			   InputOutput, CopyFromParent,
-			   valuemask,
+			   (valuemask & ~CWBackPixel),
 			   &attributes);
 	}
     }
 
 
   XMapSubwindows (dpy, tmp_win->frame);
-  XRaiseWindow (dpy, tmp_win->Parent);
-  XReparentWindow (dpy, tmp_win->w, tmp_win->Parent, 0, 0);
+
+  XReparentWindow (dpy, tmp_win->w, tmp_win->frame, 0, 0);
 
   valuemask = (CWEventMask | CWDontPropagate | CWBackingStore);
-  attributes.event_mask = (StructureNotifyMask | PropertyChangeMask |
-			   VisibilityChangeMask | EnterWindowMask |
-			   LeaveWindowMask |
-			   ColormapChangeMask | FocusChangeMask);
+  attributes.event_mask = (StructureNotifyMask | 
+  		           PropertyChangeMask |
+			   VisibilityChangeMask | 
+			   ColormapChangeMask | 
+                           EnterWindowMask | LeaveWindowMask | 
+                           FocusChangeMask);
+
   attributes.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask;
-  attributes.backing_store = WhenMapped;
 
   XChangeWindowAttributes (dpy, tmp_win->w, valuemask, &attributes);
   tmp_win->name = NoName;
@@ -502,11 +522,15 @@ AddWindow (Window w)
    * again in HandleMapNotify.
    */
   tmp_win->flags &= ~MAPPED;
+  x = tmp_win->frame_x;
+  tmp_win->frame_x = 0;
+  y = tmp_win->frame_y;
+  tmp_win->frame_y = 0;
   width = tmp_win->frame_width;
   tmp_win->frame_width = 0;
   height = tmp_win->frame_height;
   tmp_win->frame_height = 0;
-  SetupFrame (tmp_win, tmp_win->frame_x, tmp_win->frame_y, width, height, TRUE, TRUE);
+  SetupFrame (tmp_win, x, y, width, height, TRUE, TRUE);
 
   /* wait until the window is iconified and the icon window is mapped
    * before creating the icon window 
@@ -516,7 +540,6 @@ AddWindow (Window w)
   GrabKeys (tmp_win);
   XSaveContext (dpy, tmp_win->w, XfwmContext, (caddr_t) tmp_win);
   XSaveContext (dpy, tmp_win->frame, XfwmContext, (caddr_t) tmp_win);
-  XSaveContext (dpy, tmp_win->Parent, XfwmContext, (caddr_t) tmp_win);
   if (tmp_win->flags & TITLE)
     {
       XSaveContext (dpy, tmp_win->title_w, XfwmContext, (caddr_t) tmp_win);
@@ -535,9 +558,9 @@ AddWindow (Window w)
 	  XSaveContext (dpy, tmp_win->corners[i], XfwmContext, (caddr_t) tmp_win);
 	}
     }
+
   RaiseWindow (tmp_win);
   KeepOnTop();
-  MyXUngrabServer (dpy);
 
   XGetGeometry (dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
 		&JunkWidth, &JunkHeight, &JunkBW, &JunkDepth);
@@ -545,21 +568,14 @@ AddWindow (Window w)
 			 &a, &b, &JunkChild);
   tmp_win->xdiff -= a;
   tmp_win->ydiff -= b;
-  if(Scr.Options & (ClickToFocus | AutoRaiseWin))
-    {
-       /* need to grab all buttons for window that we are about to
-	* * unhighlight */
-       for (i = 0; i < 3; i++)
-	 if (Scr.buttons2grab & (1 << i))
-	   {
-	     XGrabButton (dpy, (i + 1), 0, tmp_win->frame, True,
-			  ButtonPressMask, GrabModeSync, GrabModeAsync, None,
-			  Scr.XfwmCursors[SYS]);
-	     XGrabButton (dpy, (i + 1), AnyModifier, tmp_win->frame, True,
-			  ButtonPressMask, GrabModeSync, GrabModeAsync, None,
-			  Scr.XfwmCursors[SYS]);
-	   }
-    }
+  
+  XGrabButton (dpy, AnyButton, 0, tmp_win->frame, True,
+               ButtonPressMask, GrabModeSync, GrabModeAsync, None,
+               None);
+  XGrabButton (dpy, AnyButton, AnyModifier, tmp_win->frame, True,
+               ButtonPressMask, GrabModeSync, GrabModeAsync, None,
+               None);
+  
   BroadcastConfig (M_ADD_WINDOW, tmp_win);
 
   BroadcastName (M_WINDOW_NAME, tmp_win->w, tmp_win->frame,
@@ -579,23 +595,8 @@ AddWindow (Window w)
   FetchWmColormapWindows (tmp_win);
   if (!(XGetWindowAttributes (dpy, tmp_win->w, &(tmp_win->attr))))
     tmp_win->attr.colormap = Scr.XfwmRoot.attr.colormap;
-  if (NeedToResizeToo)
-    {
-      XWarpPointer (dpy, Scr.Root, Scr.Root, 0, 0, Scr.MyDisplayWidth,
-		    Scr.MyDisplayHeight,
-		    tmp_win->frame_x + (tmp_win->frame_width >> 1),
-		    tmp_win->frame_y + (tmp_win->frame_height >> 1));
-      Event.xany.type = ButtonPress;
-      Event.xbutton.button = 1;
-      Event.xbutton.x_root = tmp_win->frame_x + (tmp_win->frame_width >> 1);
-      Event.xbutton.y_root = tmp_win->frame_y + (tmp_win->frame_height >> 1);
-      Event.xbutton.x = (tmp_win->frame_width >> 1);
-      Event.xbutton.y = (tmp_win->frame_height >> 1);
-      Event.xbutton.subwindow = None;
-      Event.xany.window = tmp_win->w;
-      resize_window (&Event, tmp_win->w, tmp_win, C_WINDOW, "", 0);
-    }
   InstallWindowColormaps (colormap_win);
+
   XSync(dpy,0);
   return (tmp_win);
 }
@@ -626,25 +627,15 @@ GrabButtons (XfwmWindow * tmp_win)
 			   tmp_win->w,
 			   True, ButtonPressMask | ButtonReleaseMask,
 			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
+			   None);
 	    }
 	  else
 	    {
-	      XGrabButton (dpy, 1, MouseEntry->Modifier,
+	      XGrabButton (dpy, AnyButton, MouseEntry->Modifier,
 			   tmp_win->w,
 			   True, ButtonPressMask | ButtonReleaseMask,
 			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
-	      XGrabButton (dpy, 2, MouseEntry->Modifier,
-			   tmp_win->w,
-			   True, ButtonPressMask | ButtonReleaseMask,
-			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
-	      XGrabButton (dpy, 3, MouseEntry->Modifier,
-			   tmp_win->w,
-			   True, ButtonPressMask | ButtonReleaseMask,
-			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
+			   None);
 	    }
 	}
       MouseEntry = MouseEntry->NextBinding;
