@@ -45,6 +45,13 @@ void DrawLinePattern (Window win,
 		      struct vector_coords *coords,
 		      int w, int h);
 
+void
+RelieveRoundedRectangle (Window win, 
+                         int x, 
+                         int y, 
+                         int w, 
+                         int h, GC Hilite, GC Shadow);
+
 #define GetButtonState(window) (onoroff ? Active : Inactive)
 
 /* macro to change window background color/pixmap */
@@ -71,7 +78,7 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
   int y, i, x, fh;
   GC ReliefGC, ShadowGC;
   Pixel BorderColor, BackColor;
-  Pixmap BackPixmap, TextColor;
+  Pixmap TextColor;
   Bool NewColor = False;
   XSetWindowAttributes attributes;
   unsigned long valuemask;
@@ -93,10 +100,6 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
 
   if (onoroff)
     {
-
-      if ((!force) && (Scr.Hilite == t))
-	return;
-
       if ((Scr.Hilite != t) || (force))
 	NewColor = True;
 
@@ -113,7 +116,6 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
       Scr.Hilite = t;
 
       TextColor = GetDecor (t, HiColors.fore);
-      BackPixmap = Scr.gray_pixmap;
       BackColor = GetDecor (t, HiColors.back);
       ReliefGC = GetDecor (t, HiReliefGC);
       ShadowGC = GetDecor (t, HiShadowGC);
@@ -122,9 +124,6 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
   else
     {
 
-      if ((!force) && (Scr.Hilite != t))
-	return;
-
       if (Scr.Hilite == t)
 	{
 	  Scr.Hilite = NULL;
@@ -133,7 +132,6 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
       if (force)
 	NewColor = True;
       TextColor = GetDecor (t, LoColors.fore);
-      BackPixmap = Scr.light_gray_pixmap;
       BackColor = GetDecor (t, LoColors.back);
       ReliefGC = GetDecor (t, LoReliefGC);
       ShadowGC = GetDecor (t, LoShadowGC);
@@ -147,21 +145,12 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
     }
   valuemask = CWBorderPixel;
   attributes.border_pixel = BorderColor;
-  if (Scr.d_depth < 2)
-    {
-      attributes.background_pixmap = BackPixmap;
-      valuemask |= CWBackPixmap;
-    }
-  else
-    {
-      attributes.background_pixel = BackColor;
-      /* attributes.background_pixmap = ParentRelative; */
-      valuemask |= CWBackPixel /* | CWBackPixmap */;
-    }
+
+  attributes.background_pixel = BackColor;
+  valuemask |= CWBackPixel;
 
   if (t->flags & TITLE)
     {
-      XChangeWindowAttributes (dpy, t->title_w, valuemask, &attributes);
       for (i = 0; i < Scr.nr_left_buttons; ++i)
 	{
 	  if (t->left_w[i] != None)
@@ -170,12 +159,9 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
 	      ButtonFace *bf = &GetDecor (t, left_buttons[i].state[bs]);
 	      ChangeWindowColor (t->left_w[i], valuemask);
 	      if (flush_expose (t->left_w[i]) || (expose_win == t->left_w[i]) ||
-		  (expose_win == None)
-		  || NewColor
-		)
+		  (expose_win == None) || NewColor)
 		{
 		  int inverted = PressedW == t->left_w[i];
-		  XClearWindow (dpy, t->left_w[i]);
 		  if (bf->style & UseTitleStyle)
 		    {
 		      ButtonFace *tsbf = &GetDecor (t, titlebar.state[bs]);
@@ -202,12 +188,9 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
 	      ButtonFace *bf = &GetDecor (t, right_buttons[i].state[bs]);
 	      ChangeWindowColor (t->right_w[i], valuemask);
 	      if (flush_expose (t->right_w[i]) || (expose_win == t->right_w[i]) ||
-		  (expose_win == None)
-		  || NewColor
-		)
+		  (expose_win == None) || NewColor)
 		{
 		  int inverted = PressedW == t->right_w[i];
-		  XClearWindow (dpy, t->right_w[i]);
 		  if (bf->style & UseTitleStyle)
 		    {
 		      ButtonFace *tsbf = &GetDecor (t, titlebar.state[bs]);
@@ -226,7 +209,8 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
 		}
 	    }
 	}
-      SetTitleBar (t, onoroff, False);
+     if (flush_expose (t->title_w) || (expose_win ==t->title_w) || (expose_win == None) || NewColor)
+       SetTitleBar (t, onoroff, False);
     }
 
   if (t->flags & BORDER)
@@ -237,7 +221,6 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
        for (i = 0; i < 4; i++)
          {
            int vertical = i % 2;
-           ChangeWindowColor (t->sides[i], valuemask);
            if ((flush_expose (t->sides[i])) || (expose_win == t->sides[i]) ||
      	       (expose_win == None) || NewColor)
      	     {
@@ -248,8 +231,7 @@ SetBorder (XfwmWindow * t, Bool onoroff, Bool force, Bool Mapped,
      	       RelieveWindow (t, t->sides[i], 0, 0,
      			      ((vertical) ? t->boundary_width : x),
      			      ((vertical) ? y : t->boundary_width),
-     			      rgc, sgc, (0x0001 << i)
-     	         );
+     			      rgc, sgc, (0x0001 << i));
      	     }
            ChangeWindowColor (t->corners[i], valuemask);
            if ((flush_expose (t->corners[i])) || (expose_win == t->corners[i]) ||
@@ -278,7 +260,10 @@ DrawButton (XfwmWindow * t, Window win, int w, int h,
 	    ButtonFace * bf, GC ReliefGC, GC ShadowGC,
 	    Boolean inverted, int stateflags)
 {
-  register int type = bf->style & ButtonFaceTypeMask;
+  int type = bf->style & ButtonFaceTypeMask;
+  GC BackGC = NULL;
+  
+  if (w - t->boundary_width + t->bw  <= 0) return;
 
   switch (type)
     {
@@ -286,29 +271,74 @@ DrawButton (XfwmWindow * t, Window win, int w, int h,
       break;
 
     case SolidButton:
-      XSetWindowBackground (dpy, win, bf->u.back);
-      flush_expose (win);
-      XClearWindow (dpy, win);
+      {
+	XRectangle bounds;
+	bounds.x = bounds.y = 0;
+	bounds.width = w;
+	bounds.height = h;
+	flush_expose (win);
+        BackGC = ((Scr.Hilite == t) ? GetDecor (t, HiBackGC) : GetDecor (t, LoBackGC));
+        
+        XSetForeground (dpy, Scr.TransMaskGC, bf->u.back);
+        if (w > h)
+          {
+            XFillRectangle (dpy, win, BackGC,0, 0, bounds.height / 2, bounds.height);
+	      
+            XFillArc (dpy, win, Scr.TransMaskGC, 0, 
+                                                 0, 
+                                                 bounds.height, 
+                                                 bounds.height - 1,
+                                                 90 * 64, 180 * 64);
+            XFillRectangle (dpy, win, Scr.TransMaskGC,
+			      bounds.height / 2 , 1,
+			      bounds.width  - bounds.height, bounds.height - 1);
+
+            XFillRectangle (dpy, win, BackGC, bounds.width  - (bounds.height / 2), 
+		                                  0, bounds.height / 2, bounds.height);
+
+            XFillArc (dpy, win, Scr.TransMaskGC, bounds.width  - bounds.height - 1, 
+                                                 0, 
+                                                 bounds.height, 
+                                                 bounds.height - 1,
+                                                 90 * 64,  - 180 * 64);
+          }
+        else
+          XFillRectangle (dpy, win, Scr.TransMaskGC,0 , 0, w, h - 1);
+      }
       break;
 
     case VectorButton:
-      if ((stateflags & MWMDecorMaximize)
-	  && (t->flags & MAXIMIZED))
-	DrawLinePattern (win,
-			 ShadowGC, ReliefGC,
-			 &bf->vector,
-			 w, h);
-      else if ((stateflags & MWMDecorSticky)
-	       && (t->flags & STICKY))
-	DrawLinePattern (win,
-			 ShadowGC, ReliefGC,
-			 &bf->vector,
-			 w, h);
+      BackGC = ((Scr.Hilite == t) ? GetDecor (t, HiBackGC) : GetDecor (t, LoBackGC));
+      if (((stateflags & MWMDecorMaximize) && (t->flags & MAXIMIZED)) ||
+          ((stateflags & DecorSticky) && (t->flags & STICKY)) ||
+          ((stateflags & DecorShaded) && (t->flags & SHADED)))
+        {
+          if (Scr.Options & BitmapButtons)
+            {
+              XSetClipOrigin(dpy, BackGC, (w - 16) / 2, (h - 16) / 2);
+              XCopyPlane(dpy, bf->bitmap_pressed, win, BackGC, 0, 0, 15, 15, (w - 16) / 2, (h - 16) / 2, 1);
+            }
+          else
+            {
+  	      DrawLinePattern (win, ShadowGC, ReliefGC, &bf->vector, w, h);
+            }
+        }
       else
-	DrawLinePattern (win,
-			 ReliefGC, ShadowGC,
-			 &bf->vector,
-			 w, h);
+        {
+          if (Scr.Options & BitmapButtons)
+            {
+              XSetClipOrigin(dpy, BackGC, (w - 16) / 2, (h - 16) / 2);
+              XCopyPlane(dpy, bf->bitmap, win, BackGC, 0, 0, 15, 15, (w - 16) / 2, (h - 16) / 2, 1);
+            }
+          else
+            {
+	      DrawLinePattern (win, ReliefGC, ShadowGC, &bf->vector, w, h);
+            }
+        }
+
+      if (inverted)
+         RelieveRectangle (win, 0, 0, w - 1, h - 1, ShadowGC, ReliefGC);
+      
       break;
 
     case GradButton:
@@ -318,23 +348,46 @@ DrawButton (XfwmWindow * t, Window win, int w, int h,
 	bounds.width = w;
 	bounds.height = h;
 	flush_expose (win);
+        BackGC = ((Scr.Hilite == t) ? GetDecor (t, HiBackGC) : GetDecor (t, LoBackGC));
 
 	{
-	  register int i = 0, dw = bounds.width / bf->u.grad.npixels + 1;
+	  int i = 0, dw = ((w > h) ? bounds.width : 0) / bf->u.grad.npixels + 1;
+	  XSetForeground (dpy, Scr.TransMaskGC, bf->u.grad.pixels[i]);
+          if (w > h)
+            {
+              XFillRectangle (dpy, win, BackGC,0, 0, bounds.height / 2, bounds.height);
+
+              XFillArc (dpy, win, Scr.TransMaskGC, 0, 
+                                                   0, 
+                                                   bounds.height, 
+                                                   bounds.height - 1,
+                                                   90 * 64, 180 * 64);
+            }
 	  while (i < bf->u.grad.npixels)
 	    {
-	      register unsigned short x = i * bounds.width / bf->u.grad.npixels;
+	      unsigned short x = ((w > h) ? (bounds.height / 2) : 0) 
+                                          + i * (bounds.width - ((w > h) ? bounds.height : 0))/ bf->u.grad.npixels;
 	      XSetForeground (dpy, Scr.TransMaskGC, bf->u.grad.pixels[i++]);
 	      XFillRectangle (dpy, win, Scr.TransMaskGC,
-			      bounds.x + x, bounds.y,
-			      dw, bounds.height);
+			      bounds.x + x, bounds.y + 1,
+			      dw, bounds.height - 2);
 	    }
+          if (w > h)
+            {
+              XFillRectangle (dpy, win, BackGC, bounds.width - (bounds.height / 2), 
+		                                  0, bounds.height / 2, bounds.height);
+
+              XFillArc (dpy, win, Scr.TransMaskGC, bounds.width  - bounds.height - 1, 
+                                                   0, 
+                                                   bounds.height, 
+                                                   bounds.height - 1,
+                                                   90 * 64,  - 180 * 64);
+            }
 	}
       }
       break;
 
     default:
-      xfwm_msg (ERR, "DrawButton", "unknown button type");
       break;
     }
 }
@@ -353,6 +406,7 @@ SetTitleBar (XfwmWindow * t, Bool onoroff, Bool NewTitle)
   int tb_flags;
   GC ReliefGC, ShadowGC;
   Pixel Forecolor, BackColor;
+  ButtonFace *bf;
 
   if (!t)
     return;
@@ -409,60 +463,46 @@ SetTitleBar (XfwmWindow * t, Bool onoroff, Bool NewTitle)
       NewFontAndColor (0, Forecolor, BackColor);
     }
 
-  /* the next bit tries to minimize redraw based upon compilation options (veliaa@rpi.edu) */
-  /* for mono, we clear an area in the title bar where the window
-   * title goes, so that its more legible. For color, no need */
-  if (Scr.d_depth < 2)
+  bf = &GetDecor (t, titlebar.state[title_state]);
+  /* draw compound titlebar (veliaa@rpi.edu) */
+  for (; bf; bf = bf->next)
+    DrawButton (t, t->title_w, t->title_width, t->title_height,
+    		bf, ShadowGC, ReliefGC, True, 0);
+  if (t->name != (char *) NULL)
     {
-      RelieveWindow (t, t->title_w, 0, 0, hor_off - 2, t->title_height,
-		     ReliefGC, ShadowGC, BOTTOM_HILITE);
-      RelieveWindow (t, t->title_w, hor_off + w + 2, 0,
-		     t->title_width - w - hor_off - 2, t->title_height,
-		     ReliefGC, ShadowGC, BOTTOM_HILITE);
-      XFillRectangle (dpy, t->title_w,
-		      (PressedW == t->title_w ? ShadowGC : ReliefGC),
-		      hor_off - 2, 0, w + 4, t->title_height);
+      XFontSet fontset = GetDecor (t, WindowFont.fontset);
+      if (fontset)
+  	XmbDrawString (dpy, t->title_w, fontset, Scr.ScratchGC3, hor_off,
+  		     GetDecor (t, WindowFont.y) + 1,
+  		     t->name, strlen (t->name));
+      else
+    	XDrawString (dpy, t->title_w, Scr.ScratchGC3, hor_off,
+    		     GetDecor (t, WindowFont.y) + 1,
+    		     t->name, strlen (t->name));
+    }
+  RelieveRoundedRectangle (t->title_w, 0, 0, t->title_width, t->title_height,
+    		      ShadowGC, ReliefGC);
+  if (!onoroff)
+    RelieveRoundedRectangle (t->title_w, 1, 1, t->title_width-2, t->title_height-2,
+    		      ReliefGC, ShadowGC);
+}
 
-      XDrawLine (dpy, t->title_w, ShadowGC, hor_off + w + 1, 0, hor_off + w + 1,
-		 t->title_height);
-      if (t->name != (char *) NULL)
-        {
-          XFontSet fontset = GetDecor (t, WindowFont.fontset);
-          if (fontset)
-            XmbDrawString (dpy, t->title_w, fontset, Scr.ScratchGC3, hor_off,
-                         GetDecor (t, WindowFont.y) + 1,
-                         t->name, strlen (t->name));
-          else
-	    XDrawString (dpy, t->title_w, Scr.ScratchGC3, hor_off,
-		         GetDecor (t, WindowFont.y) + 1,
-		         t->name, strlen (t->name));
-        }
-    }
+void
+RelieveRoundedRectangle (Window win, int x, int y, int w, int h, GC Hilite, GC Shadow)
+{
+  if (w <= 0) return;
+  if (w > h)
+   {
+      XDrawLine (dpy, win, Hilite, x + h / 2, y, w + x - h / 2, y);
+      XDrawArc  (dpy, win, Hilite, x, y, h, h - 1, 90 * 64, 135 * 64);
+      XDrawArc  (dpy, win, Shadow, x, y, h, h - 1, 270 * 64, -45 * 64);
+
+      XDrawLine (dpy, win, Shadow, x + h / 2, h + y - 1, w + x - h / 2, h + y - 1);
+      XDrawArc  (dpy, win, Shadow, w + x - h - 1, y, h, h - 1, 270 * 64, 135 * 64);
+      XDrawArc  (dpy, win, Hilite, w + x - h - 1, y, h, h - 1, 90 * 64,  - 45 * 64);
+   }
   else
-    {
-      ButtonFace *bf = &GetDecor (t, titlebar.state[title_state]);
-      /* draw compound titlebar (veliaa@rpi.edu) */
-      for (; bf; bf = bf->next)
-	DrawButton (t, t->title_w, t->title_width, t->title_height,
-		    bf, ShadowGC, ReliefGC, True, 0);
-      if (t->name != (char *) NULL)
-        {
-          XFontSet fontset = GetDecor (t, WindowFont.fontset);
-          if (fontset)
-            XmbDrawString (dpy, t->title_w, fontset, Scr.ScratchGC3, hor_off,
-                         GetDecor (t, WindowFont.y) + 1,
-                         t->name, strlen (t->name));
-          else
-	    XDrawString (dpy, t->title_w, Scr.ScratchGC3, hor_off,
-		         GetDecor (t, WindowFont.y) + 1,
-		         t->name, strlen (t->name));
-        }
-      RelieveRectangle (t->title_w, 0, 0, t->title_width, t->title_height,
-			  ShadowGC, ReliefGC);
-      if (!onoroff)
-	RelieveRectangle (t->title_w, 1, 1, t->title_width-2, t->title_height-2,
-			  ReliefGC, ShadowGC);
-    }
+    RelieveRectangle (win, x, y, w, h, Hilite, Shadow);
 }
 
 /****************************************************************************
@@ -478,14 +518,15 @@ DrawLinePattern (Window win,
 		 int w, int h)
 {
   int i = 1;
+
   for (; i < coords->num; ++i)
     {
       XDrawLine (dpy, win,
 		 coords->line_style[i] ? ReliefGC : ShadowGC,
-		 w * coords->x[i - 1] / 100,
-		 h * coords->y[i - 1] / 100,
-		 w * coords->x[i] / 100,
-		 h * coords->y[i] / 100);
+                 (w * coords->x[i - 1] / 100.0 + .5) - 1,
+                 (h * coords->y[i - 1] / 100.0 + .5) - 1,
+                 (w * coords->x[i]     / 100.0 + .5) - 1,
+                 (h * coords->y[i]     / 100.0 + .5) - 1);
     }
 }
 
@@ -494,12 +535,13 @@ DrawLinePattern (Window win,
  *  Draws the relief pattern around a window
  *
  ****************************************************************************/
-inline void
+void
 RelieveWindow (XfwmWindow * t, Window win,
 	       int x, int y, int w, int h,
 	       GC ReliefGC, GC ShadowGC, int hilite)
 {
   XSegment seg[4];
+  GC BackGC   = NULL;
   int i;
   int edge;
 
@@ -519,52 +561,61 @@ RelieveWindow (XfwmWindow * t, Window win,
   /* window sides */
   if (edge == -1)
     {
+      BackGC = ((Scr.Hilite == t) ? GetDecor (t, HiBackGC) : GetDecor (t, LoBackGC));
       switch (hilite)
 	{
 	case LEFT_HILITE:
+          XFillRectangle (dpy, win, BackGC,
+		      x + 1, y, w, h + y + 1);
 	  i = 0;
 	  seg[i].x1 = x;
 	  seg[i].y1 = y;
 	  seg[i].x2 = x;
-	  seg[i++].y2 = h + y - 1;
+	  seg[i++].y2 = h + y;
 	  XDrawSegments (dpy, win, ReliefGC, seg, i);
 	  break;
 
 	case TOP_HILITE:
+          XFillRectangle (dpy, win, BackGC,
+		      x, y + 1, w + x + 1, h);
 	  i = 0;
 	  seg[i].x1 = x;
 	  seg[i].y1 = y;
-	  seg[i].x2 = w + x - 1;
+	  seg[i].x2 = w + x;
 	  seg[i++].y2 = y;
 	  XDrawSegments (dpy, win, ReliefGC, seg, i);
 	  break;
 
 	case RIGHT_HILITE:
+          XFillRectangle (dpy, win, BackGC,
+		      x, y, w + x - 2, h + y + 1);
 	  i = 0;
 	  seg[i].x1 = w + x - 1;
 	  seg[i].y1 = y;
 	  seg[i].x2 = w + x - 1;
-	  seg[i++].y2 = h + y - 1;
+	  seg[i++].y2 = h + y;
 	  XDrawSegments (dpy, win, Scr.BlackGC, seg, i);
 	  i = 0;
 	  seg[i].x1 = w + x - 2;
 	  seg[i].y1 = y;
 	  seg[i].x2 = w + x - 2;
-	  seg[i++].y2 = h + y - 1;
+	  seg[i++].y2 = h + y;
 	  XDrawSegments (dpy, win, ShadowGC, seg, i);
 	  break;
 
 	case BOTTOM_HILITE:
+          XFillRectangle (dpy, win, BackGC,
+		      x, y, w + x + 1, h + y - 2);
 	  i = 0;
 	  seg[i].x1 = x;
 	  seg[i].y1 = h + y - 1;
-	  seg[i].x2 = w + x - 1;
+	  seg[i].x2 = w + x;
 	  seg[i++].y2 = h + y - 1;
 	  XDrawSegments (dpy, win, Scr.BlackGC, seg, i);
 	  i = 0;
 	  seg[i].x1 = x;
 	  seg[i].y1 = h + y - 2;
-	  seg[i].x2 = w + x - 1;
+	  seg[i].x2 = w + x;
 	  seg[i++].y2 = h + y - 2;
 	  XDrawSegments (dpy, win, ShadowGC, seg, i);
 	  break;
@@ -742,28 +793,34 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
             Bool sendEvent, Bool broadcast)
 {
   XWindowChanges frame_wc, xwc;
-  unsigned long frame_mask, xwcm;
   Bool Resized = False, Moved = False;
   int xwidth, ywidth, left, right, horig;
   int cx, cy, i;
 
   horig = h;
+
+  if (tmp_win->flags & TITLE)
+    tmp_win->title_height = GetDecor (tmp_win, TitleHeight) + tmp_win->bw;
+
   if (tmp_win->flags & SHADED)
     h = tmp_win->title_height + 2 * tmp_win->boundary_width;
 
   /* if windows is not being maximized, save size in case of maximization */
-  if (!(tmp_win->flags & MAXIMIZED)
-    )
+  if (!(tmp_win->flags & MAXIMIZED))
     {
       tmp_win->orig_x = x;
       tmp_win->orig_y = y;
       tmp_win->orig_wd = w;
       tmp_win->orig_ht = horig;
     }
-  if (x >= Scr.MyDisplayWidth + Scr.VxMax - Scr.Vx - 16)
-    x = Scr.MyDisplayWidth + Scr.VxMax - Scr.Vx - 16;
-  if (y >= Scr.MyDisplayHeight + Scr.VyMax - Scr.Vy - 16)
-    y = Scr.MyDisplayHeight + Scr.VyMax - Scr.Vy - 16;
+  if (x >= Scr.MyDisplayWidth - 16)
+    x = Scr.MyDisplayWidth - 16;
+  if (y >= Scr.MyDisplayHeight - 16)
+    y = Scr.MyDisplayHeight - 16;
+  if (x <= 16 - w)
+    x = 16 - w;
+  if (y <= 16 - h)
+    y = 16 - h;
 
   if ((w != tmp_win->frame_width) || (h != tmp_win->frame_height))
     Resized = True;
@@ -773,9 +830,6 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
   left = tmp_win->nr_left_buttons;
   right = tmp_win->nr_right_buttons;
 
-  if (tmp_win->flags & TITLE)
-    tmp_win->title_height = GetDecor (tmp_win, TitleHeight) + tmp_win->bw;
-
   tmp_win->title_width = w -
     (left + right) * tmp_win->title_height
     - 2 * tmp_win->boundary_width + tmp_win->bw;
@@ -790,10 +844,8 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
   cx = tmp_win->boundary_width - tmp_win->bw;
   cy = tmp_win->title_height + tmp_win->boundary_width - tmp_win->bw;
 
-  XMoveResizeWindow (dpy, tmp_win->Parent, cx, cy,
+  XMoveResizeWindow (dpy, tmp_win->w, cx, cy,
 		     tmp_win->attr.width, tmp_win->attr.height);
-  XResizeWindow (dpy, tmp_win->w,
-                     tmp_win->attr.width, tmp_win->attr.height);
   /* 
    * fix up frame and assign size/location values in tmp_win
    */
@@ -803,8 +855,7 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
   tmp_win->shade_height = tmp_win->title_height + 2 * tmp_win->boundary_width;
   tmp_win->frame_height = horig;
   frame_wc.height = h;
-  frame_mask = (CWX | CWY | CWWidth | CWHeight);
-  XConfigureWindow (dpy, tmp_win->frame, frame_mask, &frame_wc);
+  XMoveResizeWindow (dpy, tmp_win->frame, frame_wc.x, frame_wc.y, frame_wc.width, frame_wc.height);
 
   /*
    * According to the July 27, 1988 ICCCM draft, we should send a
@@ -818,7 +869,6 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
     {
       if (tmp_win->flags & TITLE)
 	{
-	  xwcm = CWWidth | CWX | CWY | CWHeight;
 	  tmp_win->title_x = tmp_win->boundary_width +
 	    (left) * tmp_win->title_height;
 	  if (tmp_win->title_x >= w - tmp_win->boundary_width)
@@ -830,10 +880,9 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
 	  xwc.height = tmp_win->title_height;
 	  xwc.x = tmp_win->title_x;
 	  xwc.y = tmp_win->title_y;
-	  XConfigureWindow (dpy, tmp_win->title_w, xwcm, &xwc);
+          XMoveResizeWindow (dpy, tmp_win->title_w, xwc.x, xwc.y, xwc.width, xwc.height);
 
 
-	  xwcm = CWX | CWY | CWHeight | CWWidth;
 	  xwc.height = tmp_win->title_height;
 	  xwc.width = tmp_win->title_height;
 
@@ -844,11 +893,11 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
 	      if (tmp_win->left_w[i] != None)
 		{
 		  if (xwc.x + tmp_win->title_height < w - tmp_win->boundary_width)
-		    XConfigureWindow (dpy, tmp_win->left_w[i], xwcm, &xwc);
+                    XMoveResizeWindow (dpy, tmp_win->left_w[i], xwc.x, xwc.y, xwc.width, xwc.height);
 		  else
 		    {
 		      xwc.x = -tmp_win->title_height;
-		      XConfigureWindow (dpy, tmp_win->left_w[i], xwcm, &xwc);
+                      XMoveResizeWindow (dpy, tmp_win->left_w[i], xwc.x, xwc.y, xwc.width, xwc.height);
 		    }
 		  xwc.x += tmp_win->title_height;
 		}
@@ -861,11 +910,11 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
 		{
 		  xwc.x -= tmp_win->title_height;
 		  if (xwc.x > tmp_win->boundary_width)
-		    XConfigureWindow (dpy, tmp_win->right_w[i], xwcm, &xwc);
+                    XMoveResizeWindow (dpy, tmp_win->right_w[i], xwc.x, xwc.y, xwc.width, xwc.height);
 		  else
 		    {
 		      xwc.x = -tmp_win->title_height;
-		      XConfigureWindow (dpy, tmp_win->right_w[i], xwcm, &xwc);
+                      XMoveResizeWindow (dpy, tmp_win->right_w[i], xwc.x, xwc.y, xwc.width, xwc.height);
 		    }
 		}
 	    }
@@ -883,7 +932,6 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
 	    tmp_win->corner_width = h / 3;
 	  xwidth = w - 2 * tmp_win->corner_width + tmp_win->bw;
 	  ywidth = h - 2 * tmp_win->corner_width;
-	  xwcm = CWWidth | CWHeight | CWX | CWY;
 	  if (xwidth < 2)
 	    xwidth = 2;
 	  if (ywidth < 2)
@@ -920,10 +968,10 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
 		  xwc.width = tmp_win->boundary_width;
 		  xwc.height = ywidth;
 		}
-	      XConfigureWindow (dpy, tmp_win->sides[i], xwcm, &xwc);
+              XMoveResizeWindow (dpy, tmp_win->sides[i], xwc.x, xwc.y, xwc.width, xwc.height);
+              XFlush (dpy);
 	    }
 
-	  xwcm = CWX | CWY | CWWidth | CWHeight;
 	  xwc.width = tmp_win->corner_width;
 	  xwc.height = tmp_win->corner_width;
 	  for (i = 0; i < 4; i++)
@@ -938,12 +986,11 @@ SetupFrame (XfwmWindow * tmp_win, int x, int y, int w, int h,
 	      else
 		xwc.y = 0;
 
-	      XConfigureWindow (dpy, tmp_win->corners[i], xwcm, &xwc);
+              XMoveResizeWindow (dpy, tmp_win->corners[i], xwc.x, xwc.y, xwc.width, xwc.height);
 	    }
 	}
     }
 
-  SetBorder (tmp_win, (Scr.Hilite == tmp_win), True, True, tmp_win->frame);
   if (ShapesSupported)
     {
       if ((Resized) && (tmp_win->wShaped))
