@@ -26,6 +26,7 @@
 #include "parse.h"
 #include "screen.h"
 #include "module.h"
+#include "constant.h"
 
 static char *exec_shell_name = "/bin/sh";
 char *ModulePath = "./";
@@ -57,7 +58,7 @@ Animate (int o_x, int o_y, int o_w, int o_h,
 
   if (Scr.Options & AnimateWin)
     {
-      gcv.line_width = 10;
+      gcv.line_width = 3;
 
       XChangeGC (dpy, Scr.DrawGC, GCLineWidth, &gcv);
 
@@ -155,7 +156,6 @@ DeferExecution (XEvent * eventp, Window * w, XfwmWindow ** tmp_win,
 	{
 	  DispatchEvent ();
 	}
-
     }
 
   *w = eventp->xany.window;
@@ -180,12 +180,6 @@ DeferExecution (XEvent * eventp, Window * w, XfwmWindow ** tmp_win,
       return (TRUE);
     }
 
-  if (*w == (*tmp_win)->Parent)
-    *w = (*tmp_win)->w;
-
-  if (original_w == (*tmp_win)->Parent)
-    original_w = (*tmp_win)->w;
-
   /* this ugly mess attempts to ensure that the release and press
    * are in the same window. */
   if ((*w != original_w) && (original_w != Scr.Root) &&
@@ -236,8 +230,8 @@ FocusOn (XfwmWindow * t, int DeIconifyOnly)
       cy = t->frame_y + t->frame_height / 2;
     }
 
-  dx = (cx + Scr.Vx) / Scr.MyDisplayWidth * Scr.MyDisplayWidth;
-  dy = (cy + Scr.Vy) / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
+  dx = cx / Scr.MyDisplayWidth * Scr.MyDisplayWidth;
+  dy = cy / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
 
   if (t->flags & ICONIFIED)
     {
@@ -293,8 +287,8 @@ WarpOn (XfwmWindow * t, int warp_x, int x_unit, int warp_y, int y_unit)
       cy = t->frame_y + ((t->flags & SHADED) ? t->shade_height : t->frame_height)  / 2;
     }
 
-  dx = (cx + Scr.Vx) / Scr.MyDisplayWidth * Scr.MyDisplayWidth;
-  dy = (cy + Scr.Vy) / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
+  dx = cx / Scr.MyDisplayWidth * Scr.MyDisplayWidth;
+  dy = cy / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
 
   if (t->flags & ICONIFIED)
     {
@@ -317,7 +311,6 @@ WarpOn (XfwmWindow * t, int warp_x, int x_unit, int warp_y, int y_unit)
       XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
     }
   RaiseWindow (t);
-  KeepOnTop ();
 
   /* If the window is still not visible, make it visible! */
   if (((t->frame_x + ((t->flags & SHADED) ? t->shade_height : t->frame_height) ) < 0) || (t->frame_y + t->frame_width < 0) ||
@@ -338,6 +331,9 @@ WarpOn (XfwmWindow * t, int warp_x, int x_unit, int warp_y, int y_unit)
 
 void Shade(XfwmWindow * tmp_win)
 {
+  XWindowAttributes winattrs;
+  unsigned long eventMask;
+
   if (tmp_win == NULL)
     return;
 
@@ -348,26 +344,33 @@ void Shade(XfwmWindow * tmp_win)
     return;
     
   tmp_win->flags |= SHADED;
-  Animate (tmp_win->frame_x,
-    	   tmp_win->frame_y,
-    	   tmp_win->frame_width,
-    	   tmp_win->frame_height,
-    	   tmp_win->frame_x,
-    	   tmp_win->frame_y,
-    	   tmp_win->frame_width,
-    	   tmp_win->title_height + 2 * tmp_win->boundary_width);
+  if (tmp_win->Desk == Scr.CurrentDesk)
+    Animate (tmp_win->frame_x,
+    	     tmp_win->frame_y,
+    	     tmp_win->frame_width,
+    	     tmp_win->frame_height,
+    	     tmp_win->frame_x,
+    	     tmp_win->frame_y,
+    	     tmp_win->frame_width,
+    	     tmp_win->title_height + 2 * tmp_win->boundary_width);
   tmp_win->shade_x = tmp_win->frame_x;
   tmp_win->shade_y = tmp_win->frame_y;
   tmp_win->shade_width = tmp_win->frame_width;
   tmp_win->shade_height = tmp_win->title_height + 2 * tmp_win->boundary_width;
+  MyXGrabServer (dpy);
 
-  XUnmapWindow (dpy, tmp_win->Parent);
+  XGetWindowAttributes (dpy, tmp_win->w, &winattrs);
+  eventMask = winattrs.your_event_mask;
+
+  XSelectInput (dpy, tmp_win->w, eventMask & ~StructureNotifyMask);
+  XUnmapWindow (dpy, tmp_win->w);
+  XSelectInput (dpy, tmp_win->w, eventMask);
 
   SetupFrame (tmp_win, tmp_win->frame_x, tmp_win->frame_y, tmp_win->frame_width,
   	  tmp_win->frame_height, TRUE, TRUE);
   Broadcast (M_SHADE, 3, tmp_win->w, tmp_win->frame,
   	     (unsigned long) tmp_win, 0, 0, 0, 0);
-  if (tmp_win == Scr.Focus)
+  if ((tmp_win == Scr.Focus) && (tmp_win->Desk == Scr.CurrentDesk))
     {
       if ((tmp_win->next) && !((tmp_win->next)->flags & SHADED) 
     			  && !((tmp_win->next) == &Scr.XfwmRoot))
@@ -383,7 +386,9 @@ void Shade(XfwmWindow * tmp_win)
 	}
       else
     	SetFocus (Scr.NoFocusWin, NULL, 1);
+      KeepOnTop ();
     }
+  MyXUngrabServer (dpy);
 }
 
 void Unshade(XfwmWindow * tmp_win)
@@ -397,27 +402,29 @@ void Unshade(XfwmWindow * tmp_win)
     return;
     
   tmp_win->flags &= ~SHADED;
-  Animate (tmp_win->frame_x,
-    	   tmp_win->frame_y,
-    	   tmp_win->frame_width,
-    	   tmp_win->title_height + 2 * tmp_win->boundary_width,
-    	   tmp_win->frame_x,
-    	   tmp_win->frame_y,
-    	   tmp_win->frame_width,
-    	   tmp_win->frame_height);
+  if (tmp_win->Desk == Scr.CurrentDesk)
+    Animate (tmp_win->frame_x,
+    	     tmp_win->frame_y,
+    	     tmp_win->frame_width,
+    	     tmp_win->title_height + 2 * tmp_win->boundary_width,
+    	     tmp_win->frame_x,
+    	     tmp_win->frame_y,
+    	     tmp_win->frame_width,
+    	     tmp_win->frame_height);
   tmp = tmp_win->frame_height;
   tmp_win->frame_height = 0;	    /* force redrawing frame */
   SetupFrame (tmp_win, tmp_win->frame_x, tmp_win->frame_y,
     	      tmp_win->frame_width, tmp, TRUE, TRUE);
   XMapWindow (dpy, tmp_win->w);
-  XMapWindow (dpy, tmp_win->Parent);
-  SetBorder (tmp_win, True, True, True, None);
-
+  if (tmp_win->Desk == Scr.CurrentDesk)
+    {
+      SetBorder (tmp_win, True, True, True, None);
+      RaiseWindow (tmp_win);
+      FocusOn (tmp_win, 1);
+      KeepOnTop ();
+    }
   Broadcast (M_UNSHADE, 3, tmp_win->w, tmp_win->frame,
     	     (unsigned long) tmp_win, 0, 0, 0, 0);
-  RaiseWindow (tmp_win);
-  FocusOn (tmp_win, 1);
-  KeepOnTop ();
 }
 
 void
@@ -889,7 +896,8 @@ refresh_function (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 {
   XSetWindowAttributes attributes;
   unsigned long valuemask;
-
+  XfwmWindow *tmp;
+  
   valuemask = CWOverrideRedirect | CWBackingStore | CWSaveUnder | CWBackPixmap;
   attributes.override_redirect = True;
   attributes.save_under = False;
@@ -905,6 +913,16 @@ refresh_function (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   XMapWindow (dpy, w);
   XDestroyWindow (dpy, w);
   XFlush (dpy);
+  
+  /* redraw all windows */
+  MyXGrabServer (dpy);
+  tmp = Scr.XfwmRoot.next;
+  while (tmp != NULL)
+    {
+      SetBorder (tmp, (Scr.Hilite == tmp), True, True, tmp->frame);
+      tmp = tmp->next;
+    }
+  MyXUngrabServer (dpy);
 }
 
 
@@ -1230,9 +1248,10 @@ SetXOR (XEvent * eventp, Window w, XfwmWindow * tmp_win,
       return;
     }
 
-  gcm = GCFunction | GCLineWidth | GCForeground | GCSubwindowMode;
+  gcm = GCFunction | GCLineWidth | GCForeground | GCSubwindowMode | GCCapStyle;
   gcv.function = GXxor;
-  gcv.line_width = 0;
+  gcv.line_width = 1;
+  gcv.cap_style = CapProjecting;
   gcv.foreground = (val1) ? (val1) : (BlackPixel (dpy, Scr.screen) ^ WhitePixel (dpy, Scr.screen));
   gcv.subwindow_mode = IncludeInferiors;
   if (Scr.DrawGC)
@@ -1241,35 +1260,6 @@ SetXOR (XEvent * eventp, Window w, XfwmWindow * tmp_win,
     Scr.DrawGC = XCreateGC (dpy, Scr.Root, gcm, &gcv);
 }
 
-void
-SetDeskSize (XEvent * eventp, Window w, XfwmWindow * tmp_win,
-	     unsigned long context, char *action, int *Module)
-{
-  int val1, val2, val1_unit, val2_unit, n;
-
-  n = GetTwoArguments (action, &val1, &val2, &val1_unit, &val2_unit);
-  if (n != 2)
-    {
-      xfwm_msg (ERR, "SetDeskSize", "DesktopSize requires two arguments");
-      return;
-    }
-  if ((val1_unit != Scr.MyDisplayWidth) ||
-      (val2_unit != Scr.MyDisplayHeight))
-    {
-      xfwm_msg (ERR, "SetDeskSize", "DeskTopSize arguments should be unitless");
-    }
-
-  Scr.VxMax = val1;
-  Scr.VyMax = val2;
-  Scr.VxMax = Scr.VxMax * Scr.MyDisplayWidth - Scr.MyDisplayWidth;
-  Scr.VyMax = Scr.VyMax * Scr.MyDisplayHeight - Scr.MyDisplayHeight;
-  if (Scr.VxMax < 0)
-    Scr.VxMax = 0;
-  if (Scr.VyMax < 0)
-    Scr.VyMax = 0;
-  Broadcast (M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax, 0, 0);
-}
-
 void
 setModulePath (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 	       unsigned long context, char *action, int *Module)
@@ -1294,46 +1284,36 @@ SetHiColor (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   XGCValues gcv;
   unsigned long gcm;
   char *hifore = NULL, *hiback = NULL;
-  extern char *white, *black;
   XfwmWindow *hilight;
   XfwmDecor *fl = &Scr.DefaultDecor;
 
   action = GetNextToken (action, &hifore);
   GetNextToken (action, &hiback);
 
-  if (Scr.d_depth > 2)
+  if (hifore != NULL)
     {
-      if (hifore != NULL)
-	{
-	  fl->HiColors.fore = GetColor (hifore);
-	}
-      if (hiback != NULL)
-	{
-	  fl->HiColors.back = GetColor (hiback);
-	}
-      fl->HiRelief.fore = GetHilite (fl->HiColors.back);
-      fl->HiRelief.back = GetShadow (fl->HiColors.back);
+      fl->HiColors.fore = GetColor (hifore);
     }
-  else
+  if (hiback != NULL)
     {
-      fl->HiColors.back = GetColor (white);
-      fl->HiColors.fore = GetColor (black);
-      fl->HiRelief.back = GetColor (black);
-      fl->HiRelief.fore = GetColor (white);
+      fl->HiColors.back = GetColor (hiback);
     }
+  fl->HiRelief.fore = GetHilite (fl->HiColors.back);
+  fl->HiRelief.back = GetShadow (fl->HiColors.back);
   if (hifore)
     free (hifore);
   if (hiback)
     free (hiback);
   gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth | GCForeground |
-    GCBackground;
+    GCBackground | GCCapStyle;
   gcv.foreground = fl->HiRelief.fore;
   gcv.background = fl->HiRelief.back;
   gcv.fill_style = FillSolid;
   gcv.plane_mask = AllPlanes;
   gcv.function = GXcopy;
   gcv.graphics_exposures = False;
-  gcv.line_width = 0;
+  gcv.line_width = 1;
+  gcv.cap_style = CapProjecting;
   if (fl->HiReliefGC != NULL)
     {
       XFreeGC (dpy, fl->HiReliefGC);
@@ -1348,6 +1328,14 @@ SetHiColor (XEvent * eventp, Window w, XfwmWindow * tmp_win,
     }
   fl->HiShadowGC = XCreateGC (dpy, Scr.Root, gcm, &gcv);
 
+  gcv.foreground = fl->HiColors.back;
+  gcv.background = BlackPixel (dpy, Scr.screen);
+  if (fl->HiBackGC != NULL)
+    {
+      XFreeGC (dpy, fl->HiBackGC);
+    }
+  fl->HiBackGC = XCreateGC (dpy, Scr.Root, gcm, &gcv);
+
   if ((Scr.flags & WindowsCaptured) && (Scr.Hilite != NULL))
     {
       hilight = Scr.Hilite;
@@ -1363,44 +1351,34 @@ SetLoColor (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   XGCValues gcv;
   unsigned long gcm;
   char *lofore = NULL, *loback = NULL;
-  extern char *white, *black;
   XfwmDecor *fl = &Scr.DefaultDecor;
 
   action = GetNextToken (action, &lofore);
   GetNextToken (action, &loback);
-  if (Scr.d_depth > 2)
+  if (lofore != NULL)
     {
-      if (lofore != NULL)
-	{
-	  fl->LoColors.fore = GetColor (lofore);
-	}
-      if (loback != NULL)
-	{
-	  fl->LoColors.back = GetColor (loback);
-	}
-      fl->LoRelief.back = GetShadow (fl->LoColors.back);
-      fl->LoRelief.fore = GetHilite (fl->LoColors.back);
+      fl->LoColors.fore = GetColor (lofore);
     }
-  else
+  if (loback != NULL)
     {
-      fl->LoColors.back = GetColor (white);
-      fl->LoColors.fore = GetColor (black);
-      fl->LoRelief.back = GetColor (black);
-      fl->LoRelief.fore = GetColor (white);
+      fl->LoColors.back = GetColor (loback);
     }
+  fl->LoRelief.back = GetShadow (fl->LoColors.back);
+  fl->LoRelief.fore = GetHilite (fl->LoColors.back);
   if (lofore)
     free (lofore);
   if (loback)
     free (loback);
   gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth | GCForeground |
-    GCBackground;
+    GCBackground | GCCapStyle;
   gcv.foreground = fl->LoRelief.fore;
   gcv.background = fl->LoRelief.back;
   gcv.fill_style = FillSolid;
   gcv.plane_mask = AllPlanes;
   gcv.function = GXcopy;
   gcv.graphics_exposures = False;
-  gcv.line_width = 0;
+  gcv.line_width = 1;
+  gcv.cap_style = CapProjecting;
   if (fl->LoReliefGC != NULL)
     {
       XFreeGC (dpy, fl->LoReliefGC);
@@ -1413,7 +1391,16 @@ SetLoColor (XEvent * eventp, Window w, XfwmWindow * tmp_win,
     {
       XFreeGC (dpy, fl->LoShadowGC);
     }
+
   fl->LoShadowGC = XCreateGC (dpy, Scr.Root, gcm, &gcv);
+
+  gcv.foreground = fl->LoColors.back;
+  gcv.background = fl->LoRelief.back;
+  if (fl->LoBackGC != NULL)
+    {
+      XFreeGC (dpy, fl->LoBackGC);
+    }
+  fl->LoBackGC = XCreateGC (dpy, Scr.Root, gcm, &gcv);
 }
 
 void
@@ -1438,28 +1425,25 @@ CursorColor (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
       return;
     }
 
-  if (Scr.d_depth > 2)
-    {
-      fore = GetXColor (hifore);
-      back = GetXColor (hiback);
-      XRecolorCursor (dpy, Scr.XfwmCursors[POSITION], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[DEFAULT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[SYS], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[TITLE_CURSOR], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[MOVE], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[MENU], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[WAIT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[SELECT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[DESTROY], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[LEFT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[RIGHT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[TOP], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[BOTTOM], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[TOP_LEFT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[TOP_RIGHT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[BOTTOM_LEFT], &fore, &back);
-      XRecolorCursor (dpy, Scr.XfwmCursors[BOTTOM_RIGHT], &fore, &back);
-    }
+  fore = GetXColor (hifore);
+  back = GetXColor (hiback);
+  XRecolorCursor (dpy, Scr.XfwmCursors[POSITION], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[DEFAULT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[SYS], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[TITLE_CURSOR], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[MOVE], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[MENU], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[WAIT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[SELECT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[DESTROY], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[LEFT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[RIGHT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[TOP], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[BOTTOM], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[TOP_LEFT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[TOP_RIGHT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[BOTTOM_LEFT], &fore, &back);
+  XRecolorCursor (dpy, Scr.XfwmCursors[BOTTOM_RIGHT], &fore, &back);
 }
 
 void
@@ -1469,45 +1453,32 @@ SetMenuColor (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   XGCValues gcv;
   unsigned long gcm;
   char *fore = NULL, *back = NULL, *selfore = NULL, *selback = NULL;
-  extern char *white, *black;
 
   action = GetNextToken (action, &fore);
   action = GetNextToken (action, &back);
   action = GetNextToken (action, &selfore);
   action = GetNextToken (action, &selback);
 
-  if (Scr.d_depth > 2)
+  if (fore != NULL)
     {
-      if (fore != NULL)
-	{
-	  Scr.MenuColors.fore = GetColor (fore);
-	}
-      if (back != NULL)
-	{
-	  Scr.MenuColors.back = GetColor (back);
-	}
-      if (selfore != NULL)
-	{
-	  Scr.MenuSelColors.fore = GetColor (selfore);
-	}
-      if (back != NULL)
-	{
-	  Scr.MenuSelColors.back = GetColor (selback);
-	}
-      Scr.MenuRelief.back = GetShadow (Scr.MenuColors.back);
-      Scr.MenuRelief.fore = GetHilite (Scr.MenuColors.back);
+      Scr.MenuColors.fore = GetColor (fore);
     }
-  else
+  if (back != NULL)
     {
-      Scr.MenuColors.back = GetColor (white);
-      Scr.MenuColors.fore = GetColor (black);
-      Scr.MenuRelief.back = GetColor (black);
-      Scr.MenuRelief.fore = GetColor (white);
-      Scr.MenuSelColors.back = GetColor (white);
-      Scr.MenuSelColors.fore = GetColor (black);
+      Scr.MenuColors.back = GetColor (back);
     }
+  if (selfore != NULL)
+    {
+      Scr.MenuSelColors.fore = GetColor (selfore);
+    }
+  if (back != NULL)
+    {
+      Scr.MenuSelColors.back = GetColor (selback);
+    }
+  Scr.MenuRelief.back = GetShadow (Scr.MenuColors.back);
+  Scr.MenuRelief.fore = GetHilite (Scr.MenuColors.back);
 
-  gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth | GCForeground | GCBackground;
+  gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth | GCForeground | GCBackground | GCCapStyle;
 
   gcv.foreground = Scr.MenuRelief.fore;
   gcv.background = Scr.MenuRelief.back;
@@ -1515,7 +1486,8 @@ SetMenuColor (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   gcv.plane_mask = AllPlanes;
   gcv.function = GXcopy;
   gcv.graphics_exposures = False;
-  gcv.line_width = 0;
+  gcv.line_width = 1;
+  gcv.cap_style = CapProjecting;
   if (Scr.MenuReliefGC != NULL)
     XChangeGC (dpy, Scr.MenuReliefGC, gcm, &gcv);
   else
@@ -1709,10 +1681,17 @@ LoadWindowFont (XEvent * eventp, Window win, XfwmWindow * tmp_win,
         }
       fl->WindowFont.height =
 	fl->WindowFont.font->ascent + fl->WindowFont.font->descent;
+
       fl->WindowFont.y = fl->WindowFont.font->ascent;
       extra_height = fl->TitleHeight;
       fl->TitleHeight = fl->WindowFont.font->ascent + fl->WindowFont.font->descent + 3;
 
+      if (fl->TitleHeight < 16)
+        {
+          fl->WindowFont.y += (16 - fl->TitleHeight) / 2;
+          fl->TitleHeight = 16;
+        }
+
       if (!newfont)
         fl->WindowFont.font = NULL; /* Restore */
 
@@ -1761,6 +1740,18 @@ FreeButtonFace (Display * dpy, ButtonFace * bf)
       free (bf->u.grad.pixels);
       bf->u.grad.pixels = NULL;
       break;
+    case VectorButton:
+      if (bf->bitmap != None)
+        {
+          XFreePixmap (dpy, bf->bitmap);
+          bf->bitmap = None;
+        }
+      if (bf->bitmap_pressed != None)
+        {
+          XFreePixmap (dpy, bf->bitmap_pressed);
+          bf->bitmap_pressed = None;
+        }
+      break;
     default:
       break;
     }
@@ -1817,7 +1808,7 @@ ReadButtonFace (char *s, ButtonFace * bf, int button, int verbose)
 	  b = button;
 	}
       s += offset;
-      if ((b > 0) && (b <= 10))
+      if ((b > 0) && (b <= 6))
 	LoadDefaultButton (bf, b);
       else
 	{
@@ -2329,32 +2320,27 @@ SetFocusMode (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 
   action = GetNextToken (action, &style);
 
-  if ((mystrncasecmp (style, "Click", 5) == 0) ||
-      (mystrncasecmp (style, "ClickToFocus", 12) == 0))
+  if (mystrncasecmp (style, "Click", 5) == 0)
     {
       if (!(Scr.Options & ClickToFocus))
 	{
 	  Scr.Options |= ClickToFocus;
 	  Scr.Options &= ~AutoRaiseWin;
-	  ReGrabKeyWindows ();
 	}
     }
-  else if ((mystrncasecmp (style, "Mouse", 6) == 0) ||
-	   (mystrncasecmp (style, "FollowMouse", 11) == 0))
+  else if ((mystrncasecmp (style, "Mouse", 5) == 0) ||
+	   (mystrncasecmp (style, "Follo", 5) == 0))
     {
       if (Scr.Options & ClickToFocus)
-	{
-	  Scr.Options &= ~ClickToFocus;
-	  ReGrabKeyWindows ();
-	}
+	Scr.Options &= ~ClickToFocus;
     }
   else
     {
       xfwm_msg (ERR, "SetFocusMode",
 		"Unknown focus behaviour, using follow mouse mode");
       Scr.Options &= ~ClickToFocus;
-      ReGrabKeyWindows ();
     }
+  free (style);
 }
 
 void
@@ -2366,25 +2352,21 @@ SetAutoRaise (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
   action = GetNextToken (action, &style);
 
   if ((mystrncasecmp (style, "On", 2) == 0) ||
-      (mystrncasecmp (style, "Yes", 3) == 0) ||
+      (mystrncasecmp (style, "Y", 1) == 0) ||
       (mystrncasecmp (style, "1", 1) == 0))
     {
       if (!(Scr.Options & AutoRaiseWin))
 	{
 	  Scr.Options |= AutoRaiseWin;
 	  Scr.Options &= ~ClickToFocus;
-	  ReGrabKeyWindows ();
 	}
     }
-  else if ((mystrncasecmp (style, "Off", 3) == 0) ||
-	   (mystrncasecmp (style, "No", 2) == 0) ||
+  else if ((mystrncasecmp (style, "Of", 2) == 0) ||
+	   (mystrncasecmp (style, "N", 1) == 0) ||
 	   (mystrncasecmp (style, "0", 1) == 0))
     {
       if (Scr.Options & AutoRaiseWin)
-	{
-	  Scr.Options &= ~AutoRaiseWin;
-	  ReGrabKeyWindows ();
-	}
+	Scr.Options &= ~AutoRaiseWin;
     }
   else
     {
@@ -2392,8 +2374,8 @@ SetAutoRaise (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 		"Unknown parameter, using AutoRaise On");
       Scr.Options |= AutoRaiseWin;
       Scr.Options &= ~ClickToFocus;
-      ReGrabKeyWindows ();
     }
+  free (style);
 }
 
 void
@@ -2422,11 +2404,11 @@ SetAnimate (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
   action = GetNextToken (action, &style);
 
   if ((mystrncasecmp (style, "On", 2) == 0) ||
-      (mystrncasecmp (style, "Yes", 3) == 0) ||
+      (mystrncasecmp (style, "Y", 1) == 0) ||
       (mystrncasecmp (style, "1", 1) == 0))
     Scr.Options |= AnimateWin;
-  else if ((mystrncasecmp (style, "Off", 3) == 0) ||
-	   (mystrncasecmp (style, "No", 2) == 0) ||
+  else if ((mystrncasecmp (style, "Of", 2) == 0) ||
+	   (mystrncasecmp (style, "N", 1) == 0) ||
 	   (mystrncasecmp (style, "0", 1) == 0))
     Scr.Options &= ~AnimateWin;
   else
@@ -2435,6 +2417,7 @@ SetAnimate (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 		"Unknown parameter, using Animate On");
       Scr.Options |= AnimateWin;
     }
+  free (style);
 }
 
 void
@@ -2446,11 +2429,11 @@ SetOpaqueMove (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
   action = GetNextToken (action, &style);
 
   if ((mystrncasecmp (style, "On", 2) == 0) ||
-      (mystrncasecmp (style, "Yes", 3) == 0) ||
+      (mystrncasecmp (style, "Y", 1) == 0) ||
       (mystrncasecmp (style, "1", 1) == 0))
     Scr.Options |= MoveOpaqueWin;
-  else if ((mystrncasecmp (style, "Off", 3) == 0) ||
-	   (mystrncasecmp (style, "No", 2) == 0) ||
+  else if ((mystrncasecmp (style, "Of", 2) == 0) ||
+	   (mystrncasecmp (style, "N", 1) == 0) ||
 	   (mystrncasecmp (style, "0", 1) == 0))
     Scr.Options &= ~MoveOpaqueWin;
   else
@@ -2459,6 +2442,7 @@ SetOpaqueMove (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 		"Unknown parameter, using opaque move On");
       Scr.Options |= MoveOpaqueWin;
     }
+  free (style);
 }
 
 void
@@ -2470,11 +2454,11 @@ SetOpaqueResize (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
   action = GetNextToken (action, &style);
 
   if ((mystrncasecmp (style, "On", 2) == 0) ||
-      (mystrncasecmp (style, "Yes", 3) == 0) ||
+      (mystrncasecmp (style, "Y", 1) == 0) ||
       (mystrncasecmp (style, "1", 1) == 0))
     Scr.Options |= ResizeOpaqueWin;
-  else if ((mystrncasecmp (style, "Off", 3) == 0) ||
-	   (mystrncasecmp (style, "No", 2) == 0) ||
+  else if ((mystrncasecmp (style, "Of", 2) == 0) ||
+	   (mystrncasecmp (style, "N", 1) == 0) ||
 	   (mystrncasecmp (style, "0", 1) == 0))
     Scr.Options &= ~ResizeOpaqueWin;
   else
@@ -2483,6 +2467,7 @@ SetOpaqueResize (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 		"Unknown parameter, using opaque resize On");
       Scr.Options |= ResizeOpaqueWin;
     }
+  free (style);
 }
 
 void
@@ -2490,19 +2475,21 @@ SetIconPos (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 	    unsigned long context, char *action, int *Module)
 {
   char *style = NULL;
+  XfwmWindow *t;
+  int n;
 
   action = GetNextToken (action, &style);
 
-  if ((mystrncasecmp (style, "Top", 3) == 0) ||
+  if ((mystrncasecmp (style, "T", 1) == 0) ||
       (mystrncasecmp (style, "0", 1) == 0))
     Scr.iconbox = 0;
-  else if ((mystrncasecmp (style, "Left", 4) == 0) ||
+  else if ((mystrncasecmp (style, "L", 1) == 0) ||
 	   (mystrncasecmp (style, "1", 1) == 0))
     Scr.iconbox = 1;
-  else if ((mystrncasecmp (style, "Bottom", 6) == 0) ||
+  else if ((mystrncasecmp (style, "B", 1) == 0) ||
 	   (mystrncasecmp (style, "2", 1) == 0))
     Scr.iconbox = 2;
-  else if ((mystrncasecmp (style, "Right", 5) == 0) ||
+  else if ((mystrncasecmp (style, "R", 1) == 0) ||
 	   (mystrncasecmp (style, "3", 1) == 0))
     Scr.iconbox = 3;
   else
@@ -2511,6 +2498,20 @@ SetIconPos (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 		"Unknown parameter, using 'top'");
       Scr.iconbox = 0;
     }
+  MyXGrabServer (dpy);
+  for (n = 0; n <= NBSCREENS; n++)
+    for (t = Scr.XfwmRoot.next; t != NULL; t = t->next)
+      {
+	if ((t->flags & ICONIFIED) && (t->Desk == n) && !(t->flags & ICON_MOVED))
+	  {
+	    AutoPlace (t, True);
+	    t->icon_arranged = True;
+	  }
+      }
+  for (t = Scr.XfwmRoot.next; t != NULL; t = t->next)
+    t->icon_arranged = False;
+  MyXUngrabServer (dpy);
+  free (style);
 }
 
 void
@@ -2523,17 +2524,20 @@ Arrange_Icons (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 
   if (!GetTwoArguments (action, &val1, &val2, &val1_unit, &val2_unit))
     val1 = val2 = Scr.CurrentDesk;
+  MyXGrabServer (dpy);
   for (n = val1; n <= val2; n++)
     for (t = Scr.XfwmRoot.next; t != NULL; t = t->next)
       {
 	if ((t->flags & ICONIFIED) && (t->Desk == n))
 	  {
+	    t->flags &= ~ICON_MOVED;
 	    AutoPlace (t, True);
 	    t->icon_arranged = True;
 	  }
       }
   for (t = Scr.XfwmRoot.next; t != NULL; t = t->next)
     t->icon_arranged = False;
+  MyXUngrabServer (dpy);
 }
 
 void
@@ -2550,6 +2554,156 @@ SetSnapSize (XEvent * eventp, Window w, XfwmWindow * tmp_win,
       return;
     }
 
-  Scr.SnapSize = val1;
+  Scr.SnapSize = ((val1 > 0) ? val1 : 0);
+}
+
+void
+wait_session (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	  unsigned long context, char *action, int *Module)
+{
+  int val1;
+  int val1_unit, n;
+
+  n = GetOneArgument (action, (long *) &val1, &val1_unit);
+  if (n != 1)
+    {
+      xfwm_msg (ERR, "WaitSession", "WaitSession requires 1 argument");
+      return;
+    }
+
+  Scr.sessionwait = ((val1 > 0) ? val1 : 0);
+}
+
+void
+SetSessionMgt (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
+	       unsigned long context, char *action, int *Module)
+{
+  char *style = NULL;
+
+  action = GetNextToken (action, &style);
+
+  if ((mystrncasecmp (style, "On", 2) == 0) ||
+      (mystrncasecmp (style, "Y", 1) == 0) ||
+      (mystrncasecmp (style, "1", 1) == 0))
+    Scr.Options |= SessionMgt;
+  else if ((mystrncasecmp (style, "Of", 2) == 0) ||
+	   (mystrncasecmp (style, "N", 1) == 0) ||
+	   (mystrncasecmp (style, "0", 1) == 0))
+    Scr.Options &= ~SessionMgt;
+  else
+    {
+      xfwm_msg (ERR, "SetSessionMgt",
+      		"Unknown parameter, using session management On");
+      Scr.Options |= SessionMgt;
+    }
+  free (style);
+}
+
+void
+SetIconGrid (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	  unsigned long context, char *action, int *Module)
+{
+  int val1;
+  int val1_unit, n;
+
+  n = GetOneArgument (action, (long *) &val1, &val1_unit);
+  if (n != 1)
+    {
+      xfwm_msg (ERR, "SetIconGrid", "Icon grid requires 1 argument");
+      return;
+    }
+
+  Scr.icongrid = ((val1 > 1) ? val1 : 1);
+}
+
+void
+SetIconSpacing (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	  unsigned long context, char *action, int *Module)
+{
+  int val1;
+  int val1_unit, n;
+
+  n = GetOneArgument (action, (long *) &val1, &val1_unit);
+  if (n != 1)
+    {
+      xfwm_msg (ERR, "SetIconSpacing", "Icon spacing requires 1 argument");
+      return;
+    }
+
+  Scr.iconspacing = ((val1 > 1) ? val1 : 1);
+}
+
+void
+use_bitmap_buttons (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	  unsigned long context, char *action, int *Module)
+{
+  char *style = NULL;
+  XfwmWindow *tmp;
+
+  action = GetNextToken (action, &style);
+
+  if ((mystrncasecmp (style, "On", 2) == 0) ||
+      (mystrncasecmp (style, "Y", 1) == 0) ||
+      (mystrncasecmp (style, "1", 1) == 0))
+    Scr.Options |= BitmapButtons;
+  else if ((mystrncasecmp (style, "Of", 2) == 0) ||
+	   (mystrncasecmp (style, "N", 1) == 0) ||
+	   (mystrncasecmp (style, "0", 1) == 0))
+    Scr.Options &= ~BitmapButtons;
+  else
+    {
+      xfwm_msg (ERR, "BitmapButtons",
+      		"Unknown parameter, using bitmap buttons");
+      Scr.Options |= BitmapButtons;
+    }
+
+  /* redraw all windows */
+  MyXGrabServer (dpy);
+  tmp = Scr.XfwmRoot.next;
+  while (tmp != NULL)
+    {
+      SetBorder (tmp, (Scr.Hilite == tmp), True, True, tmp->frame);
+      tmp = tmp->next;
+    }
+  MyXUngrabServer (dpy);
+  free (style);
+}
+
+void
+show_buttons (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	  unsigned long context, char *action, int *Module)
+{
+  int val1;
+  int val1_unit, n;
+  Bool left = FALSE;
+  Bool right = FALSE;
+  char *side;
+
+  action = GetNextToken (action, &side);
+  if (mystrncasecmp (side, "L", 1) == 0)
+    left = TRUE;
+  else if (mystrncasecmp (side, "R", 1) == 0)
+    right = TRUE;
+  free (side);
+  
+  if ((!left) && (!right))
+    {
+      xfwm_msg (ERR, "ShowButtons", "Which side ?");
+      return;
+    }
+    
+  n = GetOneArgument (action, (long *) &val1, &val1_unit);
+  if (n != 1)
+    {
+      xfwm_msg (ERR, "ShowButtons", "Missing argument !");
+      return;
+    }
+    
+  if (left && (val1 >= 0) && (val1 <= 3))
+    Scr.nr_left_buttons = val1;
+  else if (right && (val1 >= 0) && (val1 <= 3))
+    Scr.nr_right_buttons = val1;
+  else
+    xfwm_msg (ERR, "ShowButtons", "Value out of range");
 }
 
