@@ -1,5 +1,5 @@
 /****************************************************************************
- * This module is based on Twm, but has been siginificantly modified 
+ * This module is based on Twm, but has been significantly modified 
  * by Rob Nation 
  ****************************************************************************/
 /*****************************************************************************/
@@ -64,6 +64,7 @@
 #include <X11/extensions/shape.h>
 #include "module.h"
 #include "constant.h"
+#include "session.h"
 
 unsigned int mods_used = (ShiftMask | ControlMask | Mod1Mask |
 			  Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask);
@@ -78,6 +79,8 @@ XfwmWindow *Tmp_win;		/* the current xfwm window */
 int last_event_type = 0;
 Window last_event_window = 0;
 volatile int alarmed;
+static int old_x_root = 0;
+static int old_y_root = 0;
 
 extern int ShapeEventBase;
 void HandleShapeNotify (void);
@@ -116,25 +119,26 @@ InitEventHandlerJumpTable (void)
     {
       EventHandlerJumpTable[i] = NULL;
     }
-  EventHandlerJumpTable[Expose] = HandleExpose;
-  EventHandlerJumpTable[DestroyNotify] = HandleDestroyNotify;
-  EventHandlerJumpTable[MapRequest] = HandleMapRequest;
-  EventHandlerJumpTable[MapNotify] = HandleMapNotify;
-  EventHandlerJumpTable[UnmapNotify] = HandleUnmapNotify;
-  EventHandlerJumpTable[ButtonPress] = HandleButtonPress;
-  EventHandlerJumpTable[EnterNotify] = HandleEnterNotify;
-  EventHandlerJumpTable[LeaveNotify] = HandleLeaveNotify;
-  EventHandlerJumpTable[FocusIn] = HandleFocusIn;
-  EventHandlerJumpTable[FocusOut] = HandleFocusOut;
+  EventHandlerJumpTable[Expose] 	  = HandleExpose;
+  EventHandlerJumpTable[DestroyNotify] 	  = HandleDestroyNotify;
+  EventHandlerJumpTable[MapRequest] 	  = HandleMapRequest;
+  EventHandlerJumpTable[MapNotify] 	  = HandleMapNotify;
+  EventHandlerJumpTable[UnmapNotify] 	  = HandleUnmapNotify;
+  EventHandlerJumpTable[ButtonPress] 	  = HandleButtonPress;
+  EventHandlerJumpTable[EnterNotify] 	  = HandleEnterNotify;
+  EventHandlerJumpTable[LeaveNotify] 	  = HandleLeaveNotify;
+  EventHandlerJumpTable[FocusIn] 	  = HandleFocusIn;
+  EventHandlerJumpTable[FocusOut] 	  = HandleFocusOut;
   EventHandlerJumpTable[ConfigureRequest] = HandleConfigureRequest;
-  EventHandlerJumpTable[ClientMessage] = HandleClientMessage;
-  EventHandlerJumpTable[PropertyNotify] = HandlePropertyNotify;
-  EventHandlerJumpTable[KeyPress] = HandleKeyPress;
+  EventHandlerJumpTable[ClientMessage] 	  = HandleClientMessage;
+  EventHandlerJumpTable[PropertyNotify]   = HandlePropertyNotify;
+  EventHandlerJumpTable[KeyPress] 	  = HandleKeyPress;
   EventHandlerJumpTable[VisibilityNotify] = HandleVisibilityNotify;
-  EventHandlerJumpTable[ColormapNotify] = HandleColormapNotify;
-  EventHandlerJumpTable[MotionNotify] = HandleMotionNotify;
+  EventHandlerJumpTable[ColormapNotify]   = HandleColormapNotify;
+  EventHandlerJumpTable[MotionNotify] 	  = HandleMotionNotify;
   if (ShapesSupported)
-    EventHandlerJumpTable[ShapeEventBase + ShapeNotify] = HandleShapeNotify;
+    EventHandlerJumpTable[ShapeEventBase + ShapeNotify] 
+    					  = HandleShapeNotify;
 }
 
 /***********************************************************************
@@ -207,8 +211,7 @@ GetContext (XfwmWindow * t, XEvent * e, Window * w)
   if ((e->type == KeyPress) && (e->xkey.subwindow != None))
     *w = e->xkey.subwindow;
 
-  if ((e->type == ButtonPress) && (e->xbutton.subwindow != None) &&
-    ((e->xbutton.subwindow == t->w) || (e->xbutton.subwindow == t->Parent)))
+  if ((e->type == ButtonPress) && (e->xbutton.subwindow != None) && (e->xbutton.subwindow == t->w))
     *w = e->xbutton.subwindow;
 
   if (*w == Scr.Root)
@@ -217,7 +220,7 @@ GetContext (XfwmWindow * t, XEvent * e, Window * w)
     {
       if (*w == t->title_w)
 	Context = C_TITLE;
-      if ((*w == t->w) || (*w == t->Parent))
+      if (*w == t->w)
 	Context = C_WINDOW;
       if (*w == t->icon_w)
 	Context = C_ICON;
@@ -461,6 +464,7 @@ HandlePropertyNotify ()
 
       if (Tmp_win->name == NULL)
 	Tmp_win->name = NoName;
+      Tmp_win->flags |= WM_NAME_CHANGED;
       BroadcastName (M_WINDOW_NAME, Tmp_win->w, Tmp_win->frame,
 		     (unsigned long) Tmp_win, Tmp_win->name);
 
@@ -562,8 +566,13 @@ HandlePropertyNotify ()
 
 	      if (!(Tmp_win->flags & SUPPRESSICON))
 		{
-		  LowerWindow (Tmp_win);
-		  AutoPlace (Tmp_win, False);
+                  if (!(Tmp_win->flags & STARTICONIC))
+                    LowerWindow (Tmp_win);
+
+		  if ((!(Tmp_win->flags & STARTICONIC)) && 
+		      (!(Tmp_win->flags & ICON_MOVED) || !CheckIconPlace (Tmp_win)))
+		    AutoPlace (Tmp_win, False);
+
 		  if (Tmp_win->Desk == Scr.CurrentDesk)
 		    {
 		      if (Tmp_win->icon_w)
@@ -574,6 +583,7 @@ HandlePropertyNotify ()
 		}
 	      Tmp_win->flags |= ICONIFIED;
 	      DrawIconWindow (Tmp_win);
+              LowerWindow (Tmp_win);
 	    }
 	}
       break;
@@ -594,14 +604,10 @@ HandlePropertyNotify ()
 	}
       else if (Event.xproperty.atom == _XA_WM_STATE)
 	{
-	  if ((Tmp_win != NULL) && (Tmp_win == Scr.Focus) 
-                                && (Scr.Options & (ClickToFocus | AutoRaiseWin)))
+	  if ((Tmp_win != NULL) && (Tmp_win == Scr.Focus))
 	    {
 	      Scr.Focus = NULL;
-              if (Scr.Options & ClickToFocus)
-  	        SetFocus (Tmp_win->w, Tmp_win, 0);
-              else
-  	        SetFocus (Tmp_win->w, Tmp_win, 1);
+	      SetFocus (Tmp_win->w, Tmp_win, 1);
 	    }
 	}
       break;
@@ -632,22 +638,6 @@ HandleClientMessage ()
       ExecuteFunction ("Iconify", Tmp_win, &button, C_FRAME, -1);
       return;
     }
-
-  /*
-   * ** CKH - if we get here, it was an unknown client message, so send
-   * ** it to the client if it was in a window we know about.  I'm not so
-   * ** sure this should be done or not, since every other window manager
-   * ** I've looked at doesn't.  But it might be handy for a free drag and
-   * ** drop setup being developed for Linux.
-   */
-  if (Tmp_win)
-    {
-      if (Event.xclient.window != Tmp_win->w)
-	{
-	  Event.xclient.window = Tmp_win->w;
-	  XSendEvent (dpy, Tmp_win->w, False, NoEventMask, &Event);
-	}
-    }
 }
 
 /***********************************************************************
@@ -668,10 +658,14 @@ HandleExpose ()
 	{
 	  SetTitleBar (Tmp_win, (Scr.Hilite == Tmp_win), False);
 	}
-      else
+      else if ((Tmp_win->flags) & ICONIFIED)
 	{
 	  SetBorder (Tmp_win, (Scr.Hilite == Tmp_win), True, True, Event.xany.window);
 	}
+      else
+	{
+	  SetBorder (Tmp_win, (Scr.Hilite == Tmp_win), False, True, Event.xany.window);
+	}
     }
   return;
 }
@@ -706,19 +700,22 @@ HandleMapRequestKeepRaised (Window KeepRaised)
 {
   extern long isIconicState;
 
+  XFlush (dpy);
+  MyXGrabServer (dpy);
+
   Event.xany.window = Event.xmaprequest.window;
 
   if (XFindContext (dpy, Event.xany.window, XfwmContext,
 		    (caddr_t *) & Tmp_win) == XCNOENT)
     Tmp_win = NULL;
-
-  XFlush (dpy);
-
   if (!Tmp_win)
     {
       Tmp_win = AddWindow (Event.xany.window);
       if (Tmp_win == NULL)
-	return;
+        {
+          MyXUngrabServer (dpy);
+	  return;
+        }
     }
 
   if (!(Tmp_win->flags & ICONIFIED))
@@ -736,40 +733,29 @@ HandleMapRequestKeepRaised (Window KeepRaised)
       if (isIconicState != DontCareState)
 	state = isIconicState;
 
-      MyXGrabServer (dpy);
-
       switch (state)
 	{
-	case DontCareState:
-	case NormalState:
-	case InactiveState:
-	default:
-	  if (Tmp_win->Desk == Scr.CurrentDesk)
-	    {
-	      XMapWindow (dpy, Tmp_win->w);
-	      XMapWindow (dpy, Tmp_win->frame);
-	      Tmp_win->flags |= MAP_PENDING;
-	      SetMapStateProp (Tmp_win, NormalState);
-	      if((!Scr.Focus) && (Scr.Options & ClickToFocus))
-		  SetFocus(Tmp_win->w,Tmp_win,1);
-	    }
-	  else
-	    {
-	      XMapWindow (dpy, Tmp_win->w);
-	      SetMapStateProp (Tmp_win, NormalState);
-	    }
-	  break;
-
 	case IconicState:
 	  if (Tmp_win->wmhints)
-	      Iconify (Tmp_win, Tmp_win->wmhints->icon_x, Tmp_win->wmhints->icon_y);
+	    Iconify (Tmp_win, Tmp_win->wmhints->icon_x, 
+                              Tmp_win->wmhints->icon_y);
 	  else
-	      Iconify (Tmp_win, 0, 0);
+	    Iconify (Tmp_win, 0, 0);
+	  break;
+	default:
+	  SetMapStateProp (Tmp_win, NormalState);
+	  Tmp_win->flags |= MAP_PENDING;
+	  if (Tmp_win->Desk == Scr.CurrentDesk)
+	    {
+  	      XMapWindow (dpy, Tmp_win->frame);
+	      SetFocus(Tmp_win->w,Tmp_win,1);
+	    }
+          if (!(Tmp_win->flags & SHADED))
+            {
+  	      XMapWindow (dpy, Tmp_win->w);
+            }
 	  break;
 	}
-
-      XSync (dpy, 0);
-      MyXUngrabServer (dpy);
     }
   else
     {
@@ -778,8 +764,10 @@ HandleMapRequestKeepRaised (Window KeepRaised)
       if (Tmp_win->flags & SHADED)
         Unshade(Tmp_win);
       SetFocus(Tmp_win->w,Tmp_win,1);
+      KeepOnTop ();
     }
-    KeepOnTop ();
+  XSync (dpy, 0);
+  MyXUngrabServer (dpy);
 }
 
 
@@ -792,6 +780,9 @@ HandleMapRequestKeepRaised (Window KeepRaised)
 void
 HandleMapNotify ()
 {
+  XWindowAttributes winattrs;
+  unsigned long eventMask;
+
   if (!Tmp_win)
     {
       if ((Event.xmap.override_redirect == True) &&
@@ -814,7 +805,19 @@ HandleMapNotify ()
   XMapSubwindows (dpy, Tmp_win->frame);
 
   if (Tmp_win->Desk == Scr.CurrentDesk)
-      XMapWindow (dpy, Tmp_win->frame);
+    XMapWindow (dpy, Tmp_win->frame);
+
+  XSync (dpy, 0);
+
+  if (Tmp_win->flags & SHADED)
+    {
+      XGetWindowAttributes (dpy, Tmp_win->w, &winattrs);
+      eventMask = winattrs.your_event_mask;
+
+      XSelectInput (dpy, Tmp_win->w, eventMask & ~StructureNotifyMask);
+      XUnmapWindow (dpy, Tmp_win->w);
+      XSelectInput (dpy, Tmp_win->w, eventMask);
+    }
 
   if (Tmp_win->flags & ICONIFIED)
       Broadcast (M_DEICONIFY, 3, Tmp_win->w, Tmp_win->frame,
@@ -822,17 +825,15 @@ HandleMapNotify ()
   else
       Broadcast (M_MAP, 3, Tmp_win->w, Tmp_win->frame,
 		 (unsigned long) Tmp_win, 0, 0, 0, 0);
-  if ((Scr.Options & ClickToFocus) && (Scr.Focus))
-    SetFocus (Tmp_win->w, Tmp_win, 1);
+  SetFocus (Tmp_win->w, Tmp_win, 1);
   if ((!(Tmp_win->flags & (BORDER | TITLE))) && (Tmp_win->boundary_width < 2))
-    SetBorder (Tmp_win, False, True, True, Tmp_win->frame);
-  XSync (dpy, 0);
-  MyXUngrabServer (dpy);
-  XFlush (dpy);
+    SetBorder (Tmp_win, False, False, True, Tmp_win->frame);
   Tmp_win->flags |= MAPPED;
   Tmp_win->flags &= ~MAP_PENDING;
   Tmp_win->flags &= ~ICONIFIED;
   Tmp_win->flags &= ~ICON_UNMAPPED;
+  MyXUngrabServer (dpy);
+  XSync (dpy, 0);
 }
 
 
@@ -882,17 +883,16 @@ HandleUnmapNotify ()
   if (Scr.PreviousFocus == Tmp_win)
     Scr.PreviousFocus = NULL;
 
-  if ((Tmp_win == Scr.Focus) && (Scr.Options & ClickToFocus))
+  if (Tmp_win == Scr.Focus)
     {
       if (Tmp_win->next)
-	SetFocus(Tmp_win->next->w,Tmp_win->next, 1);
+	SetFocus(Tmp_win->next->w, Tmp_win->next, 1);
+      else if (Tmp_win->prev)
+	SetFocus(Tmp_win->prev->w, Tmp_win->prev, 1);
       else
 	SetFocus (Scr.NoFocusWin, NULL, 1);
     }
 
-  if (Scr.Focus == Tmp_win)
-    SetFocus (Scr.NoFocusWin, NULL, 1);
-
   if (Tmp_win == Scr.pushed_window)
     Scr.pushed_window = NULL;
 
@@ -942,8 +942,7 @@ HandleUnmapNotify ()
       Destroy (Tmp_win);	/* do not need to mash event before */
     }				/* else window no longer exists and we'll get a destroy notify */
   MyXUngrabServer (dpy);
-
-  XFlush (dpy);
+  XSync (dpy, 0);
 }
 
 
@@ -961,32 +960,29 @@ HandleButtonPress ()
   Window x;
   int LocalContext;
 
-  /* click to focus stuff goes here */
   if (Scr.Options & AutoRaiseWin)
     SetTimer(0);
-  if ((Tmp_win) && (Tmp_win != Scr.Ungrabbed) && (Scr.Options & ClickToFocus))
+  if (Tmp_win)
     {
-      SetFocus (Tmp_win->w, Tmp_win, 1);
-      RaiseWindow (Tmp_win);
-      if(!(Tmp_win->flags & ICONIFIED))
-      {
-         XSync(dpy,0);
-         XAllowEvents(dpy,ReplayPointer,CurrentTime);
-         XSync(dpy,0);
-         return;
-      }
+      if ((Event.xbutton.window == Tmp_win->frame) || 
+          (Tmp_win != Scr.Focus) || 
+          (Tmp_win != Scr.LastWindowRaised))
+        {
+          if (Tmp_win != Scr.Focus)
+            SetFocus (Tmp_win->w, Tmp_win, 1);
+          if ((GetContext(Tmp_win,&Event, &PressedW) != C_WINDOW) || 
+              (Scr.Options & (AutoRaiseWin | ClickToFocus)) || 
+              (Tmp_win->flags & ICONIFIED) ||
+              (!(Tmp_win->flags & (BORDER | TITLE))))
+            {
+              RaiseWindow (Tmp_win);
+            }
+          XSync(dpy,0);
+          XAllowEvents(dpy,ReplayPointer,CurrentTime);
+          XSync(dpy,0);
+          return;
+        }
     }
-  else if ((Tmp_win) && !(Scr.Options & ClickToFocus) &&
-           (Event.xbutton.window == Tmp_win->frame))
-  {
-    if ((Tmp_win != Scr.LastWindowRaised) &&
-        (GetContext(Tmp_win,&Event, &PressedW) == C_WINDOW))
-      RaiseWindow (Tmp_win);
-    XSync(dpy,0);
-    XAllowEvents(dpy,ReplayPointer,CurrentTime);
-    XSync(dpy,0);
-    return;
-  }
 
   XSync (dpy, 0);
   XAllowEvents (dpy, ReplayPointer, CurrentTime);
@@ -1002,8 +998,7 @@ HandleButtonPress ()
 
   ButtonWindow = Tmp_win;
 
-  /* we have to execute a function or pop up a menu
-   */
+  /* we have to execute a function or pop up a menu */
 
   modifier = vmod((Event.xkey.state) & KeyMask);
   /* need to search for an appropriate mouse binding */
@@ -1040,38 +1035,26 @@ HandleButtonPress ()
 void
 HandleEnterNotify ()
 {
-  XEnterWindowEvent *ewp = &Event.xcrossing;
-  XEvent d;
-
-  /* look for a matching leaveNotify which would nullify this enterNotify */
-  if (XCheckTypedWindowEvent (dpy, ewp->window, LeaveNotify, &d))
-    {
-      StashEventTime (&d);
-      if ((d.xcrossing.mode == NotifyNormal) &&
-	  (d.xcrossing.detail != NotifyInferior))
-	return;
-    }
-
-  /* multi screen? */
-  if (Event.xany.window == Scr.Root)
-    {
-      if ((!Scr.Focus) || (!Scr.Options & ClickToFocus))
-	SetFocus (Scr.NoFocusWin, NULL, 1);
-      if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
-	InstallWindowColormaps (NULL);
-      return;
-    }
+  XCrossingEvent *e;
+  
+  e = (XCrossingEvent *) &Event;
+  XSync (dpy, False);
 
   /* make sure its for one of our windows */
   if (!Tmp_win)
     return;
 
-  if(!(Scr.Options & ClickToFocus))
+  if(!(Scr.Options & ClickToFocus) && (!(e->focus)) &&
+     ((e->x_root != old_x_root) || (e->y_root != old_y_root)) &&
+     (e->mode == NotifyNormal))
     {
-      SetFocus(Tmp_win->w,Tmp_win,1);
+      old_x_root = e->x_root;
+      old_y_root = e->y_root;
+      SetFocus(Tmp_win->w, Tmp_win, 0);
+
       if (((Scr.Options & AutoRaiseWin) && (Scr.AutoRaiseDelay > 0)) 
          && (!(Tmp_win->flags & ICONIFIED) && !(Tmp_win->flags & RAISEDWIN)))
-	 SetTimer(Scr.AutoRaiseDelay);
+	   SetTimer(Scr.AutoRaiseDelay);
     }
   if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
     {
@@ -1108,7 +1091,7 @@ HandleLeaveNotify ()
 	      if (Scr.Focus != NULL)
 		SetFocus (Scr.NoFocusWin, NULL, 1);
 	      if (Scr.Hilite != NULL)
-		SetBorder (Scr.Hilite, False, True, True, None);
+		SetBorder (Scr.Hilite, False, False, True, None);
 	    }
 	}
     }
@@ -1132,6 +1115,7 @@ HandleConfigureRequest ()
    * Event.xany.window is Event.xconfigurerequest.parent, so Tmp_win will
    * be wrong
    */
+
   Event.xany.window = cre->window;	/* mash parent field */
   if (XFindContext (dpy, cre->window, XfwmContext, (caddr_t *) & Tmp_win) == XCNOENT)
     Tmp_win = NULL;
@@ -1201,7 +1185,7 @@ HandleConfigureRequest ()
       Tmp_win->wShaped = boundingShaped;
     }
 
-  /* Don't modify frame_XXX fields before calling SetupWindow! */
+  /* Don't modify frame_XXX fields before calling SetupFrame */
   x = Tmp_win->frame_x;
   y = Tmp_win->frame_y;
   width = Tmp_win->frame_width;
@@ -1282,8 +1266,8 @@ HandleVisibilityNotify ()
  ************************************************************************/
 int _xfwm_deskwrap (int horz, int vert)
 {
-  register int ndesks = Scr.ndesks;
-  register int desk = Scr.CurrentDesk;
+  int ndesks = Scr.ndesks;
+  int desk = Scr.CurrentDesk;
 
   if (!ndesks) ndesks = 1;
     
@@ -1388,6 +1372,16 @@ HandleRootMotionNotify ()
 void
 HandleMotionNotify ()
 {
+  XMotionEvent *e;
+  
+  e = (XMotionEvent *) &Event;
+
+  if ((e->x_root != old_x_root) || (e->y_root != old_y_root))
+    {
+      old_x_root = e->x_root;
+      old_y_root = e->y_root;
+    }
+
   /* multi screen? */
   if (Event.xany.window == Scr.Root)
     {
@@ -1400,23 +1394,41 @@ HandleMotionNotify ()
  *
  * Waits for next X event, or for an auto-raise timeout.
  *
- ****************************************************************************/
+ ***************************************************************************/
 int
 My_XNextEvent (Display * dpy, XEvent * event)
 {
   extern int fd_width, x_fd;
-  struct itimerval value;
   fd_set in_fdset, out_fdset;
-  Window child;
   Window targetWindow;
   int i, count;
   int retval;
 
-  /* Do this IMMEDIATELY prior to select, to prevent any nasty
-   * queued up X events from just hanging around waiting to be
-   * flushed */
-  /* Do this prior to the select() call, in case the timer already expired,
-   * in which case the select would never return. */
+  FD_ZERO (&in_fdset);
+  FD_SET (x_fd, &in_fdset);
+  FD_ZERO (&out_fdset);
+#ifdef HAVE_SESSION
+  if (sm_fd >= 0) FD_SET(sm_fd, &in_fdset);
+#endif
+  for (i = 0; i < npipes; i++)
+    {
+      if (readPipes[i] >= 0)
+	FD_SET (readPipes[i], &in_fdset);
+      if (pipeQueue[i] != NULL)
+	FD_SET (writePipes[i], &out_fdset);
+    }
+
+  if(alarmed)
+    {
+      SetTimer(0);
+      alarmed = False;
+      if (Scr.Focus != NULL)
+        RaiseWindow(Scr.Focus);
+      return 0;
+    }
+
+  ReapChildren ();
+
   XFlush (dpy);
   if (XPending (dpy))
     {
@@ -1425,48 +1437,8 @@ My_XNextEvent (Display * dpy, XEvent * event)
       return 1;
     }
 
-  if(alarmed)
-    {
-      alarmed = False;
-      XQueryPointer(dpy,Scr.Root,&JunkRoot,&child,&JunkX,&JunkY,&JunkX, 
-		    &JunkY,&JunkMask );
-      if((Scr.Focus != NULL) && (child == Scr.Focus->frame))
-	{
-	  if(!(Scr.Focus->flags & VISIBLE))
-	    { 
-               SetTimer(0);
-	       RaiseWindow(Scr.Focus);
-	    }
-	}
-      return 0;
-    }
-  
-  getitimer(ITIMER_REAL,&value);
-
-  ReapChildren ();
-
-  FD_ZERO (&in_fdset);
-  FD_SET (x_fd, &in_fdset);
-  FD_ZERO (&out_fdset);
-  for (i = 0; i < npipes; i++)
-    {
-      if (readPipes[i] >= 0)
-	{
-	  FD_SET (readPipes[i], &in_fdset);
-	}
-    }
-
-  for (i = 0; i < npipes; i++)
-    {
-      if (pipeQueue[i] != NULL)
-	{
-	  FD_SET (writePipes[i], &out_fdset);
-	}
-    }
-
-  XFlush (dpy);
 #ifdef __hpux
-  retval = select (fd_width, (int *) &in_fdset, (int *) &out_fdset, 0, NULL);
+  retval = select (fd_width, (fd_set *) &in_fdset, (fd_set *) &out_fdset, 0, NULL);
 #else
   retval = select (fd_width, &in_fdset, &out_fdset, 0, NULL);
 #endif
@@ -1496,6 +1468,20 @@ My_XNextEvent (Display * dpy, XEvent * event)
 	      FlushQueue (i);
 	    }
 	}
+      }
+#ifdef HAVE_SESSION
+  /* Weird : If alarm is set, ProcessICEMsgs () stucks */
+  if ((!alarmed) && (sm_fd >= 0) && (FD_ISSET(sm_fd, &in_fdset))) ProcessICEMsgs();
+#endif
+ 
+  /* Process any new X events, to save a recall if any new event have come arround */
+  XFlush (dpy);
+  if (XPending (dpy))
+    {
+      XNextEvent (dpy, event);
+      StashEventTime (event);
+      return 1;
     }
+
   return 0;
 }
