@@ -42,7 +42,6 @@
 
 #include <X11/extensions/shape.h>
 
-
 void GrabIconButtons (XfwmWindow *, Window);
 void GrabIconKeys (XfwmWindow *, Window);
 
@@ -220,21 +219,12 @@ DrawIconWindow (XfwmWindow * Tmp_win)
 
   if (Scr.Hilite == Tmp_win)
     {
-      if (Scr.d_depth < 2)
-	{
-	  Relief = Shadow = Scr.DefaultDecor.HiShadowGC;
-	  TextColor = Scr.DefaultDecor.HiColors.fore;
-	  BackColor = Scr.DefaultDecor.HiColors.back;
-	}
-      else
-	{
-	  Relief = GetDecor (Tmp_win, HiReliefGC);
-	  Shadow = GetDecor (Tmp_win, HiShadowGC);
-	  TextColor = GetDecor (Tmp_win, HiColors.fore);
-	  BackColor = GetDecor (Tmp_win, HiColors.back);
-          if (Tmp_win->icon_w != None)
-            XSetWindowBackground(dpy,Tmp_win->icon_w, GetDecor (Tmp_win, titlebar.state[Active].u.back));
-	}
+      Relief = GetDecor (Tmp_win, HiReliefGC);
+      Shadow = GetDecor (Tmp_win, HiShadowGC);
+      TextColor = GetDecor (Tmp_win, HiColors.fore);
+      BackColor = GetDecor (Tmp_win, HiColors.back);
+      if (Tmp_win->icon_w != None)
+        XSetWindowBackground(dpy,Tmp_win->icon_w, GetDecor (Tmp_win, titlebar.state[Active].u.back));
       /* resize the icon name window */
       if (Tmp_win->icon_w != None)
 	{
@@ -263,20 +253,12 @@ DrawIconWindow (XfwmWindow * Tmp_win)
     }
   else
     {
-      if (Scr.d_depth < 2)
-	{
-	  Relief = Scr.MenuGC;
-	  Shadow = Scr.MenuGC;
-	}
-      else
-	{
-	  Relief = GetDecor (Tmp_win, LoReliefGC);
-	  Shadow = GetDecor (Tmp_win, LoShadowGC);
-	  TextColor = GetDecor (Tmp_win, LoColors.fore);
-	  BackColor = GetDecor (Tmp_win, LoColors.back);
-          if (Tmp_win->icon_w != None)
-            XSetWindowBackground(dpy,Tmp_win->icon_w, GetDecor (Tmp_win, titlebar.state[Inactive].u.back));
-	}
+      Relief = GetDecor (Tmp_win, LoReliefGC);
+      Shadow = GetDecor (Tmp_win, LoShadowGC);
+      TextColor = GetDecor (Tmp_win, LoColors.fore);
+      BackColor = GetDecor (Tmp_win, LoColors.back);
+      if (Tmp_win->icon_w != None)
+        XSetWindowBackground(dpy,Tmp_win->icon_w, GetDecor (Tmp_win, titlebar.state[Inactive].u.back));
       /* resize the icon name window */
       if (Tmp_win->icon_w != None)
 	{
@@ -407,6 +389,53 @@ RedoIconName (XfwmWindow * Tmp_win)
       : \
           ((test_y + temp_h) < (Scr.MyDisplayHeight+base_y)))
 
+/* Test overlapping windows */
+#define OVERLAP(x1, y1, x2, y2, x3, y3, x4, y4) \
+   (((x1 >= x3) && (x1 <= x4) && (y1 >= y3) && (y1 <= y4)) || \
+    ((x2 >= x3) && (x2 <= x4) && (y2 >= y3) && (y2 <= y4)) || \
+    ((x1 >= x3) && (x1 <= x4) && (y2 >= y3) && (y2 <= y4)) || \
+    ((x2 >= x3) && (x2 <= x4) && (y1 >= y3) && (y1 <= y4)) || \
+    ((x3 >= x1) && (x3 <= x2) && (y3 >= y1) && (y3 <= y2)) || \
+    ((x4 >= x1) && (x4 <= x2) && (y4 >= y1) && (y4 <= y2)) || \
+    ((x3 >= x1) && (x3 <= x2) && (y4 >= y1) && (y4 <= y2)) || \
+    ((x4 >= x1) && (x4 <= x2) && (y3 >= y1) && (y3 <= y2)))
+     
+
+Bool 
+CheckIconPlace (XfwmWindow * t)
+{
+  XfwmWindow *tw;
+  Bool loc_ok = True;
+
+  tw = Scr.XfwmRoot.next;
+  MyXGrabServer (dpy);
+  while ((tw != (XfwmWindow *) 0) && (loc_ok == True))
+    {
+      if (tw->Desk == t->Desk)
+	{
+	  if ((tw->flags & ICONIFIED) &&
+	      (tw->icon_w || tw->icon_pixmap_w) &&
+	      (tw != t))
+	    {
+	      if (OVERLAP(t->icon_x_loc - Scr.iconspacing, 
+	                  t->icon_y_loc - Scr.iconspacing,
+                          t->icon_x_loc + t->icon_p_width + Scr.iconspacing, 
+			  t->icon_y_loc + t->icon_w_height + t->icon_p_height + Scr.iconspacing,
+                          tw->icon_x_loc, 
+			  tw->icon_y_loc,
+                          tw->icon_x_loc + tw->icon_p_width, 
+			  tw->icon_y_loc + tw->icon_w_height + tw->icon_p_height))
+		{
+		  loc_ok = False;
+		}
+	    }
+	}
+      tw = tw->next;
+    }
+  MyXUngrabServer (dpy);
+  return (loc_ok);
+}
+
 void
 AutoPlace (XfwmWindow * t, Bool rearrange)
 {
@@ -426,30 +455,28 @@ AutoPlace (XfwmWindow * t, Bool rearrange)
     case 1:
       base_x = 0;
       base_y = 0;
-      step_x = 3;
-      step_y = 3;
+      step_x = Scr.icongrid;
+      step_y = Scr.icongrid;
       break;
     case 2:
       base_x = 0;
-      base_y = Scr.MyDisplayHeight - height - 3;
-      step_x = 3;
-      step_y = -3;
+      base_y = ((int) (Scr.MyDisplayHeight - height) /  Scr.icongrid) * Scr.icongrid;
+      step_x = Scr.icongrid;
+      step_y = -Scr.icongrid;
       break;
     case 3:
-      base_x = Scr.MyDisplayWidth - width - 3;
+      base_x = ((int) (Scr.MyDisplayWidth - width) /  Scr.icongrid) * Scr.icongrid;
       base_y = 0;
-      step_x = -3;
-      step_y = 3;
+      step_x = -Scr.icongrid;
+      step_y = Scr.icongrid;
       break;
     default:
       base_x = 0;
       base_y = 0;
-      step_x = 3;
-      step_y = 3;
+      step_x = Scr.icongrid;
+      step_y = Scr.icongrid;
       break;
     }
-  if (!rearrange)
-    t->Desk = Scr.CurrentDesk;
   if (t->flags & ICON_MOVED)
     {
       /* just make sure the icon is on this screen */
@@ -459,6 +486,10 @@ AutoPlace (XfwmWindow * t, Bool rearrange)
 	t->icon_x_loc += Scr.MyDisplayWidth;
       if (t->icon_y_loc < 0)
 	t->icon_y_loc += Scr.MyDisplayHeight;
+      if (t->icon_x_loc > Scr.MyDisplayWidth)
+	t->icon_x_loc -= Scr.MyDisplayWidth;
+      if (t->icon_y_loc > Scr.MyDisplayHeight)
+	t->icon_y_loc -= Scr.MyDisplayHeight;
     }
   loc_ok = False;
 
@@ -470,6 +501,7 @@ AutoPlace (XfwmWindow * t, Bool rearrange)
 
   temp_h = height;
   temp_w = width;
+  MyXGrabServer (dpy);
   while (TEST_1 && (!loc_ok))
     {
       if ((Scr.iconbox == 0) || (Scr.iconbox == 2))
@@ -495,9 +527,15 @@ AutoPlace (XfwmWindow * t, Bool rearrange)
 		      tx = test_window->icon_x_loc;
 		      ty = test_window->icon_y_loc;
 
-		      if ((tx < (real_x + width + 3)) && ((tx + tw + 3) > real_x) &&
-			  (ty < (real_y + height + 3)) && ((ty + th + 3) > real_y) &&
-			  !(rearrange && !test_window->icon_arranged))
+		      if (OVERLAP(tx - Scr.iconspacing, 
+		                  ty - Scr.iconspacing, 
+				  tx + tw + Scr.iconspacing, 
+				  ty + th + Scr.iconspacing,
+				  real_x,
+				  real_y, 
+				  real_x + width,
+			          real_y + height) &&
+			  !(rearrange && !(test_window->icon_arranged) && !(test_window->flags & ICON_MOVED)))
 			{
 			  loc_ok = False;
 			}
@@ -530,6 +568,7 @@ AutoPlace (XfwmWindow * t, Bool rearrange)
     XMoveResizeWindow (dpy, t->icon_w, t->icon_xl_loc,
 		       t->icon_y_loc + t->icon_p_height,
 		       t->icon_w_width, ICON_HEIGHT);
+  MyXUngrabServer (dpy);
   Broadcast (M_ICON_LOCATION, 7, t->w, t->frame,
 	     (unsigned long) t,
 	     t->icon_x_loc, t->icon_y_loc,
@@ -548,38 +587,12 @@ AutoPlace (XfwmWindow * t, Bool rearrange)
 void
 GrabIconButtons (XfwmWindow * tmp_win, Window w)
 {
-  Binding *MouseEntry;
-
-  MouseEntry = Scr.AllBindings;
-  while (MouseEntry != (Binding *) 0)
-    {
-      if ((MouseEntry->Action != NULL) && (MouseEntry->Context & C_ICON) &&
-	  (MouseEntry->IsMouse == 1))
-	{
-	  if (MouseEntry->Button_Key > 0)
-	    XGrabButton (dpy, MouseEntry->Button_Key, MouseEntry->Modifier, w,
-			 True, ButtonPressMask | ButtonReleaseMask,
-			 GrabModeAsync, GrabModeAsync, None,
-			 Scr.XfwmCursors[DEFAULT]);
-	  else
-	    {
-	      XGrabButton (dpy, 1, MouseEntry->Modifier, w,
-			   True, ButtonPressMask | ButtonReleaseMask,
-			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
-	      XGrabButton (dpy, 2, MouseEntry->Modifier, w,
-			   True, ButtonPressMask | ButtonReleaseMask,
-			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
-	      XGrabButton (dpy, 3, MouseEntry->Modifier, w,
-			   True, ButtonPressMask | ButtonReleaseMask,
-			   GrabModeAsync, GrabModeAsync, None,
-			   Scr.XfwmCursors[DEFAULT]);
-	    }
-	}
-
-      MouseEntry = MouseEntry->NextBinding;
-    }
+  XGrabButton (dpy, AnyButton, 0, w, True,
+               ButtonPressMask, GrabModeSync, GrabModeAsync, None,
+               None);
+  XGrabButton (dpy, AnyButton, AnyModifier, w, True,
+               ButtonPressMask, GrabModeSync, GrabModeAsync, None,
+               None);
   return;
 }
 
@@ -782,22 +795,24 @@ GetIconBitmap (XfwmWindow * tmp_win)
 void
 DeIconify (XfwmWindow * tmp_win)
 {
-  XfwmWindow *t, *tmp;
+  XfwmWindow *t;
 
   if (!tmp_win)
     return;
 
-  Animate (tmp_win->icon_x_loc,
-	   tmp_win->icon_y_loc,
-	   tmp_win->icon_w_width,
-	   tmp_win->icon_w_height,
-	   tmp_win->frame_x,
-	   tmp_win->frame_y,
-	   tmp_win->frame_width,
-	   tmp_win->flags & SHADED ?
-	   tmp_win->title_height + 2 * tmp_win->boundary_width :
-	   tmp_win->frame_height);
+  if (tmp_win->Desk == Scr.CurrentDesk)
+    Animate (tmp_win->icon_x_loc,
+	     tmp_win->icon_y_loc,
+	     tmp_win->icon_w_width,
+	     tmp_win->icon_w_height,
+	     tmp_win->frame_x,
+	     tmp_win->frame_y,
+	     tmp_win->frame_width,
+	     tmp_win->flags & SHADED ?
+	     tmp_win->title_height + 2 * tmp_win->boundary_width :
+	     tmp_win->frame_height);
   /* now de-iconify transients */
+  MyXGrabServer (dpy);
   for (t = Scr.XfwmRoot.next; t != NULL; t = t->next)
     {
       if ((t == tmp_win) ||
@@ -815,16 +830,11 @@ DeIconify (XfwmWindow * tmp_win)
 	      t->flags |= MAP_PENDING;
 	    }
 	  if (!(t->flags & SHADED))
-            XMapWindow (dpy, t->Parent);
+            XMapWindow (dpy, t->w);
 	  SetMapStateProp (t, NormalState);
 	  t->flags &= ~ICONIFIED;
 	  t->flags &= ~ICON_UNMAPPED;
-	  /* Need to make sure the border is colored correctly,
-	   * in case it was stuck or unstuck while iconified. */
-	  tmp = Scr.Hilite;
-	  Scr.Hilite = t;
-	  SetBorder (t, False, True, True, None);
-	  Scr.Hilite = tmp;
+	  t->flags &= ~STARTICONIC;
 	  if (!(t->flags & SHADED))
 	    XRaiseWindow (dpy, t->w);
 	  if (t->icon_w)
@@ -839,8 +849,8 @@ DeIconify (XfwmWindow * tmp_win)
     {
       RaiseWindow (tmp_win);
       FocusOn (tmp_win, 1);
-      KeepOnTop ();
     }
+  MyXUngrabServer (dpy);
   return;
 }
 
@@ -862,8 +872,7 @@ Iconify (XfwmWindow * tmp_win, int def_x, int def_y)
 
   XGetWindowAttributes (dpy, tmp_win->w, &winattrs);
   eventMask = winattrs.your_event_mask;
-
-  if ((tmp_win) && (tmp_win == Scr.Hilite))
+  if ((tmp_win->Desk == Scr.CurrentDesk) && (tmp_win) && (tmp_win == Scr.Hilite))
     {
       if (tmp_win->next)
 	SetFocus (tmp_win->next->w, tmp_win->next, 1);
@@ -872,6 +881,7 @@ Iconify (XfwmWindow * tmp_win, int def_x, int def_y)
     }
 
   /* iconify transients first */
+  MyXGrabServer (dpy);
   for (t = Scr.XfwmRoot.next; t != NULL; t = t->next)
     {
       if ((t == tmp_win) ||
@@ -907,6 +917,7 @@ Iconify (XfwmWindow * tmp_win, int def_x, int def_y)
 	    }
 	}
     }
+  MyXUngrabServer (dpy);
   if (tmp_win->icon_w == None)
     {
       if (tmp_win->flags & ICON_MOVED)
@@ -933,7 +944,9 @@ Iconify (XfwmWindow * tmp_win, int def_x, int def_y)
       tmp_win->icon_w_width = tmp_win->icon_p_width;
     }
 
-  AutoPlace (tmp_win, False);
+  if ((!(tmp_win->flags & STARTICONIC)) && 
+      (!(tmp_win->flags & ICON_MOVED) || !CheckIconPlace (tmp_win)))
+    AutoPlace (tmp_win, False);
   tmp_win->flags |= ICONIFIED;
   tmp_win->flags &= ~ICON_UNMAPPED;
   Broadcast (M_ICONIFY, 7, tmp_win->w, tmp_win->frame,
@@ -941,41 +954,44 @@ Iconify (XfwmWindow * tmp_win, int def_x, int def_y)
 	     tmp_win->icon_x_loc, tmp_win->icon_y_loc,
 	     tmp_win->icon_w_width,
 	     tmp_win->icon_w_height + tmp_win->icon_p_height);
-  Animate (tmp_win->frame_x,
-	   tmp_win->frame_y,
-	   tmp_win->frame_width,
-	   tmp_win->flags & SHADED ?
-	   tmp_win->title_height + 2 * tmp_win->boundary_width :
-	   tmp_win->frame_height,
-	   tmp_win->icon_x_loc,
-	   tmp_win->icon_y_loc,
-	   tmp_win->icon_w_width,
-	   tmp_win->icon_w_height);
   BroadcastConfig (M_CONFIGURE_WINDOW, tmp_win);
 
-  LowerWindow (tmp_win);
 
   if (tmp_win->Desk == Scr.CurrentDesk)
     {
+      LowerWindow (tmp_win);
+      if (!(tmp_win->flags & STARTICONIC))
+        Animate (tmp_win->frame_x,
+	     tmp_win->frame_y,
+	     tmp_win->frame_width,
+	     tmp_win->flags & SHADED ?
+	     tmp_win->title_height + 2 * tmp_win->boundary_width :
+	     tmp_win->frame_height,
+	     tmp_win->icon_x_loc,
+	     tmp_win->icon_y_loc,
+	     tmp_win->icon_w_width,
+	     tmp_win->icon_w_height);
+
       if (tmp_win->icon_w != None)
 	XMapWindow (dpy, tmp_win->icon_w);
 
-      if (tmp_win->icon_pixmap_w != None)
+      if (tmp_win->icon_pixmap_w != None);
 	XMapWindow (dpy, tmp_win->icon_pixmap_w);
       KeepOnTop ();
+
+      if ((tmp_win->Desk == Scr.CurrentDesk) && (tmp_win == Scr.Focus))
+        {
+          if (Scr.PreviousFocus == Scr.Focus)
+	    Scr.PreviousFocus = NULL;
+          if (tmp_win->next)
+	    SetFocus (tmp_win->next->w, tmp_win->next, 1);
+          else if (tmp_win->prev)
+	    SetFocus (tmp_win->prev->w, tmp_win->prev, 1);
+          else
+	    SetFocus (Scr.NoFocusWin, NULL, 1);
+        }
     }
 
-  if (tmp_win == Scr.Focus)
-    {
-      if (Scr.PreviousFocus == Scr.Focus)
-	Scr.PreviousFocus = NULL;
-      if (tmp_win->next)
-	SetFocus (tmp_win->next->w, tmp_win->next, 1);
-      else if (tmp_win->prev)
-	SetFocus (tmp_win->prev->w, tmp_win->prev, 1);
-      else
-	SetFocus (Scr.NoFocusWin, NULL, 1);
-    }
   return;
 }
 
