@@ -190,10 +190,7 @@ PaintEntry (MenuRoot * mr, MenuItem * mi)
   text_y = y_offset + Scr.StdFont.y;
 
   ShadowGC = Scr.MenuShadowGC;
-  if (Scr.d_depth < 2)
-    ReliefGC = Scr.MenuShadowGC;
-  else
-    ReliefGC = Scr.MenuReliefGC;
+  ReliefGC = Scr.MenuReliefGC;
 
   currentGC = Scr.MenuGC;
 
@@ -544,7 +541,7 @@ menuShortcuts (XEvent * ev)
 int
 UpdateMenu (int sticks)
 {
-  int done;
+  int done = 0;
   int retval;
   MenuItem *InitialMI;
   MenuRoot *actual_mr;
