@@ -42,8 +42,6 @@
 #include <Imlib.h>
 #endif
 
-XfwmWindow *FocusOnNextTimeStamp = NULL;
-
 char NoName[] = "Untitled";	/* name if no name in XA_WM_NAME */
 char NoClass[] = "NoClass";	/* Class if no res_class in class hints */
 char NoResource[] = "NoResource";	/* Class if no res_name in class hints */
@@ -159,12 +157,6 @@ Destroy (XfwmWindow * Tmp_win)
 	SetFocus (Scr.NoFocusWin, NULL, 1);
     }
 
-  if (Tmp_win == FocusOnNextTimeStamp)
-    FocusOnNextTimeStamp = NULL;
-
-  if (Tmp_win == Scr.Ungrabbed)
-    Scr.Ungrabbed = NULL;
-
   if (Tmp_win == Scr.pushed_window)
     Scr.pushed_window = NULL;
 
@@ -174,10 +166,6 @@ Destroy (XfwmWindow * Tmp_win)
   XDestroyWindow (dpy, Tmp_win->frame);
   XDeleteContext (dpy, Tmp_win->frame, XfwmContext);
 
-  XDestroyWindow (dpy, Tmp_win->Parent);
-
-  XDeleteContext (dpy, Tmp_win->Parent, XfwmContext);
-
   XDeleteContext (dpy, Tmp_win->w, XfwmContext);
 
   if ((Tmp_win->icon_w) && (Tmp_win->flags & PIXMAP_OURS))
@@ -408,11 +396,7 @@ StashEventTime (XEvent * ev)
    * old one (in which case the system clock may have changed) */
   if ((NewTimestamp > lastTimestamp) || ((lastTimestamp - NewTimestamp) > 30000))
     lastTimestamp = NewTimestamp;
-  if (FocusOnNextTimeStamp)
-    {
-      SetFocus (FocusOnNextTimeStamp->w, FocusOnNextTimeStamp, 1);
-      FocusOnNextTimeStamp = NULL;
-    }
+
   return True;
 }
 
@@ -485,8 +469,10 @@ GrabEm (int cursor)
 {
   int i = 0, val = 0;
   unsigned int mask;
+  Cursor vs = None;
 
   XSync (dpy, 0);
+  vs = ((cursor >= 0) ? Scr.XfwmCursors[cursor] : None);
   /* move the keyboard focus prior to grabbing the pointer to
    * eliminate the enterNotify and exitNotify events that go
    * to the windows */
@@ -497,8 +483,7 @@ GrabEm (int cursor)
     | EnterWindowMask | LeaveWindowMask;
   while ((i < 1000) && (val = XGrabPointer (dpy, Scr.Root, True, mask,
 				     GrabModeAsync, GrabModeAsync, Scr.Root,
-				    Scr.XfwmCursors[cursor], CurrentTime) !=
-			GrabSuccess))
+				     vs, CurrentTime) != GrabSuccess))
     {
       i++;
       /* If you go too fast, other windows may not get a change to release
@@ -619,12 +604,12 @@ MapIt (XfwmWindow * t)
       if (t->icon_w != None)
 	XMapWindow (dpy, t->icon_w);
     }
-  else if (t->flags & MAPPED)
+  else if (t->flags & (MAPPED | MAP_PENDING))
     {
+      if (!(t->flags & SHADED))
+	XMapWindow (dpy, t->w);
       XMapWindow (dpy, t->frame);
       t->flags |= MAP_PENDING;
-      if (!(t->flags & SHADED))
-	XMapWindow (dpy, t->Parent);
     }
 }
 
@@ -635,6 +620,7 @@ RaiseWindow (XfwmWindow * t)
   int count, i;
   Window *wins;
 
+  MyXGrabServer (dpy);
   /* raise the target, at least */
   count = 1;
   Broadcast (M_RAISE_WINDOW, 3, t->w, t->frame, (unsigned long) t, 0, 0, 0, 0);
@@ -706,7 +692,6 @@ RaiseWindow (XfwmWindow * t)
   if (!(t->flags & ONTOP))
     {
       wins[i++] = t->frame;
-      Scr.LastWindowRaised = t;
     }
 
   if (i > 0)
@@ -716,6 +701,8 @@ RaiseWindow (XfwmWindow * t)
   free (wins);
 
   t->flags |= RAISEDWIN;
+  Scr.LastWindowRaised = t;
+  MyXUngrabServer (dpy);
 }
 
 
