@@ -5,6 +5,8 @@
 #include "menus.h"
 #include "utils.h"
 
+#include "config.h"
+
 /************************************************************************
  * ReapChildren - wait() for all dead child processes
  ************************************************************************/
@@ -165,16 +167,13 @@ extern int MappedNotOverride (Window w);
 extern void GrabButtons (XfwmWindow *);
 extern void GrabKeys (XfwmWindow *);
 extern void GetWindowSizeHints (XfwmWindow *);
-extern void SwitchPages (Bool, Bool);
-extern void NextPage (void);
-extern void PrevPage (void);
-extern void moveLoop (XfwmWindow *, int, int, int, int, int *, int *, Bool, Bool);
+extern Bool moveLoop (XfwmWindow *, int, int, int, int, int *, int *, Bool, Bool);
 
 extern void Keyboard_shortcuts (XEvent *, int);
 extern void RedoIconName (XfwmWindow *);
 extern void DrawIconWindow (XfwmWindow *);
 extern void CreateIconWindow (XfwmWindow * tmp_win, int def_x, int def_y);
-extern inline void RelieveWindow (XfwmWindow *, Window,
+extern void RelieveWindow (XfwmWindow *, Window,
 				  int, int, int, int, GC, GC, int);
 void RelieveParts (XfwmWindow * t, int i, GC hor, GC vert);
 #define NO_HILITE     0x0000
@@ -200,7 +199,6 @@ extern MenuRoot *NewMenuRoot (char *name, int function_or_popup);
 extern void AddToMenu (MenuRoot *, char *, char *);
 extern void MakeMenu (MenuRoot *);
 extern void CaptureAllWindows (void);
-extern void ReGrabKeyWindows (void);
 extern void SetTimer (int);
 extern int flush_expose (Window w);
 void ExecuteFunction (char *Action, XfwmWindow * tmp_win, XEvent * eventp,
@@ -212,6 +210,7 @@ extern int GetContext (XfwmWindow *, XEvent *, Window * dummy);
 extern void ConstrainSize (XfwmWindow *, int *, int *);
 extern void SetShape (XfwmWindow *, int);
 extern void AutoPlace (XfwmWindow *, Bool);
+extern Bool CheckIconPlace (XfwmWindow *);
 void executeModule (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 		    unsigned long context, char *action, int *Module);
 extern void SetFocus (Window, XfwmWindow *, Bool FocusByMouse);
@@ -308,7 +307,7 @@ void CursorColor (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 		  unsigned long context, char *action, int *Module);
 void SetColormapFocus (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 		       unsigned long context, char *action, int *Module);
-inline void DrawPattern (Window, GC, GC, int, int, int);
+void DrawPattern (Window, GC, GC, int, int, int);
 Pixel GetShadow (Pixel);
 Pixel GetHilite (Pixel);
 
@@ -327,7 +326,7 @@ void AddFuncKey (char *, int, int, int, char *, int, int, MenuRoot *,
 		 char, char);
 char *GetNextPtr (char *ptr);
 
-void InteractiveMove (Window * w, XfwmWindow * tmp_win, int *FinalX, int *FinalY,
+Bool InteractiveMove (Window * w, XfwmWindow * tmp_win, int *FinalX, int *FinalY,
 		      XEvent * eventp);
 
 MenuRoot *FindPopup (char *action);
@@ -413,6 +412,14 @@ void SetIconPos (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 		 unsigned long context, char *action, int *Module);
 void Arrange_Icons (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 		    unsigned long context, char *action, int *Module);
+void SetIconGrid (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+		 unsigned long context, char *action, int *Module);
+void SetIconSpacing (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+		 unsigned long context, char *action, int *Module);
+void use_bitmap_buttons (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+		 unsigned long context, char *action, int *Module);
+void show_buttons (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+      	         unsigned long context, char *action, int *Module);
 void DestroyMenu (MenuRoot * mr);
 void GetColors (void);
 XColor GetXColor (char *);
@@ -455,12 +462,6 @@ void SetMenuFont (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 		  unsigned long context, char *action, int *Module);
 void SetTitleStyle (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 		    unsigned long context, char *action, int *Module);
-/*
- * void AddTitleStyle(XEvent *eventp,Window w,XfwmWindow *tmp_win,
- * unsigned long context, char *action,int* Module);
- */
-void SetDeskSize (XEvent * eventp, Window w, XfwmWindow * tmp_win,
-		  unsigned long context, char *action, int *Module);
 void SetXOR (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 	     unsigned long context, char *action, int *Module);
 void SetClick (XEvent * eventp, Window w, XfwmWindow * tmp_win,
@@ -494,6 +495,10 @@ void SnapMove (int *FinalX, int *FinalY, int Width, int Height,
 	       XfwmWindow * skip);
 void SetSnapSize (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 	       unsigned long context, char *action, int *Module);
+void wait_session (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	       unsigned long context, char *action, int *Module);
+void SetSessionMgt (XEvent * eventp, Window w, XfwmWindow * tmp_win,
+	       unsigned long context, char *action, int *Module);
 
 int check_existfile(char *filename);
 /*
