@@ -43,7 +43,7 @@ char **pipeName;
 unsigned long *PipeMask;
 struct queue_buff_struct **pipeQueue;
 
-inline int PositiveWrite (int module, unsigned long *ptr, int size);
+int PositiveWrite (int module, unsigned long *ptr, int size);
 void DeleteQueueBuff (int module);
 void AddToQueue (int module, unsigned long *ptr, int size, int done);
 extern void AddToModList (char *tline);
@@ -573,7 +573,7 @@ SendStrToModule (XEvent * eventp, Window junk, XfwmWindow * tmp_win,
 
 
 
-inline int
+int
 PositiveWrite (int module, unsigned long *ptr, int size)
 {
   if ((pipeOn[module] < 0) || (!((PipeMask[module]) & ptr[1])))
@@ -673,8 +673,6 @@ send_list_func (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   if (*Module >= 0)
     {
       SendPacket (*Module, M_NEW_DESK, 1, Scr.CurrentDesk, 0, 0, 0, 0, 0, 0);
-      SendPacket (*Module, M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-		  Scr.VxMax, Scr.VyMax, 0, 0);
       if (Scr.Hilite != NULL)
 	SendPacket (*Module, M_FOCUS_CHANGE, 5, Scr.Hilite->w, Scr.Hilite->frame,
 		    (unsigned long) Scr.Hilite,
