@@ -314,11 +314,6 @@ PlaceWindow (XfwmWindow * tmp_win, unsigned long tflag, int Desk)
 	    }
 	}
     }
-  /* I think it would be good to switch to the selected desk
-   * whenever a new window pops up, except during initialization */
-  if ((!PPosOverride) && (!(tmp_win->flags & SHOW_ON_MAP)))
-    changeDesks (0, tmp_win->Desk, 1, 1);
-
 
   /* Desk has been selected, now pick a location for the window */
   /*
@@ -483,7 +478,7 @@ GetGravityOffsets (XfwmWindow * tmp, int *xp, int *yp)
     {1, 1},			/* SouthEastGravity */
     {0, 0},			/* StaticGravity */
   };
-  register int g = ((tmp->hints.flags & PWinGravity)
+  int g = ((tmp->hints.flags & PWinGravity)
 		    ? tmp->hints.win_gravity : NorthWestGravity);
 
   if (g < ForgetGravity || g > StaticGravity)
@@ -567,14 +562,12 @@ snapxywh (int *x, int *y, int w, int h, int minx, int miny, int maxx, int maxy,
   if (INTERSECT (*x, *x + w, minx, maxx) && BORNE ((*y + h) - (inside ? maxy : miny)))
   {
     *y = (inside ? maxy : miny) - h;
-    if (*y < 0) *y = 0;
     changed |= CHANGE_VERT;
   }
 
   if (INTERSECT (*x, *x + w, minx, maxx) && BORNE (*y - (inside ? miny : maxy)))
   {
     *y = (inside ? miny : maxy);
-    if (*y < 0) *y = 0;
     changed |= CHANGE_VERT;
   }
 
