@@ -49,11 +49,11 @@ void
 resize_window (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 	       unsigned long context, char *action, int *Module)
 {
-  Bool finished = FALSE, done = FALSE, abort = FALSE;
+  Bool finished = FALSE, done = FALSE, abort = FALSE, window_deleted = FALSE;
   int x, y, mouse_x, mouse_y;
   int delta_x;
   int delta_y;
-  Window ResizeWindow;
+  Window ResizeWindow;  
   extern int Stashed_X, Stashed_Y;
   int val1, val2, val1_unit, val2_unit, n;
   XGCValues gcv;
@@ -73,8 +73,7 @@ resize_window (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 
   if (!(Scr.Options & ResizeOpaqueWin))
     {
-      gcv.line_width = ((tmp_win->boundary_width > 4)
-	  	    ? (tmp_win->boundary_width - 2) : 2);
+      gcv.line_width = 3;
       XChangeGC (dpy, Scr.DrawGC, GCLineWidth, &gcv);
     }
 
@@ -98,8 +97,8 @@ resize_window (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   if (menuFromFrameOrWindowOrTitlebar)
     {
       XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, Stashed_X, Stashed_Y);
-      XFlush (dpy);
     }
+  XFlush (dpy);
 
   if (!GrabEm (MOVE))
     {
@@ -211,15 +210,15 @@ resize_window (XEvent * eventp, Window w, XfwmWindow * tmp_win,
       XNextEvent(dpy, &Event);
       StashEventTime (&Event);
 
-      if (Event.type == MotionNotify) 
-	while(XCheckMaskEvent(dpy, ButtonMotionMask |	ButtonReleaseMask |
-			      PointerMotionMask,&Event))
-	  {
-	    StashEventTime(&Event);      
-	    if (Event.type == ButtonRelease) break;
-	  }
+      if (Event.type == MotionNotify)
+        while (XCheckMaskEvent (dpy, PointerMotionMask | ButtonMotionMask |
+        		        ButtonPressMask | ButtonRelease, &Event))
+         {
+           StashEventTime(&Event);      
+           if (Event.type == ButtonRelease) break;
+         }
 
-      done = FALSE;
+       done = FALSE;
       /* Handle a limited number of key press events to allow mouseless
        * operation */
       if (Event.type == KeyPress)
@@ -254,12 +253,22 @@ resize_window (XEvent * eventp, Window w, XfwmWindow * tmp_win,
 	  done = TRUE;
 	  break;
 	case DestroyNotify:
-	  if (!(Scr.Options & ResizeOpaqueWin)) 
-	    MoveOutline (tmp_win, 0,0,0,0);
-	  finished = TRUE;
-	  done = TRUE;
-	  XPutBackEvent(dpy, &Event);
+          if (Event.xdestroywindow.window == tmp_win->frame)
+            {
+	      finished = TRUE;
+	      done = TRUE;
+              window_deleted = True;
+            }
+          DispatchEvent ();
 	  break;
+        case ConfigureRequest:
+          /* Here we shall override the value_mask to avoid programs such as
+             xmovie to generate a change in their geometry during resize 
+             operation, otherwise xfwm enters an infinite loop */
+          if (Event.xconfigurerequest.window == tmp_win->w)
+            Event.xconfigurerequest.value_mask &= ~(CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
+          DispatchEvent ();
+          break;
 	default:
 	  if (!(Scr.Options & ResizeOpaqueWin)) 
 	    MoveOutline (tmp_win, 0,0,0,0);
@@ -276,23 +285,36 @@ resize_window (XEvent * eventp, Window w, XfwmWindow * tmp_win,
   if (!(Scr.Options & ResizeOpaqueWin))
     MoveOutline (tmp_win, 0, 0, 0, 0);
 
-  if (!abort)
+  if (!window_deleted)
     {
-      ConstrainSize (tmp_win, &dragWidth, &dragHeight);
-      SetupFrame (tmp_win, dragx - tmp_win->bw,
-		  dragy - tmp_win->bw, dragWidth, ((tmp_win->flags & SHADED) ? initHeight : dragHeight), FALSE, TRUE);
+      if (!abort)
+        {
+          ConstrainSize (tmp_win, &dragWidth, &dragHeight);
+          SetupFrame (tmp_win, dragx - tmp_win->bw,
+		      dragy - tmp_win->bw, dragWidth, ((tmp_win->flags & SHADED) ? initHeight : dragHeight), FALSE, TRUE);
+        }
+      else
+          SetupFrame (tmp_win, origx, origy, origWidth, ((tmp_win->flags & SHADED) ? initHeight : origHeight), FALSE, TRUE);
     }
-  else
-      SetupFrame (tmp_win, origx, origy, origWidth, ((tmp_win->flags & SHADED) ? initHeight : origHeight), FALSE, TRUE);
-
   UninstallRootColormap ();
   ResizeWindow = None;
   if (!(Scr.Options & ResizeOpaqueWin))
     MyXUngrabServer (dpy);
   UngrabEm (); 
+  XSync (dpy, 0);
+  /* Flush the event queue and discard all ConfigureRequest relative to resized window */
+  while ((!window_deleted) && My_XNextEvent (dpy, &Event))
+    {
+      if ((Event.type == ConfigureRequest) &&(Event.xconfigurerequest.window == tmp_win->w))
+        Event.xconfigurerequest.value_mask &= ~(CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
+      else if ((Event.type == DestroyNotify) &&(Event.xdestroywindow.window == tmp_win->frame))
+        window_deleted = True;
+      DispatchEvent ();
+    }
+
+  XFlush (dpy);
   xmotion = 0;
   ymotion = 0;
-
   return;
 }
 
@@ -589,9 +611,10 @@ void DisplayPosition(XfwmWindow *tmp_win, int x, int y, int w, int h)
   XDrawString (dpy, Scr.Root, Scr.HintsGC,
 	       x + w + 5, y + (h + Scr.StdFont.height) / 2,
 	       strh, strlen(strh));
-  XDrawString (dpy, Scr.Root, Scr.HintsGC,
-	       x + (w - offsetd) / 2, y + (h + Scr.StdFont.height) / 2,
-	       strd, strlen(strd));
+  if (h > 3 * Scr.StdFont.height)
+    XDrawString (dpy, Scr.Root, Scr.HintsGC,
+	         x + (w - offsetd) / 2, y + (h + Scr.StdFont.height) / 2,
+	         strd, strlen(strd));
   points[0].x = x + 15;
   points[0].y = y + h + 5;
   points[1].x = x;
