@@ -79,6 +79,8 @@ ProcessNewStyle (XEvent * eventp,
 	      restofline += 11;
 	      off_flags |= BW_FLAG;
 	      sscanf (restofline, "%d", &bw);
+              /* Avoid border width = 1, use 0 or >= 2 */
+              if (bw == 1) bw = 2;
 	      while (isspace (*restofline))
 		restofline++;
 	      while ((!isspace (*restofline)) && (*restofline != 0) &&
