@@ -23,6 +23,9 @@
 #include <unistd.h>
 #include <stdarg.h>
 #include <locale.h>
+#include <pwd.h>
+#include <sys/types.h>
+
 #include "xfwm.h"
 #include "menus.h"
 #include "misc.h"
@@ -30,6 +33,7 @@
 #include "parse.h"
 #include "module.h"
 #include "constant.h"
+#include "session.h"
 
 #include <X11/keysym.h>
 #include <X11/Xproto.h>
@@ -48,7 +52,15 @@
 #include <fcntl.h>
 #endif
 
-#include <syslog.h>
+#include "close.xbm"
+#include "iconify.xbm"
+#include "maximize.xbm"
+#include "maximize_pressed.xbm"
+#include "menu.xbm"
+#include "shade.xbm"
+#include "shade_pressed.xbm"
+#include "stick.xbm"
+#include "stick_pressed.xbm"
 
 #define MAXHOSTNAME 255
 
@@ -108,19 +120,38 @@ unsigned int HyperMask;
 char **g_argv;
 int g_argc;
 
-/* assorted gray bitmaps for decorative borders */
-#define g_width 2
-#define g_height 2
-static char g_bits[] =
-{0x02, 0x01};
+Atom _XA_MIT_PRIORITY_COLORS;
+Atom _XA_WM_CHANGE_STATE;
+Atom _XA_WM_STATE;
+Atom _XA_WM_COLORMAP_WINDOWS;
+extern Atom _XA_WM_PROTOCOLS;
+Atom _XA_WM_TAKE_FOCUS;
+Atom _XA_WM_DELETE_WINDOW;
+Atom _XA_WM_DESKTOP;
+Atom _XA_MwmAtom;
+Atom _XA_MOTIF_WM;
 
-#define l_g_width 4
-#define l_g_height 2
-static char l_g_bits[] =
-{0x08, 0x02};
+Atom _XA_OL_WIN_ATTR;
+Atom _XA_OL_WT_BASE;
+Atom _XA_OL_WT_CMD;
+Atom _XA_OL_WT_HELP;
+Atom _XA_OL_WT_NOTICE;
+Atom _XA_OL_WT_OTHER;
+Atom _XA_OL_DECOR_ADD;
+Atom _XA_OL_DECOR_DEL;
+Atom _XA_OL_DECOR_CLOSE;
+Atom _XA_OL_DECOR_RESIZE;
+Atom _XA_OL_DECOR_HEADER;
+Atom _XA_OL_DECOR_ICON_NAME;
+Atom _XA_WIN_WORKSPACE_COUNT;
+Atom _XA_WIN_WORKSPACE;
 
-#define s_g_width 4
-#define s_g_height 4
+Atom _XA_WM_WINDOW_ROLE;
+Atom _XA_WM_CLIENT_LEADER;
+Atom _XA_SM_CLIENT_ID;
+Atom _XA_XFWM_FLAGS;
+Atom _XA_XFWM_ICONPOS_X;
+Atom _XA_XFWM_ICONPOS_Y;
 
 int ShapeEventBase, ShapeErrorBase;
 Boolean ShapesSupported = False;
@@ -130,6 +161,9 @@ extern XEvent Event;
 Bool Restarting = False;
 int fd_width, x_fd;
 char *display_name = NULL;
+char *client_id = NULL;
+
+char restore_filename [MAXSTRLEN];
 
 /***********************************************************************
  *
@@ -138,12 +172,13 @@ char *display_name = NULL;
  *
  ***********************************************************************
  */
+static void InternUsefulAtoms (void);
+static void InitVariables (void);
+
 int
 main (int argc, char **argv)
 {
   XSetWindowAttributes attributes;	/* attributes for create windows */
-  static void InternUsefulAtoms (void);
-  static void InitVariables (void);
   void enterAlarm(int);
   int i;
   extern int x_fd;
@@ -151,7 +186,11 @@ main (int argc, char **argv)
   char *display_string;
   char message[255];
   Bool single = False;
-  Bool option_error = FALSE;
+  Bool option_error = False;
+  Bool custom_filename = False;
+  Bool launch_xfce = True;
+  char *temp_path = NULL;
+  struct passwd *pwd;
     
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, XFCE_LOCALE_DIR);
@@ -160,12 +199,41 @@ main (int argc, char **argv)
   g_argv = argv;
   g_argc = argc;
 
+  temp_path = getenv ("SM_SAVE_DIR");
+  if (!temp_path)
+    {
+      pwd = getpwuid (getuid ());
+      temp_path = pwd->pw_dir;
+    }
+  strcpy (restore_filename, temp_path);
+  strcat (restore_filename, "/.xfwm-session");
+  custom_filename = False;
+
   for (i = 1; i < argc; i++)
     {
       if (mystrncasecmp (argv[i], "-s", 2) == 0)
 	{
 	  single = True;
 	}
+      else if (mystrncasecmp (argv[i], "-noxfce", 7) == 0)
+	{
+	  launch_xfce = False;
+	}
+#ifdef HAVE_SESSION
+      else if (strncasecmp(argv[i], "-clientId", 9) == 0)
+        {
+	  if (++i >= argc)
+	    usage ();
+	  client_id = argv[i];
+        }
+#endif
+      else if (strncasecmp(argv[i], "-restore", 8) == 0)
+        {
+	  if (++i >= argc)
+	    usage();
+	  strcpy (restore_filename, argv[i]);
+          custom_filename = True;          
+        }
       else if (mystrncasecmp (argv[i], "-d", 2) == 0)
 	{
 	  if (++i >= argc)
@@ -220,8 +288,9 @@ main (int argc, char **argv)
 	}
       else if (mystrncasecmp (argv[i], "-version", 8) == 0)
 	{
-	  xfwm_msg (INFO, "main", "Xfwm Version %s compiled on %s at %s\n",
+	  fprintf (stderr, "Xfwm Version %s compiled on %s at %s\n",
 		    VERSION, __DATE__, __TIME__);
+	  exit (0);
 	}
       else
 	{
@@ -257,10 +326,6 @@ main (int argc, char **argv)
   Scr.screen = DefaultScreen (dpy);
   Scr.NumberOfScreens = ScreenCount (dpy);
 
-#ifdef HAVE_IMLIB
-  imlib_id=Imlib_init(dpy);
-#endif
-   
   initModifiers();
   master_pid = getpid ();
 
@@ -292,6 +357,13 @@ main (int argc, char **argv)
 	      dpy = XOpenDisplay (message);
 	      Scr.screen = myscreen;
 	      Scr.NumberOfScreens = ScreenCount (dpy);
+              if ((!custom_filename) && (myscreen != 0))
+                {
+                  char temp[4];
+                  
+                  sprintf (temp, "-%i", (int) Scr.screen);
+                  strcat (restore_filename, temp);
+                }
 
 	      break;
 	    }
@@ -351,6 +423,10 @@ main (int argc, char **argv)
       exit (1);
     }
 
+#ifdef HAVE_IMLIB
+  imlib_id=Imlib_init(dpy);
+#endif
+   
   ShapesSupported = XShapeQueryExtension (dpy, &ShapeEventBase, &ShapeErrorBase);
 
   InternUsefulAtoms ();
@@ -370,7 +446,6 @@ main (int argc, char **argv)
   XSync (dpy, 0);
 
   XSetErrorHandler ((XErrorHandler) XfwmErrorHandler);
-
   BlackoutScreen ();		/* if they want to hide the capture/startup */
 
   CreateCursors ();
@@ -378,9 +453,6 @@ main (int argc, char **argv)
   InitEventHandlerJumpTable ();
   initModules ();
 
-  Scr.gray_bitmap =
-    XCreateBitmapFromData (dpy, Scr.Root, g_bits, g_width, g_height);
-
   XDefineCursor (dpy, Scr.Root, Scr.XfwmCursors[DEFAULT]);
 
   SetRCDefaults ();
@@ -398,18 +470,9 @@ main (int argc, char **argv)
     {
       ExecuteFunction (default_config_command, NULL, &Event, C_ROOT, -1);
     }
-
-  if (Scr.d_depth < 2)
-    {
-      Scr.gray_pixmap =
-	XCreatePixmapFromBitmapData (dpy, Scr.Root, g_bits, g_width, g_height,
-				   Scr.MenuColors.fore, Scr.MenuColors.back,
-				     Scr.d_depth);
-      Scr.light_gray_pixmap =
-	XCreatePixmapFromBitmapData (dpy, Scr.Root, l_g_bits, l_g_width, l_g_height,
-				   Scr.MenuColors.fore, Scr.MenuColors.back,
-				     Scr.d_depth);
-    }
+  
+  if (launch_xfce)
+    ExecuteFunction ("Module xfce", NULL, &Event, C_ROOT, -1);
 
   /* create a window which will accept the keyboard focus when no other 
    * windows have it */
@@ -427,6 +490,9 @@ main (int argc, char **argv)
 
   XSync (dpy, 0);
   UnBlackoutScreen ();		/* if we need to remove blackout window */
+  if (Scr.Options & SessionMgt)
+    LoadWindowStates(restore_filename);
+  SessionInit(client_id);
   HandleEvents ();
   return (0);
 }
@@ -457,51 +523,6 @@ StartupStuff (void)
       if (mr != NULL)
 	ExecuteFunction ("Function InitFunction", NULL, &Event, C_ROOT, -1);
     }
-}				/* StartupStuff */
-
-/***********************************************************************
- *
- *  Procedure:
- *      ReGrabKeyWindows
- *
- * Set up or remove grabs on windows (fourdan@xfce.org)
- *
- ***********************************************************************/
-
-void
-ReGrabKeyWindows (void)
-{
-  XfwmWindow *tmp;
-  int j;
-
-  MyXGrabServer (dpy);
-  for (tmp = Scr.XfwmRoot.next; tmp != NULL; tmp = tmp->next)
-    {
-      if(Scr.Options & (ClickToFocus | AutoRaiseWin))
-	{
-	   for (j = 0; j < 3; j++)
-	     if (Scr.buttons2grab & (1 << j))
-	       {
-		 XGrabButton (dpy, (j + 1), 0, tmp->frame, True,
-			      ButtonPressMask, GrabModeSync, GrabModeAsync, None,
-			      Scr.XfwmCursors[SYS]);
-		 XGrabButton (dpy, (j + 1), AnyModifier, tmp->frame, True,
-			      ButtonPressMask, GrabModeSync, GrabModeAsync, None,
-			      Scr.XfwmCursors[SYS]);
-	       }
-	}
-      else
-        {
-	   for (j = 0; j < 3; j++)
-	     if (Scr.buttons2grab & (1 << j))
-	       {
-		 XUngrabButton (dpy, (j + 1), 0, tmp->frame);
-		 XUngrabButton (dpy, (j + 1), AnyModifier, tmp->frame);
-	       }
-        }
-    }
-  MyXUngrabServer (dpy);
-  XSync (dpy, 0);
 }
 
 /***********************************************************************
@@ -605,6 +626,15 @@ CaptureAllWindows (void)
 	      XChangeProperty (dpy, tmp->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
 			       PropModeReplace, (unsigned char *) data, 1);
 
+              data[0] = (unsigned long) (tmp->flags & (STICKY | ICON_MOVED | SHADED));
+              XChangeProperty (dpy, tmp->w, _XA_XFWM_FLAGS, _XA_XFWM_FLAGS, 32,
+		               PropModeReplace, (unsigned char *) data, 1);
+              data[0] = (unsigned long) tmp->icon_x_loc;
+              XChangeProperty (dpy, tmp->w, _XA_XFWM_ICONPOS_X, _XA_XFWM_ICONPOS_X, 32,
+		               PropModeReplace, (unsigned char *) data, 1);
+              data[0] = (unsigned long) tmp->icon_y_loc;
+              XChangeProperty (dpy, tmp->w, _XA_XFWM_ICONPOS_Y, _XA_XFWM_ICONPOS_Y, 32,
+		               PropModeReplace, (unsigned char *) data, 1);
 	      XSelectInput (dpy, tmp->w, 0);
 	      w = tmp->w;
 	      XUnmapWindow (dpy, tmp->frame);
@@ -644,10 +674,10 @@ SetRCDefaults ()
   {
     "XORValue 0",
     "ModulePath $PATH",
-    "ActiveColor white #A4A4A8",
-    "InactiveColor black #A4A4A8",
-    "TitleStyle Active Solid #C66B78",
-    "TitleStyle Inactive Solid #9198A4",
+    "ActiveColor white #C0C0C0",
+    "InactiveColor black #C0C0C0",
+    "TitleStyle Active Solid #005B80",
+    "TitleStyle Inactive Solid #A4A4A4",
     "CursorColor red white",
     "FocusMode Click",
     "AnimateWin On",
@@ -656,12 +686,15 @@ SetRCDefaults ()
     "AutoRaise Off",
     "AutoRaiseDelay 250",
     "SnapSize 10",
-    "MenuColor  black #A4A4A8 white #8091A4",
+    "SessionManagement On",
+    "WaitSession 10",
+    "MenuColor black #c0c0c0 white #c0c0c0",
     "IconFont fixed",
+    "IconGrid 10",
+    "IconSpacing 5",
     "WindowFont fixed",
     "MenuFont fixed",
-    "DeskTopSize 1 1",
-    "Style \"*\" Title, BorderWidth 7",
+    "Style \"*\" Title, BorderWidth 6",
     "Style \"*\" RandomPlacement, SmartPlacement",
     "Style \"XFce*\" Sticky",
     "Style \"XFbd*\" Sticky",
@@ -714,23 +747,15 @@ SetRCDefaults ()
     "+ \"I\" WindowId $0 WarpToWindow 50 50",
     "AddToFunc \"InitFunction\"",
     "+ \"I\" Desk 0 0",
-    "+ \"I\" Module xfce",
     "AddToFunc \"RestartFunction\"",
     "+ \"I\" Desk 0 0",
-    "+ \"I\" Module xfce",
     "AddToFunc \"__Iconfunc__\"",
-    "+ \"I\" Focus",
-    "+ \"C\" Raise",
     "+ \"M\" Move",
     "+ \"D\" Iconify",
-    "AddToFunc \"__Raisemove__\"",
-    "+ \"C\" Raise",
-    "+ \"M\" Raise",
+    "AddToFunc \"__Move__\"",
     "+ \"M\" Move",
     "+ \"D\" Maximize",
-    "AddToFunc \"__Raiseresize__\"",
-    "+ \"C\" Raise",
-    "+ \"M\" Raise",
+    "AddToFunc \"__Resize__\"",
     "+ \"M\" Resize",
     "AddToFunc \"__builtin_windowops_or_die__\"",
     "+ \"M\" PopUp \"__builtin_window_menu__\"",
@@ -744,24 +769,25 @@ SetRCDefaults ()
     "AddToFunc \"__builtin_iconify_and_focus__"
     "+ \"I\" Iconify",
     "+ \"I\" Current [Iconic] __builtin_Focus_Prev__",
-    "Mouse 1 T A Function __Raisemove__",
-    "Mouse 2 T A Function __Raisemove__",
+    "Mouse 1 T A Function __Move__",
+    "Mouse 2 T A Function __Move__",
     "Mouse 3 T A Shade",
-    "Mouse 0 2 A Maximize",
+    "Mouse 0 1 A __builtin_windowops_or_die__",
+    "Mouse 0 4 A Maximize",
     "Mouse 0 3 A Stick",
-    "Mouse 0 4 A Iconify",
-    "Mouse 0 FS A Function __Raiseresize__",
+    "Mouse 0 6 A Iconify",
+    "Mouse 0 5 A Shade",
+    "Mouse 0 2 A Close",
+    "Mouse 0 2 S Destroy",
+    "Mouse 0 FS A Function __Resize__",
     "Mouse 1 FS S Move",
     "Mouse 2 FS A Move",
     "Mouse 3 FS S Move",
     "Mouse 0 I A Function __Iconfunc__",
-    "Mouse 1 1 A __builtin_windowops_or_die__",
     "Mouse 1 R A Menu __builtin_root_menu__ Nop",
     "Mouse 2 R A Menu __builtin_window_menu__ Nop",
-    "Mouse 2 1 A __builtin_windowops_or_die__",
     "Mouse 3 I A Menu __builtin_window_menu__ Nop",
     "Mouse 3 R A WindowList",
-    "Mouse 3 1 A Close",
     "Key Tab A M Prev [CurrentDesk] __builtin_focus_raise__",
     "Key Tab A SM Next [CurrentDesk] __builtin_focus_raise__",
     "Key Escape A S __builtin_iconify_and_focus__",
@@ -794,7 +820,10 @@ SetRCDefaults ()
     "Key F7 A C Desk 0 6",
     "Key F8 A C Desk 0 7",
     "Key F9 A C Desk 0 8",
-    "Key F10 A  C Desk 0 9",
+    "Key F10 A C Desk 0 9",
+    "Key Left A CM Desk -1",
+    "Key Right A CM Desk 1",
+    "Key Delete A CM exec xflock",
     "Key L5 IW N RaiseLower",
     "Key L5 IW S Lower",
     "Key L5 IW C Raise",
@@ -950,40 +979,6 @@ void initModifiers()
     ButtonKeyMask = KeyMask | ButtonMask;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *	InternUsefulAtoms:
- *            Dont really know what it does
- *
- ***********************************************************************
- */
-Atom _XA_MIT_PRIORITY_COLORS;
-Atom _XA_WM_CHANGE_STATE;
-Atom _XA_WM_STATE;
-Atom _XA_WM_COLORMAP_WINDOWS;
-extern Atom _XA_WM_PROTOCOLS;
-Atom _XA_WM_TAKE_FOCUS;
-Atom _XA_WM_DELETE_WINDOW;
-Atom _XA_WM_DESKTOP;
-Atom _XA_MwmAtom;
-Atom _XA_MOTIF_WM;
-
-Atom _XA_OL_WIN_ATTR;
-Atom _XA_OL_WT_BASE;
-Atom _XA_OL_WT_CMD;
-Atom _XA_OL_WT_HELP;
-Atom _XA_OL_WT_NOTICE;
-Atom _XA_OL_WT_OTHER;
-Atom _XA_OL_DECOR_ADD;
-Atom _XA_OL_DECOR_DEL;
-Atom _XA_OL_DECOR_CLOSE;
-Atom _XA_OL_DECOR_RESIZE;
-Atom _XA_OL_DECOR_HEADER;
-Atom _XA_OL_DECOR_ICON_NAME;
-Atom _XA_WIN_WORKSPACE_COUNT;
-Atom _XA_WIN_WORKSPACE;
-
 static void
 InternUsefulAtoms (void)
 {
@@ -998,6 +993,9 @@ InternUsefulAtoms (void)
   _XA_WM_TAKE_FOCUS = XInternAtom (dpy, "WM_TAKE_FOCUS", False);
   _XA_WM_DELETE_WINDOW = XInternAtom (dpy, "WM_DELETE_WINDOW", False);
   _XA_WM_DESKTOP = XInternAtom (dpy, "WM_DESKTOP", False);
+  _XA_XFWM_FLAGS = XInternAtom (dpy, "XFWM_FLAGS", False);
+  _XA_XFWM_ICONPOS_X = XInternAtom (dpy, "XFWM_ICONPOS_X", False);
+  _XA_XFWM_ICONPOS_Y = XInternAtom (dpy, "XFWM_ICONPOS_Y", False);
   _XA_MwmAtom = XInternAtom (dpy, "_MOTIF_WM_HINTS", False);
   _XA_MOTIF_WM = XInternAtom (dpy, "_MOTIF_WM_INFO", False);
 
@@ -1017,6 +1015,10 @@ InternUsefulAtoms (void)
   _XA_WIN_WORKSPACE_COUNT = XInternAtom (dpy, "_WIN_WORKSPACE_COUNT", False);
   _XA_WIN_WORKSPACE = XInternAtom (dpy, "_WIN_WORKSPACE", False);
 
+  _XA_WM_WINDOW_ROLE=XInternAtom(dpy, "WM_WINDOW_ROLE",False);
+  _XA_WM_CLIENT_LEADER=XInternAtom(dpy, "WM_CLIENT_LEADER",False);
+  _XA_SM_CLIENT_ID=XInternAtom(dpy, "SM_CLIENT_ID",False);
+
   return;
 }
 
@@ -1076,7 +1078,7 @@ on_signal (int sig_num)
 void
 Restart (int nonsense)
 {
-  Done (1, *g_argv);
+  Done (XFWM_RESTART, *g_argv);
   SIGNAL_RETURN;
 }
 
@@ -1093,7 +1095,7 @@ CreateCursors (void)
   /* define cursors */
   Scr.XfwmCursors[POSITION] = XCreateFontCursor (dpy, XC_top_left_corner);
   Scr.XfwmCursors[DEFAULT] = XCreateFontCursor (dpy, XFCE_CURS);
-  Scr.XfwmCursors[SYS] = XCreateFontCursor (dpy, MENU_CURS);
+  Scr.XfwmCursors[SYS] = XCreateFontCursor (dpy, XFCE_CURS);
   Scr.XfwmCursors[TITLE_CURSOR] = XCreateFontCursor (dpy, XFCE_CURS);
   Scr.XfwmCursors[MOVE] = XCreateFontCursor (dpy, XC_fleur);
   Scr.XfwmCursors[MENU] = XCreateFontCursor (dpy, MENU_CURS);
@@ -1120,25 +1122,27 @@ void
 LoadDefaultLeftButton (ButtonFace * bf, int i)
 {
   bf->style = VectorButton;
-  switch (i % 2)
+  switch (i % 3)
     {
     case 0:
       bf->vector.x[0] = 20;
-      bf->vector.y[0] = 40;
-      bf->vector.line_style[0] = 0;
+      bf->vector.y[0] = 42;
+      bf->vector.line_style[0] = 1;
       bf->vector.x[1] = 80;
-      bf->vector.y[1] = 40;
-      bf->vector.line_style[1] = 0;
+      bf->vector.y[1] = 42;
+      bf->vector.line_style[1] = 1;
       bf->vector.x[2] = 80;
-      bf->vector.y[2] = 60;
-      bf->vector.line_style[2] = 1;
+      bf->vector.y[2] = 58;
+      bf->vector.line_style[2] = 0;
       bf->vector.x[3] = 20;
-      bf->vector.y[3] = 60;
-      bf->vector.line_style[3] = 1;
+      bf->vector.y[3] = 58;
+      bf->vector.line_style[3] = 0;
       bf->vector.x[4] = 20;
-      bf->vector.y[4] = 40;
-      bf->vector.line_style[4] = 0;
+      bf->vector.y[4] = 42;
+      bf->vector.line_style[4] = 1;
       bf->vector.num = 5;
+      bf->bitmap = XCreateBitmapFromData(dpy, Scr.Root, menu_bits, menu_width, menu_height);
+      bf->bitmap_pressed = None;
       break;
     case 1:
       bf->vector.x[0] = 30;
@@ -1169,6 +1173,54 @@ LoadDefaultLeftButton (ButtonFace * bf, int i)
       bf->vector.y[8] = 20;
       bf->vector.line_style[8] = 1;
       bf->vector.num = 9;
+      bf->bitmap = XCreateBitmapFromData(dpy, Scr.Root, stick_bits, stick_width, stick_height);
+      bf->bitmap_pressed = XCreateBitmapFromData(dpy, Scr.Root, stick_pressed_bits, stick_pressed_width, stick_pressed_height);
+      break;
+    case 2:
+      bf->vector.x[0] = 20;
+      bf->vector.y[0] = 36;
+      bf->vector.line_style[0] = 1;
+      bf->vector.x[1] = 20;
+      bf->vector.y[1] = 20;
+      bf->vector.line_style[1] = 1;
+      bf->vector.x[2] = 80;
+      bf->vector.y[2] = 20;
+      bf->vector.line_style[2] = 1;
+      bf->vector.x[3] = 80;
+      bf->vector.y[3] = 36;
+      bf->vector.line_style[3] = 0;
+      bf->vector.x[4] = 50;
+      bf->vector.y[4] = 36;
+      bf->vector.line_style[4] = 0;
+      bf->vector.x[5] = 70;
+      bf->vector.y[5] = 56;
+      bf->vector.line_style[5] = 0;
+      bf->vector.x[6] = 58;
+      bf->vector.y[6] = 56;
+      bf->vector.line_style[6] = 0;
+      bf->vector.x[7] = 58;
+      bf->vector.y[7] = 80;
+      bf->vector.line_style[7] = 0;
+      bf->vector.x[8] = 42;
+      bf->vector.y[8] = 80;
+      bf->vector.line_style[8] = 0;
+      bf->vector.x[9] = 42;
+      bf->vector.y[9] = 56;
+      bf->vector.line_style[9] = 1;
+      bf->vector.x[10] = 30;
+      bf->vector.y[10] = 56;
+      bf->vector.line_style[10] = 0;
+      bf->vector.x[11] = 50;
+      bf->vector.y[11] = 36;
+      bf->vector.line_style[11] = 1;
+      bf->vector.x[12] = 20;
+      bf->vector.y[12] = 36;
+      bf->vector.line_style[12] = 0;
+      bf->vector.num = 13;
+      bf->bitmap = XCreateBitmapFromData(dpy, Scr.Root, shade_bits, shade_width, shade_height);
+      bf->bitmap_pressed = XCreateBitmapFromData(dpy, Scr.Root, shade_pressed_bits, shade_pressed_width, shade_pressed_height);
+      break;
+    default:
       break;
     }
 }
@@ -1183,9 +1235,53 @@ void
 LoadDefaultRightButton (ButtonFace * bf, int i)
 {
   bf->style = VectorButton;
-  switch (i % 2)
+  switch (i % 3)
     {
     case 0:
+      bf->vector.x[0] = 15;
+      bf->vector.y[0] = 20;
+      bf->vector.line_style[0] = 1;
+      bf->vector.x[1] = 40;
+      bf->vector.y[1] = 20;
+      bf->vector.line_style[1] = 1;
+      bf->vector.x[2] = 50;
+      bf->vector.y[2] = 40;
+      bf->vector.line_style[2] = 0;
+      bf->vector.x[3] = 60;
+      bf->vector.y[3] = 20;
+      bf->vector.line_style[3] = 1;
+      bf->vector.x[4] = 85;
+      bf->vector.y[4] = 20;
+      bf->vector.line_style[4] = 1;
+      bf->vector.x[5] = 65;
+      bf->vector.y[5] = 50;
+      bf->vector.line_style[5] = 0;
+      bf->vector.x[6] = 85;
+      bf->vector.y[6] = 80;
+      bf->vector.line_style[6] = 0;
+      bf->vector.x[7] = 60;
+      bf->vector.y[7] = 80;
+      bf->vector.line_style[7] = 0;
+      bf->vector.x[8] = 50;
+      bf->vector.y[8] = 60;
+      bf->vector.line_style[8] = 1;
+      bf->vector.x[9] = 40;
+      bf->vector.y[9] = 80;
+      bf->vector.line_style[9] = 0;
+      bf->vector.x[10] = 15;
+      bf->vector.y[10] = 80;
+      bf->vector.line_style[10] = 0;
+      bf->vector.x[11] = 35;
+      bf->vector.y[11] = 50;
+      bf->vector.line_style[11] = 1;
+      bf->vector.x[12] = 15;
+      bf->vector.y[12] = 20;
+      bf->vector.line_style[12] = 1;
+      bf->vector.num = 13;
+      bf->bitmap = XCreateBitmapFromData(dpy, Scr.Root, close_bits, close_width, close_height);
+      bf->bitmap_pressed = None;
+      break;
+    case 1:
       bf->vector.x[0] = 20;
       bf->vector.y[0] = 20;
       bf->vector.line_style[0] = 1;
@@ -1202,24 +1298,30 @@ LoadDefaultRightButton (ButtonFace * bf, int i)
       bf->vector.y[4] = 20;
       bf->vector.line_style[4] = 1;
       bf->vector.num = 5;
+      bf->bitmap = XCreateBitmapFromData(dpy, Scr.Root, maximize_bits, maximize_width, maximize_height);
+      bf->bitmap_pressed = XCreateBitmapFromData(dpy, Scr.Root, maximize_pressed_bits, maximize_pressed_width, maximize_pressed_height);
       break;
-    case 1:
-      bf->vector.x[0] = 39;
-      bf->vector.y[0] = 39;
-      bf->vector.line_style[0] = 0;
-      bf->vector.x[1] = 61;
-      bf->vector.y[1] = 39;
-      bf->vector.line_style[1] = 0;
-      bf->vector.x[2] = 61;
-      bf->vector.y[2] = 61;
-      bf->vector.line_style[2] = 1;
-      bf->vector.x[3] = 39;
-      bf->vector.y[3] = 61;
-      bf->vector.line_style[3] = 1;
-      bf->vector.x[4] = 39;
-      bf->vector.y[4] = 39;
-      bf->vector.line_style[4] = 0;
+    case 2:
+      bf->vector.x[0] = 42;
+      bf->vector.y[0] = 42;
+      bf->vector.line_style[0] = 1;
+      bf->vector.x[1] = 58;
+      bf->vector.y[1] = 42;
+      bf->vector.line_style[1] = 1;
+      bf->vector.x[2] = 58;
+      bf->vector.y[2] = 58;
+      bf->vector.line_style[2] = 0;
+      bf->vector.x[3] = 42;
+      bf->vector.y[3] = 58;
+      bf->vector.line_style[3] = 0;
+      bf->vector.x[4] = 42;
+      bf->vector.y[4] = 42;
+      bf->vector.line_style[4] = 1;
       bf->vector.num = 5;
+      bf->bitmap = XCreateBitmapFromData(dpy, Scr.Root, iconify_bits, iconify_width, iconify_height);
+      bf->bitmap_pressed = None;
+      break;
+    default:
       break;
     }
 }
@@ -1233,15 +1335,12 @@ LoadDefaultRightButton (ButtonFace * bf, int i)
 void
 LoadDefaultButton (ButtonFace * bf, int i)
 {
-  int n = i / 2;
-  if ((n * 2) == i)
-    {
-      if (--n < 0)
-	n = 2;
-      LoadDefaultRightButton (bf, n);
-    }
+  bf->bitmap = None;
+  bf->bitmap_pressed = None;
+  if (i & 1)
+    LoadDefaultRightButton(bf, i / 2);
   else
-    LoadDefaultLeftButton (bf, n);
+    LoadDefaultLeftButton(bf, i / 2);
 }
 
 extern void FreeButtonFace (Display * dpy, ButtonFace * bf);
@@ -1256,7 +1355,7 @@ void
 ResetAllButtons (XfwmDecor * fl)
 {
   int i = 0;
-  for (; i < 2; ++i)
+  for (; i < 3; ++i)
     {
       int j;
 
@@ -1276,9 +1375,12 @@ ResetAllButtons (XfwmDecor * fl)
 	}
     }
   fl->left_buttons[0].flags |= MWMDecorMenu;
-  fl->left_buttons[1].flags |= MWMDecorSticky;
-  fl->right_buttons[1].flags |= MWMDecorMinimize;
-  fl->right_buttons[0].flags |= MWMDecorMaximize;
+  fl->left_buttons[1].flags |= DecorSticky;
+  fl->left_buttons[2].flags |= DecorShaded;
+  /* Treat "close" button similar to menu button for Motif hints*/
+  fl->right_buttons[0].flags |= MWMDecorMenu; 
+  fl->right_buttons[1].flags |= MWMDecorMaximize;
+  fl->right_buttons[2].flags |= MWMDecorMinimize;
 }
 
 /***********************************************************************
@@ -1291,8 +1393,6 @@ void
 DestroyXfwmDecor (XfwmDecor * fl)
 {
   int i;
-  /* reset to default button set (frees allocated mem) */
-  ResetAllButtons (fl);
   for (i = 0; i < 3; ++i)
     {
       int j = 0;
@@ -1321,6 +1421,16 @@ DestroyXfwmDecor (XfwmDecor * fl)
       XFreeGC (dpy, fl->HiShadowGC);
       fl->HiShadowGC = NULL;
     }
+  if (fl->HiBackGC != NULL)
+    {
+      XFreeGC (dpy, fl->HiBackGC);
+      fl->HiBackGC = NULL;
+    }
+  if (fl->LoBackGC != NULL)
+    {
+      XFreeGC (dpy, fl->LoBackGC);
+      fl->LoBackGC = NULL;
+    }
 }
 
 /***********************************************************************
@@ -1338,9 +1448,13 @@ InitXfwmDecor (XfwmDecor * fl)
   fl->HiShadowGC = NULL;
   fl->LoReliefGC = NULL;
   fl->LoShadowGC = NULL;
+  fl->LoBackGC = NULL;
+  fl->LoBackGC = NULL;
 
   /* initialize title-bar button styles */
   tmpbf.style = SimpleButton;
+  tmpbf.bitmap = None;
+  tmpbf.bitmap_pressed = None;
   tmpbf.next = NULL;
   for (i = 0; i < 2; ++i)
     {
@@ -1352,8 +1466,6 @@ InitXfwmDecor (XfwmDecor * fl)
 	}
     }
 
-  /* reset to default button set */
-  ResetAllButtons (fl);
 
   /* initialize title-bar styles */
   fl->titlebar.flags = 0;
@@ -1362,13 +1474,21 @@ InitXfwmDecor (XfwmDecor * fl)
     {
       fl->titlebar.state[i].style = SimpleButton;
       fl->titlebar.state[i].next = NULL;
+      fl->titlebar.state[i].bitmap = None;
+      fl->titlebar.state[i].bitmap_pressed = None;
     }
 
   /* initialize border texture styles */
   fl->BorderStyle.active.style = SimpleButton;
   fl->BorderStyle.inactive.style = SimpleButton;
+  fl->BorderStyle.active.bitmap = None;
+  fl->BorderStyle.active.bitmap_pressed = None;
+  fl->BorderStyle.inactive.bitmap = None;
+  fl->BorderStyle.inactive.bitmap_pressed = None;
   fl->BorderStyle.active.next = NULL;
   fl->BorderStyle.inactive.next = NULL;
+  /* reset to default button set */
+  ResetAllButtons (fl);
 }
 
 /***********************************************************************
@@ -1390,11 +1510,6 @@ InitVariables (void)
   Scr.TheList = NULL;
 
   Scr.DefaultIcon = NULL;
-  Scr.gray_bitmap = (Pixmap) NULL; 
-  Scr.gray_pixmap = (Pixmap) NULL; 
-  Scr.light_gray_pixmap = (Pixmap) NULL;
-  Scr.sticky_gray_pixmap = (Pixmap) NULL;
-  
   Scr.TransMaskGC = (GC) NULL;
   Scr.DrawGC = (GC) NULL;  
   Scr.MenuGC = (GC) NULL;
@@ -1426,8 +1541,7 @@ InitVariables (void)
   Scr.CornerWidth = CORNER_WIDTH;
   Scr.Hilite = NULL;
   Scr.Focus = NULL;
-  Scr.Ungrabbed = NULL;
-  Scr.Options = (ClickToFocus | AnimateWin);
+  Scr.Options = (ClickToFocus | AnimateWin | BitmapButtons);
   Scr.AutoRaiseDelay = 200;
   Scr.iconbox = 0;
   Scr.StdFont.font = NULL;
@@ -1436,11 +1550,9 @@ InitVariables (void)
   Scr.WindowFont.fontset = NULL;
   Scr.IconFont.font = NULL;
   Scr.IconFont.fontset = NULL;
-
-  Scr.ndesks = 4;
-  Scr.VxMax = 2 * Scr.MyDisplayWidth;
-  Scr.VyMax = 2 * Scr.MyDisplayHeight;
-  Scr.Vx = Scr.Vy = 0;
+  Scr.icongrid = 10;
+  Scr.iconspacing = 5;
+  Scr.ndesks = 1;
   {
     Atom atype;
     int aformat;
@@ -1468,7 +1580,6 @@ InitVariables (void)
 	    Restarting = True;
 	    Scr.CurrentDesk = *(unsigned long *) prop;
                         	if (Scr.CurrentDesk > 10) Scr.CurrentDesk = 1;
-            syslog (LOG_DEBUG, __FUNCTION__">_WIN_WORKSPACE=%i\n", Scr.CurrentDesk); 
 	  }
       }
     if ((XGetWindowProperty (dpy, Scr.Root, _XA_WIN_WORKSPACE_COUNT, 0L, 1L, False,
@@ -1479,25 +1590,24 @@ InitVariables (void)
 	  {
 	    Restarting = True;
 	    Scr.ndesks = *(unsigned long *) prop;
-                        	if (Scr.ndesks > 10) Scr.ndesks = 10;
-            syslog (LOG_DEBUG, __FUNCTION__">_WIN_WORKSPACE_COUNT=%i\n", Scr.ndesks); 
+            if (Scr.ndesks > 10) Scr.ndesks = 10;
 	  }
       }
   }
 
   Scr.EdgeScrollX = Scr.EdgeScrollY = 0;
-  Scr.ClickTime = 150;
+  Scr.ClickTime = 300;
   Scr.ColormapFocus = COLORMAP_FOLLOWS_MOUSE;
 
   /* set major operating modes */
   Scr.NumBoxes = 0;
 
   Scr.randomx = Scr.randomy = 0;
-  Scr.buttons2grab = 7;
 
   InitXfwmDecor (&Scr.DefaultDecor);
 
   Scr.SnapSize = 10;
+  Scr.sessionwait = 10;
   
   /* Not the right place for this, should only be called once somewhere .. */
   InitPictureCMap (dpy, Scr.Root);
@@ -1543,7 +1653,7 @@ Reborder (void)
 void
 SigDone (int nonsense)
 {
-  Done (0, NULL);
+  Done (XFWM_HALT, NULL);
   SIGNAL_RETURN;
 }
 
@@ -1556,6 +1666,12 @@ Done (int restart, char *command)
   if (mr != NULL)
     ExecuteFunction ("Function ExitFunction", NULL, &Event, C_ROOT, -1);
 
+  if (!restart)
+    {
+      /* LogoutICEConn(); */
+      CloseICEConn();
+    }
+
   /* Close all my pipes */
   ClosePipes ();
 
@@ -1565,11 +1681,6 @@ Done (int restart, char *command)
 
   if (ModulePath) free(ModulePath);
 
-  if (Scr.gray_bitmap)		XFreePixmap(dpy, Scr.gray_bitmap); 
-  if (Scr.gray_pixmap)		XFreePixmap(dpy, Scr.gray_pixmap);
-  if (Scr.light_gray_pixmap)	XFreePixmap(dpy, Scr.light_gray_pixmap);
-  if (Scr.sticky_gray_pixmap)	XFreePixmap(dpy, Scr.sticky_gray_pixmap);
-
   if (Scr.TransMaskGC) 		XFreeGC(dpy, Scr.TransMaskGC);
   if (Scr.DrawGC) 		XFreeGC(dpy, Scr.DrawGC);  
   if (Scr.MenuGC) 		XFreeGC(dpy, Scr.MenuGC);
@@ -1595,6 +1706,14 @@ Done (int restart, char *command)
   if (Scr.WindowFont.font) XFreeFont(dpy, Scr.WindowFont.font);
   if (Scr.WindowFont.fontset) XFreeFontSet(dpy, Scr.WindowFont.fontset);
 
+  /* Take time to save the current session (if ICE is not available) */
+
+  if (!(restart == XFWM_END) && (Scr.Options & SessionMgt))
+    builtin_save_session ();
+
+  /* Free up structure allocated by session management */
+  FreeWindowStates();
+
   DestroyXfwmDecor (&Scr.DefaultDecor);
 
   if (restart)
@@ -1681,6 +1800,7 @@ static XErrorHandler
 XfwmErrorHandler (Display * dpy, XErrorEvent * event)
 {
   extern int last_event_type;
+  char buf[64];
 
   /* some errors are acceptable, mostly they're caused by 
    * trying to update a lost  window */
@@ -1692,7 +1812,9 @@ XfwmErrorHandler (Display * dpy, XErrorEvent * event)
     return 0;
 
 
+  XGetErrorText (dpy, event->error_code, buf, 63);
   xfwm_msg (ERR, "XfwmErrorHandler", "*** internal error ***");
+  xfwm_msg (ERR, "XfwmErrorHandler", "%s", buf);
   xfwm_msg (ERR, "XfwmErrorHandler", "Request %d, Error %d, EventType: %d",
 	    event->request_code,
 	    event->error_code,
@@ -1704,7 +1826,18 @@ void
 usage (void)
 {
   fprintf (stderr, "\nXfwm Version %s Usage:\n\n", VERSION);
-  fprintf (stderr, "  %s [-d dpy] [-f config_cmd] [-s] [-blackout] [-version] [-h]\n\n", g_argv[0]);
+  fprintf (stderr, "  %s [-d dpy] [-f config_cmd] [-s] [-blackout] [-version] [-h]\n"
+                   "  [ -noxfce ] [ -sync ] [ -clientId n] [ -restore ]\n\n", g_argv[0]);
+  fprintf (stderr, "  -blackout Blackout screen at startup\n");
+  fprintf (stderr, "  -cmd      Specify the command to run as startup\n");
+  fprintf (stderr, "  -d        Specify the display to connect\n");
+  fprintf (stderr, "  -f        Specify the configuration file to load\n");
+  fprintf (stderr, "  -h        Display this text\n");
+  fprintf (stderr, "  -noxfce   Do not start xfce panel automatically\n");
+  fprintf (stderr, "  -s        Forces xfwm to run on a single screen\n");
+  fprintf (stderr, "            (when multi-screen available)\n");
+  fprintf (stderr, "  -sync     Run in synchronized mode\n");
+  fprintf (stderr, "  -version  Dispay xfwm version and exit\n\n");
 }
 
 /****************************************************************************
@@ -1723,6 +1856,15 @@ SaveDesktopState ()
       data[0] = (unsigned long) t->Desk;
       XChangeProperty (dpy, t->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
 		       PropModeReplace, (unsigned char *) data, 1);
+      data[0] = (unsigned long) (t->flags & (STICKY | ICONIFIED| ICON_MOVED | SHADED | WM_NAME_CHANGED));
+      XChangeProperty (dpy, t->w, _XA_XFWM_FLAGS, _XA_XFWM_FLAGS, 32,
+		       PropModeReplace, (unsigned char *) data, 1);
+      data[0] = (unsigned long) t->icon_x_loc;
+      XChangeProperty (dpy, t->w, _XA_XFWM_ICONPOS_X, _XA_XFWM_ICONPOS_X, 32,
+		       PropModeReplace, (unsigned char *) data, 1);
+      data[0] = (unsigned long) t->icon_y_loc;
+      XChangeProperty (dpy, t->w, _XA_XFWM_ICONPOS_Y, _XA_XFWM_ICONPOS_Y, 32,
+		       PropModeReplace, (unsigned char *) data, 1);
     }
 
   data[0] = (unsigned long) Scr.CurrentDesk;
@@ -1736,6 +1878,17 @@ SaveDesktopState ()
 static void
 SetMWM_INFO (Window window)
 {
+  struct mwminfo
+  {
+    long flags;
+    Window win;
+  }  motif_wm_info;
+
+  motif_wm_info.flags     = 2;
+  motif_wm_info.win = window;
+
+  XChangeProperty(dpy,Scr.Root,_XA_MOTIF_WM,_XA_MOTIF_WM,32,
+		  PropModeReplace,(unsigned char *)&motif_wm_info,2);
 }
 
 void
